@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2013-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1032 $
- * $Date: 2014-05-29 23:38:27 +0200 (Thu, 29 May 2014) $
+ * $Revision: 1086 $
+ * $Date: 2014-08-11 00:14:03 +0200 (Mon, 11 Aug 2014) $
  *
  */
 
@@ -330,7 +330,7 @@ bool FeedFilter::Term::Compile(char* szToken)
 }
 
 /*
- * If pFeedItemInfo is NULL, only field type info is returned
+ * If pFeedItemInfo is NULL, only field name is validated
  */
 bool FeedFilter::Term::GetFieldData(const char* szField, FeedItemInfo* pFeedItemInfo,
 	const char** StrValue, long long* IntValue)
@@ -408,6 +408,11 @@ bool FeedFilter::Term::GetFieldData(const char* szField, FeedItemInfo* pFeedItem
 		*IntValue = pFeedItemInfo ? pFeedItemInfo->GetDupeScore() : 0;
 		return true;
 	}
+	else if (!strcasecmp(szField, "dupestatus"))
+	{
+		*StrValue = pFeedItemInfo ? pFeedItemInfo->GetDupeStatus() : NULL;
+		return true;
+	}
 	else if (!strncasecmp(szField, "attr-", 5))
 	{
 		if (pFeedItemInfo)
