@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2013-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  * $Revision: 0 $
- * $Date: 2014-02-24 23:11:14 +0100 (Mon, 24 Feb 2014) $
+ * $Date: 2014-08-11 00:14:03 +0200 (Mon, 11 Aug 2014) $
  *
  */
 
@@ -165,6 +165,7 @@ private:
 	char*				m_szDupeKey;
 	int					m_iDupeScore;
 	EDupeMode			m_eDupeMode;
+	char*				m_szDupeStatus;
 	SharedFeedData*		m_pSharedFeedData;
 	Attributes			m_Attributes;
 
@@ -219,6 +220,7 @@ public:
 	void				SetDupeScore(int iDupeScore) { m_iDupeScore = iDupeScore; }
 	EDupeMode			GetDupeMode() { return m_eDupeMode; }
 	void				SetDupeMode(EDupeMode eDupeMode) { m_eDupeMode = eDupeMode; }
+	const char*			GetDupeStatus();
 	Attributes*			GetAttributes() { return &m_Attributes; }
 };
 
