@@ -18,14 +18,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1043 $
- * $Date: 2014-06-13 23:53:27 +0200 (Fri, 13 Jun 2014) $
+ * $Revision: 1151 $
+ * $Date: 2014-11-01 12:00:40 +0100 (Sat, 01 Nov 2014) $
  *
  */
 
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "config.h"					  
 #endif
 
 #ifdef WIN32
@@ -107,8 +107,9 @@ static const char* OPTION_NZBDIR				= "NzbDir";
 static const char* OPTION_WEBDIR				= "WebDir";
 static const char* OPTION_CONFIGTEMPLATE		= "ConfigTemplate";
 static const char* OPTION_SCRIPTDIR				= "ScriptDir";
-static const char* OPTION_CREATELOG				= "CreateLog";
 static const char* OPTION_LOGFILE				= "LogFile";
+static const char* OPTION_WRITELOG				= "WriteLog";
+static const char* OPTION_ROTATELOG				= "RotateLog";
 static const char* OPTION_APPENDCATEGORYDIR		= "AppendCategoryDir";
 static const char* OPTION_LOCKFILE				= "LockFile";
 static const char* OPTION_DAEMONUSERNAME		= "DaemonUsername";
@@ -124,11 +125,11 @@ static const char* OPTION_SECUREPORT			= "SecurePort";
 static const char* OPTION_SECURECERT			= "SecureCert";
 static const char* OPTION_SECUREKEY				= "SecureKey";
 static const char* OPTION_AUTHORIZEDIP			= "AuthorizedIP";
-static const char* OPTION_CONNECTIONTIMEOUT		= "ConnectionTimeout";
+static const char* OPTION_ARTICLETIMEOUT		= "ArticleTimeout";
+static const char* OPTION_URLTIMEOUT			= "UrlTimeout";
 static const char* OPTION_SAVEQUEUE				= "SaveQueue";
 static const char* OPTION_RELOADQUEUE			= "ReloadQueue";
 static const char* OPTION_CREATEBROKENLOG		= "CreateBrokenLog";
-static const char* OPTION_RESETLOG				= "ResetLog";
 static const char* OPTION_DECODE				= "Decode";
 static const char* OPTION_RETRIES				= "Retries";
 static const char* OPTION_RETRYINTERVAL			= "RetryInterval";
@@ -144,7 +145,10 @@ static const char* OPTION_DETAILTARGET			= "DetailTarget";
 static const char* OPTION_PARCHECK				= "ParCheck";
 static const char* OPTION_PARREPAIR				= "ParRepair";
 static const char* OPTION_PARSCAN				= "ParScan";
+static const char* OPTION_PARQUICK				= "ParQuick";
 static const char* OPTION_PARRENAME				= "ParRename";
+static const char* OPTION_PARBUFFER				= "ParBuffer";
+static const char* OPTION_PARTHREADS			= "ParThreads";
 static const char* OPTION_HEALTHCHECK			= "HealthCheck";
 static const char* OPTION_SCANSCRIPT			= "ScanScript";
 static const char* OPTION_QUEUESCRIPT			= "QueueScript";
@@ -155,7 +159,7 @@ static const char* OPTION_CURSESTIME			= "CursesTime";
 static const char* OPTION_CURSESGROUP			= "CursesGroup";
 static const char* OPTION_CRCCHECK				= "CrcCheck";
 static const char* OPTION_DIRECTWRITE			= "DirectWrite";
-static const char* OPTION_WRITEBUFFERSIZE		= "WriteBufferSize";
+static const char* OPTION_WRITEBUFFER			= "WriteBuffer";
 static const char* OPTION_NZBDIRINTERVAL		= "NzbDirInterval";
 static const char* OPTION_NZBDIRFILEAGE			= "NzbDirFileAge";
 static const char* OPTION_PARCLEANUPQUEUE		= "ParCleanupQueue";
@@ -181,6 +185,8 @@ static const char* OPTION_FEEDHISTORY			= "FeedHistory";
 static const char* OPTION_URLFORCE				= "UrlForce";
 static const char* OPTION_TIMECORRECTION		= "TimeCorrection";
 static const char* OPTION_PROPAGATIONDELAY		= "PropagationDelay";
+static const char* OPTION_ARTICLECACHE			= "ArticleCache";
+static const char* OPTION_EVENTINTERVAL			= "EventInterval";
 
 // obsolete options
 static const char* OPTION_POSTLOGKIND			= "PostLogKind";
@@ -199,6 +205,8 @@ static const char* OPTION_RELOADURLQUEUE		= "ReloadUrlQueue";
 static const char* OPTION_RELOADPOSTQUEUE		= "ReloadPostQueue";
 static const char* OPTION_NZBPROCESS			= "NZBProcess";
 static const char* OPTION_NZBADDEDPROCESS		= "NZBAddedProcess";
+static const char* OPTION_CREATELOG				= "CreateLog";
+static const char* OPTION_RESETLOG				= "ResetLog";
 
 const char* BoolNames[] = { "yes", "no", "true", "false", "1", "0", "on", "off", "enable", "disable", "enabled", "disabled" };
 const int BoolValues[] = { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0 };
@@ -210,6 +218,7 @@ static const char* SCAN_SCRIPT_SIGNATURE = "SCAN";
 static const char* QUEUE_SCRIPT_SIGNATURE = "QUEUE";
 static const char* SCHEDULER_SCRIPT_SIGNATURE = "SCHEDULER";
 static const char* END_SCRIPT_SIGNATURE = " SCRIPT";
+static const char* QUEUE_EVENTS_SIGNATURE = "### QUEUE EVENTS:";
 
 #ifndef WIN32
 const char* PossibleConfigLocations[] =
@@ -386,6 +395,7 @@ Options::Script::Script(const char* szName, const char* szLocation)
 	m_bScanScript = false;
 	m_bQueueScript = false;
 	m_bSchedulerScript = false;
+	m_szQueueEvents = NULL;
 }
 
 Options::Script::~Script()
@@ -393,6 +403,7 @@ Options::Script::~Script()
 	free(m_szName);
 	free(m_szLocation);
 	free(m_szDisplayName);
+	free(m_szQueueEvents);
 }
 
 void Options::Script::SetDisplayName(const char* szDisplayName)
@@ -401,16 +412,28 @@ void Options::Script::SetDisplayName(const char* szDisplayName)
 	m_szDisplayName = strdup(szDisplayName);
 }
 
+void Options::Script::SetQueueEvents(const char* szQueueEvents)
+{
+	free(m_szQueueEvents);
+	m_szQueueEvents = szQueueEvents ? strdup(szQueueEvents) : NULL;
+}
 
-Options::ScriptList::~ScriptList()
+
+Options::Scripts::~Scripts()
+{
+	Clear();
+}
+
+void Options::Scripts::Clear()
 {
 	for (iterator it = begin(); it != end(); it++)
 	{
 		delete *it;
 	}
+	clear();
 }
 
-Options::Script* Options::ScriptList::Find(const char* szName)
+Options::Script* Options::Scripts::Find(const char* szName)
 {
 	for (iterator it = begin(); it != end(); it++)
 	{
@@ -452,7 +475,6 @@ Options::Options(int argc, char* argv[])
 	m_bPauseScan			= false;
 	m_bTempPauseDownload	= false;
 	m_bCreateBrokenLog		= false;
-	m_bResetLog				= false;
 	m_iDownloadRate			= 0;
 	m_iEditQueueAction		= 0;
 	m_pEditQueueIDList		= NULL;
@@ -465,7 +487,8 @@ Options::Options(int argc, char* argv[])
 	m_iAddPriority			= 0;
 	m_szAddNZBFilename		= NULL;
 	m_bAddPaused			= false;
-	m_iConnectionTimeout	= 0;
+	m_iArticleTimeout		= 0;
+	m_iUrlTimeout			= 0;
 	m_iTerminateTimeout		= 0;
 	m_bServerMode			= false;
 	m_bDaemonMode			= false;
@@ -495,12 +518,16 @@ Options::Options(int argc, char* argv[])
 	m_iLogBufferSize		= 0;
 	m_iLogLines				= 0;
 	m_iWriteLogKind			= 0;
-	m_bCreateLog			= false;
+	m_eWriteLog				= wlAppend;
+	m_iRotateLog			= 0;
 	m_szLogFile				= NULL;
 	m_eParCheck				= pcManual;
 	m_bParRepair			= false;
 	m_eParScan				= psLimited;
+	m_bParQuick				= true;
 	m_bParRename			= false;
+	m_iParBuffer			= 0;
+	m_iParThreads			= 0;
 	m_eHealthCheck			= hcNone;
 	m_szScriptOrder			= NULL;
 	m_szPostScript			= NULL;
@@ -514,7 +541,7 @@ Options::Options(int argc, char* argv[])
 	m_bCursesGroup			= false;
 	m_bCrcCheck				= false;
 	m_bDirectWrite			= false;
-	m_iWriteBufferSize		= 0;
+	m_iWriteBuffer			= 0;
 	m_iNzbDirInterval		= 0;
 	m_iNzbDirFileAge		= 0;
 	m_bParCleanupQueue		= false;
@@ -543,6 +570,8 @@ Options::Options(int argc, char* argv[])
 	m_iTimeCorrection		= 0;
 	m_iLocalTimeOffset		= 0;
 	m_iPropagationDelay		= 0;
+	m_iArticleCache			= 0;
+	m_iEventInterval		= 0;
 
 	// Option "ConfigFile" will be initialized later, but we want
 	// to see it at the top of option list, so we add it first
@@ -605,6 +634,8 @@ Options::Options(int argc, char* argv[])
 	InitCategories();
 	InitScheduler();
 	InitFeeds();
+	InitScripts();
+	InitConfigTemplates();
 
 	if (m_bPrintOptions)
 	{
@@ -732,7 +763,8 @@ void Options::InitDefault()
 	SetOption(OPTION_WEBDIR, "");
 	SetOption(OPTION_CONFIGTEMPLATE, "");
 	SetOption(OPTION_SCRIPTDIR, "${MainDir}/scripts");
-	SetOption(OPTION_CREATELOG, "yes");
+	SetOption(OPTION_WRITELOG, "append");
+	SetOption(OPTION_ROTATELOG, "3");
 	SetOption(OPTION_APPENDCATEGORYDIR, "yes");
 	SetOption(OPTION_OUTPUTMODE, "curses");
 	SetOption(OPTION_DUPECHECK, "yes");
@@ -746,11 +778,11 @@ void Options::InitDefault()
 	SetOption(OPTION_SECURECERT, "");
 	SetOption(OPTION_SECUREKEY, "");
 	SetOption(OPTION_AUTHORIZEDIP, "");
-	SetOption(OPTION_CONNECTIONTIMEOUT, "60");
+	SetOption(OPTION_ARTICLETIMEOUT, "60");
+	SetOption(OPTION_URLTIMEOUT, "60");
 	SetOption(OPTION_SAVEQUEUE, "yes");
 	SetOption(OPTION_RELOADQUEUE, "yes");
 	SetOption(OPTION_CREATEBROKENLOG, "yes");
-	SetOption(OPTION_RESETLOG, "no");
 	SetOption(OPTION_DECODE, "yes");
 	SetOption(OPTION_RETRIES, "3");
 	SetOption(OPTION_RETRYINTERVAL, "10");
@@ -766,7 +798,10 @@ void Options::InitDefault()
 	SetOption(OPTION_PARCHECK, "auto");
 	SetOption(OPTION_PARREPAIR, "yes");
 	SetOption(OPTION_PARSCAN, "limited");
+	SetOption(OPTION_PARQUICK, "yes");
 	SetOption(OPTION_PARRENAME, "yes");
+	SetOption(OPTION_PARBUFFER, "16");
+	SetOption(OPTION_PARTHREADS, "1");
 	SetOption(OPTION_HEALTHCHECK, "none");
 	SetOption(OPTION_SCRIPTORDER, "");
 	SetOption(OPTION_POSTSCRIPT, "");
@@ -780,7 +815,7 @@ void Options::InitDefault()
 	SetOption(OPTION_CURSESGROUP, "no");
 	SetOption(OPTION_CRCCHECK, "yes");
 	SetOption(OPTION_DIRECTWRITE, "yes");
-	SetOption(OPTION_WRITEBUFFERSIZE, "0");
+	SetOption(OPTION_WRITEBUFFER, "0");
 	SetOption(OPTION_NZBDIRINTERVAL, "5");
 	SetOption(OPTION_NZBDIRFILEAGE, "60");
 	SetOption(OPTION_PARCLEANUPQUEUE, "yes");
@@ -809,6 +844,8 @@ void Options::InitDefault()
 	SetOption(OPTION_URLFORCE, "yes");
 	SetOption(OPTION_TIMECORRECTION, "0");
 	SetOption(OPTION_PROPAGATIONDELAY, "0");
+	SetOption(OPTION_ARTICLECACHE, "0");
+	SetOption(OPTION_EVENTINTERVAL, "0");
 }
 
 void Options::InitOptFile()
@@ -880,28 +917,13 @@ void Options::InitOptFile()
 	m_bConfigInitialized = true;
 }
 
-void Options::CheckDir(char** dir, const char* szOptionName, bool bAllowEmpty, bool bCreate)
+void Options::CheckDir(char** dir, const char* szOptionName,
+	const char* szParentDir, bool bAllowEmpty, bool bCreate)
 {
 	char* usedir = NULL;
 	const char* tempdir = GetOption(szOptionName);
-	if (tempdir && strlen(tempdir) > 0)
-	{
-		int len = strlen(tempdir);
-		usedir = (char*) malloc(len + 2);
-		strcpy(usedir, tempdir);
-		char ch = usedir[len-1];
-		if (ch == ALT_PATH_SEPARATOR)
-		{
-			usedir[len-1] = PATH_SEPARATOR;
-		}
-		else if (ch != PATH_SEPARATOR)
-		{
-			usedir[len] = PATH_SEPARATOR;
-			usedir[len + 1] = '\0';
-		}
-		Util::NormalizePathSeparators(usedir);
-	}
-	else
+
+	if (Util::EmptyStr(tempdir))
 	{
 		if (!bAllowEmpty)
 		{
@@ -911,6 +933,45 @@ void Options::CheckDir(char** dir, const char* szOptionName, bool bAllowEmpty, b
 		return;
 	}
 
+	int len = strlen(tempdir);
+	usedir = (char*)malloc(len + 2);
+	strcpy(usedir, tempdir);
+	char ch = usedir[len-1];
+	Util::NormalizePathSeparators(usedir);
+	if (ch != PATH_SEPARATOR)
+	{
+		usedir[len] = PATH_SEPARATOR;
+		usedir[len + 1] = '\0';
+	}
+
+	if (!(usedir[0] == PATH_SEPARATOR || usedir[0] == ALT_PATH_SEPARATOR ||
+		(usedir[0] && usedir[1] == ':')) &&
+		!Util::EmptyStr(szParentDir))
+	{
+		// convert relative path to absolute path
+		int plen = strlen(szParentDir);
+		int len2 = len + plen + 4;
+		char* usedir2 = (char*)malloc(len2);
+		if (szParentDir[plen-1] == PATH_SEPARATOR || szParentDir[plen-1] == ALT_PATH_SEPARATOR)
+		{
+			snprintf(usedir2, len2, "%s%s", szParentDir, usedir);
+		}
+		else
+		{
+			snprintf(usedir2, len2, "%s%c%s", szParentDir, PATH_SEPARATOR, usedir);
+		}
+		usedir2[len2-1] = '\0';
+		free(usedir);
+
+		usedir = usedir2;
+		Util::NormalizePathSeparators(usedir);
+
+		int ulen = strlen(usedir);
+		usedir[ulen-1] = '\0';
+		SetOption(szOptionName, usedir);
+		usedir[ulen-1] = PATH_SEPARATOR;
+	}
+
 	// Ensure the dir is created
 	char szErrBuf[1024];
 	if (bCreate && !Util::ForceDirectories(usedir, szErrBuf, sizeof(szErrBuf)))
@@ -922,12 +983,14 @@ void Options::CheckDir(char** dir, const char* szOptionName, bool bAllowEmpty, b
 
 void Options::InitOptions()
 {
-	CheckDir(&m_szDestDir, OPTION_DESTDIR, false, false);
-	CheckDir(&m_szInterDir, OPTION_INTERDIR, true, true);
-	CheckDir(&m_szTempDir, OPTION_TEMPDIR, false, true);
-	CheckDir(&m_szQueueDir, OPTION_QUEUEDIR, false, true);
-	CheckDir(&m_szWebDir, OPTION_WEBDIR, true, false);
-	CheckDir(&m_szScriptDir, OPTION_SCRIPTDIR, true, false);
+	const char* szMainDir = GetOption(OPTION_MAINDIR);
+
+	CheckDir(&m_szDestDir, OPTION_DESTDIR, szMainDir, false, false);
+	CheckDir(&m_szInterDir, OPTION_INTERDIR, szMainDir, true, true);
+	CheckDir(&m_szTempDir, OPTION_TEMPDIR, szMainDir, false, true);
+	CheckDir(&m_szQueueDir, OPTION_QUEUEDIR, szMainDir, false, true);
+	CheckDir(&m_szWebDir, OPTION_WEBDIR, NULL, true, false);
+	CheckDir(&m_szScriptDir, OPTION_SCRIPTDIR, szMainDir, true, false);
 
 	m_szConfigTemplate		= strdup(GetOption(OPTION_CONFIGTEMPLATE));
 	m_szScriptOrder			= strdup(GetOption(OPTION_SCRIPTORDER));
@@ -949,7 +1012,8 @@ void Options::InitOptions()
 	m_szParIgnoreExt		= strdup(GetOption(OPTION_PARIGNOREEXT));
 
 	m_iDownloadRate			= (int)(ParseFloatValue(OPTION_DOWNLOADRATE) * 1024);
-	m_iConnectionTimeout	= ParseIntValue(OPTION_CONNECTIONTIMEOUT, 10);
+	m_iArticleTimeout		= ParseIntValue(OPTION_ARTICLETIMEOUT, 10);
+	m_iUrlTimeout			= ParseIntValue(OPTION_URLTIMEOUT, 10);
 	m_iTerminateTimeout		= ParseIntValue(OPTION_TERMINATETIMEOUT, 10);
 	m_iRetries				= ParseIntValue(OPTION_RETRIES, 10);
 	m_iRetryInterval		= ParseIntValue(OPTION_RETRYINTERVAL, 10);
@@ -957,9 +1021,10 @@ void Options::InitOptions()
 	m_iSecurePort			= ParseIntValue(OPTION_SECUREPORT, 10);
 	m_iUrlConnections		= ParseIntValue(OPTION_URLCONNECTIONS, 10);
 	m_iLogBufferSize		= ParseIntValue(OPTION_LOGBUFFERSIZE, 10);
+	m_iRotateLog			= ParseIntValue(OPTION_ROTATELOG, 10);
 	m_iUMask				= ParseIntValue(OPTION_UMASK, 8);
 	m_iUpdateInterval		= ParseIntValue(OPTION_UPDATEINTERVAL, 10);
-	m_iWriteBufferSize		= ParseIntValue(OPTION_WRITEBUFFERSIZE, 10);
+	m_iWriteBuffer			= ParseIntValue(OPTION_WRITEBUFFER, 10);
 	m_iNzbDirInterval		= ParseIntValue(OPTION_NZBDIRINTERVAL, 10);
 	m_iNzbDirFileAge		= ParseIntValue(OPTION_NZBDIRFILEAGE, 10);
 	m_iDiskSpace			= ParseIntValue(OPTION_DISKSPACE, 10);
@@ -973,17 +1038,20 @@ void Options::InitOptions()
 	}
 	m_iTimeCorrection *= 60;
 	m_iPropagationDelay		= ParseIntValue(OPTION_PROPAGATIONDELAY, 10) * 60;
+	m_iArticleCache			= ParseIntValue(OPTION_ARTICLECACHE, 10);
+	m_iEventInterval		= ParseIntValue(OPTION_EVENTINTERVAL, 10);
+	m_iParBuffer			= ParseIntValue(OPTION_PARBUFFER, 10);
+	m_iParThreads			= ParseIntValue(OPTION_PARTHREADS, 10);
 
-	CheckDir(&m_szNzbDir, OPTION_NZBDIR, m_iNzbDirInterval == 0, true);
+	CheckDir(&m_szNzbDir, OPTION_NZBDIR, szMainDir, m_iNzbDirInterval == 0, true);
 
 	m_bCreateBrokenLog		= (bool)ParseEnumValue(OPTION_CREATEBROKENLOG, BoolCount, BoolNames, BoolValues);
-	m_bResetLog				= (bool)ParseEnumValue(OPTION_RESETLOG, BoolCount, BoolNames, BoolValues);
 	m_bAppendCategoryDir	= (bool)ParseEnumValue(OPTION_APPENDCATEGORYDIR, BoolCount, BoolNames, BoolValues);
 	m_bContinuePartial		= (bool)ParseEnumValue(OPTION_CONTINUEPARTIAL, BoolCount, BoolNames, BoolValues);
 	m_bSaveQueue			= (bool)ParseEnumValue(OPTION_SAVEQUEUE, BoolCount, BoolNames, BoolValues);
 	m_bDupeCheck			= (bool)ParseEnumValue(OPTION_DUPECHECK, BoolCount, BoolNames, BoolValues);
-	m_bCreateLog			= (bool)ParseEnumValue(OPTION_CREATELOG, BoolCount, BoolNames, BoolValues);
 	m_bParRepair			= (bool)ParseEnumValue(OPTION_PARREPAIR, BoolCount, BoolNames, BoolValues);
+	m_bParQuick				= (bool)ParseEnumValue(OPTION_PARQUICK, BoolCount, BoolNames, BoolValues);
 	m_bParRename			= (bool)ParseEnumValue(OPTION_PARRENAME, BoolCount, BoolNames, BoolValues);
 	m_bReloadQueue			= (bool)ParseEnumValue(OPTION_RELOADQUEUE, BoolCount, BoolNames, BoolValues);
 	m_bCursesNZBName		= (bool)ParseEnumValue(OPTION_CURSESNZBNAME, BoolCount, BoolNames, BoolValues);
@@ -1033,6 +1101,11 @@ void Options::InitOptions()
 	m_eErrorTarget = (EMessageTarget)ParseEnumValue(OPTION_ERRORTARGET, TargetCount, TargetNames, TargetValues);
 	m_eDebugTarget = (EMessageTarget)ParseEnumValue(OPTION_DEBUGTARGET, TargetCount, TargetNames, TargetValues);
 	m_eDetailTarget = (EMessageTarget)ParseEnumValue(OPTION_DETAILTARGET, TargetCount, TargetNames, TargetValues);
+
+	const char* WriteLogNames[] = { "none", "append", "reset", "rotate" };
+	const int WriteLogValues[] = { wlNone, wlAppend, wlReset, wlRotate };
+	const int WriteLogCount = 4;
+	m_eWriteLog = (EWriteLog)ParseEnumValue(OPTION_WRITELOG, WriteLogCount, WriteLogNames, WriteLogValues);
 }
 
 int Options::ParseEnumValue(const char* OptName, int argc, const char * argn[], const int argv[])
@@ -1042,7 +1115,6 @@ int Options::ParseEnumValue(const char* OptName, int argc, const char * argn[],
 	{
 		ConfigError("Undefined value for option \"%s\"", OptName);
 		return argv[0];
-		//abort("FATAL ERROR: Undefined value for option \"%s\"\n", OptName);
 	}
 
 	int iDefNum = 0;
@@ -1728,7 +1800,7 @@ void Options::PrintUsage(char* com)
 		"    <Names>                 List of names (with options \"FN\" and \"GN\"),\n"
 		"                            e. g.: \"my nzb download%cmyfile.nfo\" \"another nzb\"\n"
 		"    <RegExs>                List of regular expressions (options \"FR\", \"GR\")\n"
-		"                            using POSIX Extended Regular Expression Syntax",
+		"                            using POSIX Extended Regular Expression Syntax\n",
 		Util::BaseFileName(com),
 		PATH_SEPARATOR);
 }
@@ -1801,6 +1873,16 @@ void Options::InitFileArg(int argc, char* argv[])
 	}
 }
 
+const char* Options::GetControlIP()
+{
+	if ((m_bRemoteClientMode || m_eClientOperation != opClientNoOperation) &&
+		!strcmp(m_szControlIP, "0.0.0.0"))
+	{
+		return "127.0.0.1";
+	}
+	return m_szControlIP;
+}
+
 void Options::SetOption(const char* optname, const char* value)
 {
 	OptEntry* pOptEntry = FindOption(optname);
@@ -1989,7 +2071,7 @@ void Options::InitServers()
 		n++;
 	}
 
-	g_pServerPool->SetTimeout(GetConnectionTimeout());
+	g_pServerPool->SetTimeout(GetArticleTimeout());
 }
 
 void Options::InitCategories()
@@ -2033,7 +2115,7 @@ void Options::InitCategories()
 			char* szDestDir = NULL;
 			if (ndestdir && ndestdir[0] != '\0')
 			{
-				CheckDir(&szDestDir, destdiroptname, false, false);
+				CheckDir(&szDestDir, destdiroptname, m_szDestDir, false, false);
 			}
 
 			Category* pCategory = new Category(nname, szDestDir, bUnpack, npostscript);
@@ -2118,8 +2200,7 @@ void Options::InitFeeds()
 
 void Options::InitScheduler()
 {
-	int n = 1;
-	while (true)
+	for (int n = 1; ; n++)
 	{
 		char optname[128];
 
@@ -2158,41 +2239,42 @@ void Options::InitScheduler()
 			break;
 		}
 
-		bool bOK = true;
-
 		if (!completed)
 		{
 			ConfigError("Task definition not complete for \"Task%i\"", n);
-			bOK = false;
+			continue;
 		}
 
 		snprintf(optname, sizeof(optname), "Task%i.Command", n);
 		optname[sizeof(optname)-1] = '\0';
 
 		const char* CommandNames[] = { "pausedownload", "pause", "unpausedownload", "resumedownload", "unpause", "resume",
+			"pausepostprocess", "unpausepostprocess", "resumepostprocess", "pausepost", "unpausepost", "resumepost",
 			"downloadrate", "setdownloadrate", "rate", "speed", "script", "process", "pausescan", "unpausescan", "resumescan",
 			"activateserver", "activateservers", "deactivateserver", "deactivateservers", "fetchfeed", "fetchfeeds" };
 		const int CommandValues[] = { Scheduler::scPauseDownload, Scheduler::scPauseDownload, Scheduler::scUnpauseDownload,
-			Scheduler::scUnpauseDownload, Scheduler::scUnpauseDownload, Scheduler::scUnpauseDownload, Scheduler::scDownloadRate,
-			Scheduler::scDownloadRate, Scheduler::scDownloadRate, Scheduler::scDownloadRate, Scheduler::scScript,
-			Scheduler::scProcess, Scheduler::scPauseScan, Scheduler::scUnpauseScan, Scheduler::scUnpauseScan,
+			Scheduler::scUnpauseDownload, Scheduler::scUnpauseDownload, Scheduler::scUnpauseDownload,
+			Scheduler::scPausePostProcess, Scheduler::scUnpausePostProcess, Scheduler::scUnpausePostProcess,
+			Scheduler::scPausePostProcess, Scheduler::scUnpausePostProcess, Scheduler::scUnpausePostProcess,
+			Scheduler::scDownloadRate, Scheduler::scDownloadRate, Scheduler::scDownloadRate, Scheduler::scDownloadRate,
+			Scheduler::scScript, Scheduler::scProcess, Scheduler::scPauseScan, Scheduler::scUnpauseScan, Scheduler::scUnpauseScan,
 			Scheduler::scActivateServer, Scheduler::scActivateServer, Scheduler::scDeactivateServer,
 			Scheduler::scDeactivateServer, Scheduler::scFetchFeed, Scheduler::scFetchFeed };
-		const int CommandCount = 21;
+		const int CommandCount = 27;
 		Scheduler::ECommand eCommand = (Scheduler::ECommand)ParseEnumValue(optname, CommandCount, CommandNames, CommandValues);
 
 		if (szParam && strlen(szParam) > 0 && eCommand == Scheduler::scProcess &&
 			!Util::SplitCommandLine(szParam, NULL))
 		{
 			ConfigError("Invalid value for option \"Task%i.Param\"", n);
-			bOK = false;
+			continue;
 		}
 
 		int iWeekDays = 0;
 		if (szWeekDays && !ParseWeekDays(szWeekDays, &iWeekDays))
 		{
 			ConfigError("Invalid value for option \"Task%i.WeekDays\": \"%s\"", n, szWeekDays);
-			bOK = false;
+			continue;
 		}
 
 		if (eCommand == Scheduler::scDownloadRate)
@@ -2204,13 +2286,13 @@ void Options::InitScheduler()
 				if (!szErr || *szErr != '\0' || iDownloadRate < 0)
 				{
 					ConfigError("Invalid value for option \"Task%i.Param\": \"%s\"", n, szDownloadRate);
-					bOK = false;
+					continue;
 				}
 			}
 			else
 			{
 				ConfigError("Task definition not complete for \"Task%i\". Option \"Task%i.Param\" is missing", n, n);
-				bOK = false;
+				continue;
 			}
 		}
 
@@ -2222,52 +2304,41 @@ void Options::InitScheduler()
 			Util::EmptyStr(szParam))
 		{
 			ConfigError("Task definition not complete for \"Task%i\". Option \"Task%i.Param\" is missing", n, n);
-			bOK = false;
+			continue;
 		}
 
 		int iHours, iMinutes;
-		const char** pTime = &szTime;
-		while (*pTime)
+		Tokenizer tok(szTime, ";,");
+		while (const char* szOneTime = tok.Next())
 		{
-			if (!ParseTime(pTime, &iHours, &iMinutes))
+			if (!ParseTime(szOneTime, &iHours, &iMinutes))
 			{
-				ConfigError("Invalid value for option \"Task%i.Time\": \"%s\"", n, pTime);
+				ConfigError("Invalid value for option \"Task%i.Time\": \"%s\"", n, szOneTime);
 				break;
 			}
 
-			if (bOK)
+			if (iHours == -1)
 			{
-				if (iHours == -1)
+				for (int iEveryHour = 0; iEveryHour < 24; iEveryHour++)
 				{
-					for (int iEveryHour = 0; iEveryHour < 24; iEveryHour++)
-					{
-						Scheduler::Task* pTask = new Scheduler::Task(n, iEveryHour, iMinutes, iWeekDays, eCommand, szParam);
-						g_pScheduler->AddTask(pTask);
-					}
-				}
-				else
-				{
-					Scheduler::Task* pTask = new Scheduler::Task(n, iHours, iMinutes, iWeekDays, eCommand, szParam);
+					Scheduler::Task* pTask = new Scheduler::Task(n, iEveryHour, iMinutes, iWeekDays, eCommand, szParam);
 					g_pScheduler->AddTask(pTask);
 				}
 			}
+			else
+			{
+				Scheduler::Task* pTask = new Scheduler::Task(n, iHours, iMinutes, iWeekDays, eCommand, szParam);
+				g_pScheduler->AddTask(pTask);
+			}
 		}
-
-		n++;
 	}
 }
 
-/*
-* Parses Time string and moves current string pointer to the next time token.
-*/
-bool Options::ParseTime(const char** pTime, int* pHours, int* pMinutes)
+bool Options::ParseTime(const char* szTime, int* pHours, int* pMinutes)
 {
-	const char* szTime = *pTime;
-	const char* szComma = strchr(szTime, ',');
-
 	int iColons = 0;
 	const char* p = szTime;
-	while (*p && (!szComma || p != szComma))
+	while (*p)
 	{
 		if (!strchr("0123456789: *", *p))
 		{
@@ -2314,15 +2385,6 @@ bool Options::ParseTime(const char** pTime, int* pHours, int* pMinutes)
 		return false;
 	}
 
-	if (szComma)
-	{
-		*pTime = szComma + 1;
-	}
-	else
-	{
-		*pTime = NULL;
-	}
-
 	return true;
 }
 
@@ -2595,6 +2657,12 @@ bool Options::ValidateOptionName(const char* optname, const char* optvalue)
 		return true;
 	}
 
+	if (!strcasecmp(optname, OPTION_CREATELOG) || !strcasecmp(optname, OPTION_RESETLOG))
+	{
+		ConfigWarn("Option \"%s\" is obsolete, ignored, use \"%s\" instead", optname, OPTION_WRITELOG);
+		return true;
+	}
+
 	return false;
 }
 
@@ -2649,6 +2717,19 @@ void Options::ConvertOldOption(char *szOption, int iOptionBufLen, char *szValue,
 		strncpy(szOption + iNameLen - 10, ".PostScript", iOptionBufLen - 9 /* strlen("Category.") */);
 	}
 
+	if (!strcasecmp(szOption, "WriteBufferSize"))
+	{
+		strncpy(szOption, "WriteBuffer", iOptionBufLen);
+		int val = strtol(szValue, NULL, 10);
+		val = val == -1 ? 1024 : val / 1024;
+		snprintf(szValue, iValueBufLen, "%i", val);
+	}	
+
+	if (!strcasecmp(szOption, "ConnectionTimeout"))
+	{
+		strncpy(szOption, "ArticleTimeout", iOptionBufLen);
+	}
+
 	szOption[iOptionBufLen-1] = '\0';
 	szOption[iValueBufLen-1] = '\0';
 }
@@ -2704,6 +2785,28 @@ void Options::CheckOptions()
 			m_szConfigTemplate = strdup("");
 		}
 	}
+
+	if (m_iArticleCache < 0)
+	{
+		m_iArticleCache = 0;
+	}
+	else if (sizeof(void*) == 4 && m_iArticleCache > 1900)
+	{
+		ConfigError("Invalid value for option \"ArticleCache\": %i. Changed to 1900", m_iArticleCache);
+		m_iArticleCache = 1900;
+	}
+	else if (sizeof(void*) == 4 && m_iParBuffer > 1900)
+	{
+		ConfigError("Invalid value for option \"ParBuffer\": %i. Changed to 1900", m_iParBuffer);
+		m_iParBuffer = 1900;
+	}
+
+	if (sizeof(void*) == 4 && m_iParBuffer + m_iArticleCache > 1900)
+	{
+		ConfigError("Options \"ArticleCache\" and \"ParBuffer\" in total cannot use more than 1900MB of memory in 32-Bit mode. Changed to 1500 and 400");
+		m_iArticleCache = 1900;
+		m_iParBuffer = 400;
+	}
 }
 
 void Options::ParseFileIDList(int argc, char* argv[], int optind)
@@ -2925,7 +3028,7 @@ bool Options::SaveConfig(OptEntries* pOptEntries)
 	}
 
 	// close and truncate the file
-	int pos = ftell(infile);
+	int pos = (int)ftell(infile);
 	fclose(infile);
 
 	Util::TruncateFile(m_szConfigFilename, pos);
@@ -2950,12 +3053,13 @@ bool Options::LoadConfigTemplates(ConfigTemplates* pConfigTemplates)
 		return true;
 	}
 
-	ScriptList scriptList;
-	LoadScriptList(&scriptList);
+	Scripts scriptList;
+	LoadScripts(&scriptList);
 
 	const int iBeginSignatureLen = strlen(BEGIN_SCRIPT_SIGNATURE);
+	const int iQueueEventsSignatureLen = strlen(QUEUE_EVENTS_SIGNATURE);
 
-	for (ScriptList::iterator it = scriptList.begin(); it != scriptList.end(); it++)
+	for (Scripts::iterator it = scriptList.begin(); it != scriptList.end(); it++)
 	{
 		Script* pScript = *it;
 
@@ -2988,7 +3092,9 @@ bool Options::LoadConfigTemplates(ConfigTemplates* pConfigTemplates)
 				continue;
 			}
 
-			if (bInConfig)
+			bool bSkip = !strncmp(buf, QUEUE_EVENTS_SIGNATURE, iQueueEventsSignatureLen);
+
+			if (bInConfig && !bSkip)
 			{
 				stringBuilder.Append(buf);
 			}
@@ -3006,50 +3112,64 @@ bool Options::LoadConfigTemplates(ConfigTemplates* pConfigTemplates)
 	return true;
 }
 
-void Options::LoadScriptList(ScriptList* pScriptList)
+void Options::InitConfigTemplates()
+{
+	if (!LoadConfigTemplates(&m_ConfigTemplates))
+	{
+		error("Could not read configuration templates");
+	}
+}
+
+void Options::InitScripts()
+{
+	LoadScripts(&m_Scripts);
+}
+
+void Options::LoadScripts(Scripts* pScripts)
 {
 	if (strlen(m_szScriptDir) == 0)
 	{
 		return;
 	}
 
-	ScriptList tmpScriptList;
-	LoadScriptDir(&tmpScriptList, m_szScriptDir, false);
-	tmpScriptList.sort(CompareScripts);
+	Scripts tmpScripts;
+	LoadScriptDir(&tmpScripts, m_szScriptDir, false);
+	tmpScripts.sort(CompareScripts);
 
 	// first add all scripts from m_szScriptOrder
 	Tokenizer tok(m_szScriptOrder, ",;");
 	while (const char* szScriptName = tok.Next())
 	{
-		Script* pScript = tmpScriptList.Find(szScriptName);
+		Script* pScript = tmpScripts.Find(szScriptName);
 		if (pScript)
 		{
-			tmpScriptList.remove(pScript);
-			pScriptList->push_back(pScript);
+			tmpScripts.remove(pScript);
+			pScripts->push_back(pScript);
 		}
 	}
 
 	// second add all other scripts from scripts directory
-	for (ScriptList::iterator it = tmpScriptList.begin(); it != tmpScriptList.end(); it++)
+	for (Scripts::iterator it = tmpScripts.begin(); it != tmpScripts.end(); it++)
 	{
 		Script* pScript = *it;
-		if (!pScriptList->Find(pScript->GetName()))
+		if (!pScripts->Find(pScript->GetName()))
 		{
-			pScriptList->push_back(pScript);
+			pScripts->push_back(pScript);
 		}
 	}
 
-	tmpScriptList.clear();
+	tmpScripts.clear();
 
-	BuildScriptDisplayNames(pScriptList);
+	BuildScriptDisplayNames(pScripts);
 }
 
-void Options::LoadScriptDir(ScriptList* pScriptList, const char* szDirectory, bool bIsSubDir)
+void Options::LoadScriptDir(Scripts* pScripts, const char* szDirectory, bool bIsSubDir)
 {
 	int iBufSize = 1024*10;
 	char* szBuffer = (char*)malloc(iBufSize+1);
 
 	const int iBeginSignatureLen = strlen(BEGIN_SCRIPT_SIGNATURE);
+	const int iQueueEventsSignatureLen = strlen(QUEUE_EVENTS_SIGNATURE);
 
 	DirBrowser dir(szDirectory);
 	while (const char* szFilename = dir.Next())
@@ -3105,12 +3225,26 @@ void Options::LoadScriptDir(ScriptList* pScriptList, const char* szDirectory, bo
 								}
 								szScriptName[1024-1] = '\0';
 
+								char* szQueueEvents = NULL;
+								if (bQueueScript)
+								{
+									while (char* szLine = tok.Next())
+									{
+										if (!strncmp(szLine, QUEUE_EVENTS_SIGNATURE, iQueueEventsSignatureLen))
+										{
+											szQueueEvents = szLine + iQueueEventsSignatureLen;
+											break;
+										}
+									}
+								}
+
 								Script* pScript = new Script(szScriptName, szFullFilename);
 								pScript->SetPostScript(bPostScript);
 								pScript->SetScanScript(bScanScript);
 								pScript->SetQueueScript(bQueueScript);
 								pScript->SetSchedulerScript(bSchedulerScript);
-								pScriptList->push_back(pScript);
+								pScript->SetQueueEvents(szQueueEvents);
+								pScripts->push_back(pScript);
 								break;
 							}
 						}
@@ -3122,7 +3256,7 @@ void Options::LoadScriptDir(ScriptList* pScriptList, const char* szDirectory, bo
 				snprintf(szFullFilename, 1024, "%s%s%c", szDirectory, szFilename, PATH_SEPARATOR);
 				szFullFilename[1024-1] = '\0';
 
-				LoadScriptDir(pScriptList, szFullFilename, true);
+				LoadScriptDir(pScripts, szFullFilename, true);
 			}
 		}
 	}
@@ -3135,12 +3269,12 @@ bool Options::CompareScripts(Script* pScript1, Script* pScript2)
 	return strcmp(pScript1->GetName(), pScript2->GetName()) < 0;
 }
 
-void Options::BuildScriptDisplayNames(ScriptList* pScriptList)
+void Options::BuildScriptDisplayNames(Scripts* pScripts)
 {
 	// trying to use short name without path and extension.
 	// if there are other scripts with the same short name - using a longer name instead (with ot without extension)
 
-	for (ScriptList::iterator it = pScriptList->begin(); it != pScriptList->end(); it++)
+	for (Scripts::iterator it = pScripts->begin(); it != pScripts->end(); it++)
 	{
 		Script* pScript = *it;
 
@@ -3151,7 +3285,7 @@ void Options::BuildScriptDisplayNames(ScriptList* pScriptList)
 
 		const char* szDisplayName = Util::BaseFileName(szShortName);
 
-		for (ScriptList::iterator it2 = pScriptList->begin(); it2 != pScriptList->end(); it2++)
+		for (Scripts::iterator it2 = pScripts->begin(); it2 != pScripts->end(); it2++)
 		{
 			Script* pScript2 = *it2;
 
