@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 981 $
- * $Date: 2014-04-09 23:09:45 +0200 (Wed, 09 Apr 2014) $
+ * $Revision: 1170 $
+ * $Date: 2014-11-24 23:35:28 +0100 (Mon, 24 Nov 2014) $
  *
  */
 
@@ -40,7 +40,6 @@
 #include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/resource.h>
 #ifdef HAVE_SYS_PRCTL_H
 #include <sys/prctl.h>
 #endif
@@ -53,9 +52,6 @@
 #ifndef DISABLE_PARCHECK
 #include <iostream>
 #endif
-#ifdef HAVE_BACKTRACE
-#include <execinfo.h>
-#endif
 
 #include "nzbget.h"
 #include "ServerPool.h"
@@ -79,8 +75,11 @@
 #include "Scanner.h"
 #include "FeedCoordinator.h"
 #include "Maintenance.h"
+#include "ArticleWriter.h"
 #include "StatMeter.h"
+#include "QueueScript.h"
 #include "Util.h"
+#include "StackTrace.h"
 #ifdef WIN32
 #include "NTService.h"
 #endif
@@ -92,15 +91,7 @@ void Reload();
 void Cleanup();
 void ProcessClientRequest();
 #ifndef WIN32
-void InstallSignalHandlers();
 void Daemonize();
-void PrintBacktrace();
-#ifdef HAVE_SYS_PRCTL_H
-void EnableDumpCore();
-#endif
-#ifdef DEBUG
-void MakeSegFault();
-#endif
 #endif
 #ifndef DISABLE_PARCHECK
 void DisableCout();
@@ -123,6 +114,8 @@ Scheduler* g_pScheduler = NULL;
 Scanner* g_pScanner = NULL;
 FeedCoordinator* g_pFeedCoordinator = NULL;
 Maintenance* g_pMaintenance = NULL;
+ArticleCache* g_pArticleCache = NULL;
+QueueScriptCoordinator* g_pQueueScriptCoordinator = NULL;
 int g_iArgumentCount;
 char* (*g_szEnvironmentVariables)[] = NULL;
 char* (*g_szArguments)[] = NULL;
@@ -137,7 +130,7 @@ int main(int argc, char *argv[], char *argp[])
 #ifdef _DEBUG
 	_CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE | _CRTDBG_MODE_DEBUG);
 	_CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDERR);
-	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF
+	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF
 #ifdef DEBUG_CRTMEMLEAKS
 		| _CRTDBG_CHECK_CRT_DF | _CRTDBG_CHECK_ALWAYS_DF
 #endif
@@ -174,12 +167,6 @@ int main(int argc, char *argv[], char *argp[])
 
 	RunMain();
 
-#ifdef WIN32
-#ifdef _DEBUG
-	_CrtDumpMemoryLeaks();
-#endif
-#endif
-
 	return 0;
 }
 
@@ -223,7 +210,9 @@ void Run(bool bReload)
 	g_pDupeCoordinator = new DupeCoordinator();
 	g_pUrlCoordinator = new UrlCoordinator();
 	g_pFeedCoordinator = new FeedCoordinator();
+	g_pArticleCache = new ArticleCache();
 	g_pMaintenance = new Maintenance();
+	g_pQueueScriptCoordinator = new QueueScriptCoordinator();
 
 	debug("Reading options");
 	g_pOptions = new Options(g_iArgumentCount, *g_szArguments);
@@ -236,14 +225,9 @@ void Run(bool bReload)
 	}
 #endif
 	
-	if (g_pOptions->GetServerMode() && g_pOptions->GetCreateLog() && g_pOptions->GetResetLog())
-	{
-		debug("Deleting old log-file");
-		g_pLog->ResetLog();
-	}
-
 	g_pLog->InitOptions();
 	g_pScanner->InitOptions();
+	g_pQueueScriptCoordinator->InitOptions();
 
 	if (g_pOptions->GetDaemonMode())
 	{
@@ -277,23 +261,13 @@ void Run(bool bReload)
 		g_pStatMeter->Init();
 	}
 
-#ifndef WIN32
-#ifdef HAVE_SYS_PRCTL_H
-	if (g_pOptions->GetDumpCore())
-	{
-		EnableDumpCore();
-	}
-#endif
-#endif
+	InstallErrorHandler();
 
-#ifndef WIN32
-	InstallSignalHandlers();
 #ifdef DEBUG
 	if (g_pOptions->GetTestBacktrace())
 	{
-		MakeSegFault();
+		TestSegFault();
 	}
-#endif
 #endif
 
 	// client request
@@ -355,7 +329,7 @@ void Run(bool bReload)
 				abort("FATAL ERROR: Parsing NZB-document %s failed\n\n", g_pOptions->GetArgFilename() ? g_pOptions->GetArgFilename() : "N/A");
 				return;
 			}
-			g_pScanner->InitPPParameters(szCategory, pNZBFile->GetNZBInfo()->GetParameters());
+			g_pScanner->InitPPParameters(szCategory, pNZBFile->GetNZBInfo()->GetParameters(), false);
 			g_pQueueCoordinator->AddNZBFileToQueue(pNZBFile, NULL, false);
 			delete pNZBFile;
 		}
@@ -369,12 +343,17 @@ void Run(bool bReload)
 		g_pUrlCoordinator->Start();
 		g_pPrePostProcessor->Start();
 		g_pFeedCoordinator->Start();
+		if (g_pOptions->GetArticleCache() > 0)
+		{
+			g_pArticleCache->Start();
+		}
 
 		// enter main program-loop
 		while (g_pQueueCoordinator->IsRunning() || 
 			g_pUrlCoordinator->IsRunning() || 
 			g_pPrePostProcessor->IsRunning() ||
-			g_pFeedCoordinator->IsRunning())
+			g_pFeedCoordinator->IsRunning() ||
+			g_pArticleCache->IsRunning())
 		{
 			if (!g_pOptions->GetServerMode() && 
 				!g_pQueueCoordinator->HasMoreJobs() && 
@@ -398,6 +377,10 @@ void Run(bool bReload)
 				{
 					g_pFeedCoordinator->Stop();
 				}
+				if (!g_pArticleCache->IsStopped())
+				{
+					g_pArticleCache->Stop();
+				}
 			}
 			usleep(100 * 1000);
 		}
@@ -407,8 +390,11 @@ void Run(bool bReload)
 		debug("UrlCoordinator stopped");
 		debug("PrePostProcessor stopped");
 		debug("FeedCoordinator stopped");
+		debug("ArticleCache stopped");
 	}
 
+	ScriptController::TerminateAll();
+
 	// Stop network-server
 	if (g_pRemoteServer)
 	{
@@ -595,6 +581,7 @@ void ExitProc()
 			g_pUrlCoordinator->Stop();
 			g_pPrePostProcessor->Stop();
 			g_pFeedCoordinator->Stop();
+			g_pArticleCache->Stop();
 		}
 	}
 }
@@ -606,114 +593,6 @@ void Reload()
 	ExitProc();
 }
 
-#ifndef WIN32
-#ifdef DEBUG
-typedef void(*sighandler)(int);
-std::vector<sighandler> SignalProcList;
-#endif
-
-/*
- * Signal handler
- */
-void SignalProc(int iSignal)
-{
-	switch (iSignal)
-	{
-		case SIGINT:
-			signal(SIGINT, SIG_DFL);   // Reset the signal handler
-			ExitProc();
-			break;
-
-		case SIGTERM:
-			signal(SIGTERM, SIG_DFL);   // Reset the signal handler
-			ExitProc();
-			break;
-
-		case SIGCHLD:
-			// ignoring
-			break;
-
-#ifdef DEBUG
-		case SIGSEGV:
-			signal(SIGSEGV, SIG_DFL);   // Reset the signal handler
-			PrintBacktrace();
-			break;
-#endif
-	}
-}
-
-void InstallSignalHandlers()
-{
-	signal(SIGINT, SignalProc);
-	signal(SIGTERM, SignalProc);
-	signal(SIGPIPE, SIG_IGN);
-#ifdef DEBUG
-	signal(SIGSEGV, SignalProc);
-#endif
-#ifdef SIGCHLD_HANDLER
-    // it could be necessary on some systems to activate a handler for SIGCHLD
-    // however it make troubles on other systems and is deactivated by default
-	signal(SIGCHLD, SignalProc);
-#endif
-}
-
-void PrintBacktrace()
-{
-#ifdef HAVE_BACKTRACE
-	printf("Segmentation fault, tracing...\n");
-	
-	void *array[100];
-	size_t size;
-	char **strings;
-	size_t i;
-
-	size = backtrace(array, 100);
-	strings = backtrace_symbols(array, size);
-
-	// first trace to screen
-	printf("Obtained %zd stack frames\n", size);
-	for (i = 0; i < size; i++)
-	{
-		printf("%s\n", strings[i]);
-	}
-
-	// then trace to log
-	error("Segmentation fault, tracing...");
-	error("Obtained %zd stack frames", size);
-	for (i = 0; i < size; i++)
-	{
-		error("%s", strings[i]);
-	}
-
-	free(strings);
-#else
-	error("Segmentation fault");
-#endif
-}
-
-#ifdef DEBUG
-void MakeSegFault()
-{
-	char* N = NULL;
-	strcpy(N, "");
-}
-#endif
-
-#ifdef HAVE_SYS_PRCTL_H
-/**
-* activates the creation of core-files
-*/
-void EnableDumpCore()
-{
-	rlimit rlim;
-	rlim.rlim_cur= RLIM_INFINITY;
-	rlim.rlim_max= RLIM_INFINITY;
-	setrlimit(RLIMIT_CORE, &rlim);
-	prctl(PR_SET_DUMPABLE, 1);
-}
-#endif
-#endif
-
 void Cleanup()
 {
 	debug("Cleaning up global objects");
@@ -793,6 +672,16 @@ void Cleanup()
 	g_pFeedCoordinator = NULL;
 	debug("FeedCoordinator deleted");
 
+	debug("Deleting ArticleCache");
+	delete g_pArticleCache;
+	g_pArticleCache = NULL;
+	debug("ArticleCache deleted");
+
+	debug("Deleting QueueScriptCoordinator");
+	delete g_pQueueScriptCoordinator;
+	g_pQueueScriptCoordinator = NULL;
+	debug("QueueScriptCoordinator deleted");
+
 	debug("Deleting Maintenance");
 	delete g_pMaintenance;
 	g_pMaintenance = NULL;
