@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1022 $
- * $Date: 2014-05-22 18:58:16 +0200 (Thu, 22 May 2014) $
+ * $Revision: 1122 $
+ * $Date: 2014-09-08 21:35:11 +0200 (Mon, 08 Sep 2014) $
  *
  */
 
@@ -50,7 +50,6 @@
 #define	S_ISREG(mode)	 __S_ISTYPE((mode), _S_IFREG)
 #define	S_DIRMODE NULL
 #define usleep(usec) Sleep((usec) / 1000)
-#define gettimeofday(tm, ignore) _ftime(tm)
 #define socklen_t int
 #define SHUT_WR 0x01
 #define SHUT_RDWR 0x02
@@ -59,6 +58,8 @@
 #define LINE_ENDING "\r\n"
 #define pid_t int
 #define atoll _atoi64
+#define fseek _fseeki64
+#define ftell _ftelli64
 #ifndef FSCTL_SET_SPARSE
 #define FSCTL_SET_SPARSE 590020
 #endif
