@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 969 $
- * $Date: 2014-03-20 22:37:32 +0100 (Thu, 20 Mar 2014) $
+ * $Revision: 1140 $
+ * $Date: 2014-10-10 23:13:02 +0200 (Fri, 10 Oct 2014) $
  *
  */
 
@@ -34,6 +34,7 @@
 #include "Thread.h"
 #include "NNTPConnection.h"
 #include "Decoder.h"
+#include "ArticleWriter.h"
 
 class ArticleDownloader : public Thread, public Subject
 {
@@ -47,13 +48,20 @@ public:
 		adFailed,
 		adRetry,
 		adCrcError,
-		adDecoding,
-		adJoining,
-		adJoined,
 		adNotFound,
 		adConnectError,
 		adFatalError
 	};
+
+	class ArticleWriterImpl : public ArticleWriter
+	{
+	private:
+		 ArticleDownloader*		m_pOwner;
+	protected:
+		virtual void	SetLastUpdateTimeNow() { m_pOwner->SetLastUpdateTimeNow(); } 
+	public:
+		void			SetOwner(ArticleDownloader* pOwner) { m_pOwner = pOwner; }
+	};
 			
 private:
 	FileInfo*			m_pFileInfo;
@@ -61,31 +69,25 @@ private:
 	NNTPConnection* 	m_pConnection;
 	EStatus				m_eStatus;
 	Mutex			 	m_mutexConnection;
-	const char*			m_szResultFilename;
-	char*				m_szTempFilename;
-	char*				m_szArticleFilename;
 	char*				m_szInfoName;
-	char*				m_szOutputFilename;
+	char				m_szConnectionName[250];
+	char*				m_szArticleFilename;
 	time_t				m_tLastUpdateTime;
 	Decoder::EFormat	m_eFormat;
 	YDecoder			m_YDecoder;
 	UDecoder			m_UDecoder;
-	FILE*				m_pOutFile;
-	bool				m_bDuplicate;
+	ArticleWriterImpl	m_ArticleWriter;
 	ServerStatList		m_ServerStats;
+	bool				m_bWritingStarted;
+	int					m_iDownloadedSize;
 
 	EStatus				Download();
-	bool				Write(char* szLine, int iLen);
-	bool				PrepareFile(char* szLine);
-	bool				CreateOutputFile(int iSize);
-	void				BuildOutputFilename();
 	EStatus				DecodeCheck();
 	void				FreeConnection(bool bKeepConnected);
 	EStatus				CheckResponse(const char* szResponse, const char* szComment);
 	void				SetStatus(EStatus eStatus) { m_eStatus = eStatus; }
-	const char* 		GetTempFilename() { return m_szTempFilename; }
-	void 				SetTempFilename(const char* v);
-	void 				SetOutputFilename(const char* v);
+	bool				Write(char* szLine, int iLen);
+	void				AddServerData();
 
 public:
 						ArticleDownloader();
@@ -102,11 +104,12 @@ public:
 	time_t				GetLastUpdateTime() { return m_tLastUpdateTime; }
 	void				SetLastUpdateTimeNow() { m_tLastUpdateTime = ::time(NULL); }
 	const char* 		GetArticleFilename() { return m_szArticleFilename; }
-	void				SetInfoName(const char* v);
+	void				SetInfoName(const char* szInfoName);
 	const char*			GetInfoName() { return m_szInfoName; }
-	void				CompleteFileParts();
-	static bool			MoveCompletedFiles(NZBInfo* pNZBInfo, const char* szOldDestDir);
+	const char*			GetConnectionName() { return m_szConnectionName; }
 	void				SetConnection(NNTPConnection* pConnection) { m_pConnection = pConnection; }
+	void				CompleteFileParts() { m_ArticleWriter.CompleteFileParts(); }
+	int					GetDownloadedSize() { return m_iDownloadedSize; }
 
 	void				LogDebugInfo();
 };
