@@ -1,7 +1,7 @@
 /*
  *  This file is part of nzbget
  *
- *  Copyright (C) 2007-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ *  Copyright (C) 2007-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1022 $
- * $Date: 2014-05-22 18:58:16 +0200 (Thu, 22 May 2014) $
+ * $Revision: 1164 $
+ * $Date: 2014-11-22 21:05:48 +0100 (Sat, 22 Nov 2014) $
  *
  */
 
@@ -29,6 +29,7 @@
 
 #ifdef WIN32
 #include "win32.h"
+#include "Mmsystem.h"
 #endif
 
 #ifndef DISABLE_PARCHECK
@@ -37,17 +38,17 @@
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
-#ifdef WIN32
-#include <par2cmdline.h>
-#include <par2repairer.h>
-#else
+#ifndef WIN32
 #include <unistd.h>
-#include <libpar2/par2cmdline.h>
-#include <libpar2/par2repairer.h>
 #endif
+#include <vector>
 #include <algorithm>
 
+#include "par2cmdline.h"
+#include "par2repairer.h"
+
 #include "nzbget.h"
+#include "Thread.h"
 #include "ParChecker.h"
 #include "ParCoordinator.h"
 #include "Log.h"
@@ -66,25 +67,77 @@ const char* Par2CmdLineErrStr[] = { "OK",
 	"internal error occurred",
 	"out of memory" };
 
+// Sleep interval for synchronisation (microseconds)
+#ifdef WIN32
+// Windows doesn't allow sleep intervals less than one millisecond
+#define SYNC_SLEEP_INTERVAL 1000
+#else
+#define SYNC_SLEEP_INTERVAL 100
+#endif
+
+class RepairThread;
 
 class Repairer : public Par2Repairer
 {
 private:
-    CommandLine	commandLine;
+	typedef vector<Thread*> Threads;
+
+	CommandLine		commandLine;
+	ParChecker*		m_pOwner;
+	Threads			m_Threads;
+	bool			m_bParallel;
+
+#ifdef HAVE_SPINLOCK
+	SpinLock		progresslock;
+#else
+	Mutex			progresslock;
+#endif
+
+	virtual void	BeginRepair();
+	virtual void	EndRepair();
+	void			RepairBlock(u32 inputindex, u32 outputindex, size_t blocklength);
+
+protected:
+	virtual void	sig_filename(std::string filename) { m_pOwner->signal_filename(filename); }
+	virtual void	sig_progress(double progress) { m_pOwner->signal_progress(progress); }
+	virtual void	sig_done(std::string filename, int available, int total) { m_pOwner->signal_done(filename, available, total); }
+
+	virtual bool	ScanDataFile(DiskFile *diskfile, Par2RepairerSourceFile* &sourcefile,
+		MatchType &matchtype, MD5Hash &hashfull, MD5Hash &hash16k, u32 &count);
+	virtual bool	RepairData(u32 inputindex, size_t blocklength);
 
 public:
-	Result		PreProcess(const char *szParFilename);
-	Result		Process(bool dorepair);
+					Repairer(ParChecker* pOwner) { m_pOwner = pOwner; }
+	Result			PreProcess(const char *szParFilename);
+	Result			Process(bool dorepair);
 
 	friend class ParChecker;
+	friend class RepairThread;
+};
+
+class RepairThread : public Thread
+{
+private:
+	Repairer*		m_pOwner;
+	u32				m_inputindex;
+	u32				m_outputindex;
+	size_t			m_blocklength;
+	volatile bool	m_bWorking;
+
+protected:
+	virtual void	Run();
+
+public:
+					RepairThread(Repairer* pOwner) { this->m_pOwner = pOwner; m_bWorking = false; }
+	void			RepairBlock(u32 inputindex, u32 outputindex, size_t blocklength);
+	bool			IsWorking() { return m_bWorking; }
 };
 
 Result Repairer::PreProcess(const char *szParFilename)
 {
-#ifdef HAVE_PAR2_BUGFIXES_V2
-	// Ensure linking against the patched version of libpar2
-	BugfixesPatchVersion2();
-#endif
+	char szMemParam[20];
+	snprintf(szMemParam, 20, "-m%i", g_pOptions->GetParBuffer());
+	szMemParam[20-1] = '\0';
 
 	if (g_pOptions->GetParScan() == Options::psFull)
 	{
@@ -98,16 +151,16 @@ Result Repairer::PreProcess(const char *szParFilename)
 			szBasename[1] = '\0';
 		}
 
-		const char* argv[] = { "par2", "r", "-v", "-v", szParFilename, szWildcardParam };
-		if (!commandLine.Parse(6, (char**)argv))
+		const char* argv[] = { "par2", "r", "-v", "-v", szMemParam, szParFilename, szWildcardParam };
+		if (!commandLine.Parse(7, (char**)argv))
 		{
 			return eInvalidCommandLineArguments;
 		}
 	}
 	else
 	{
-		const char* argv[] = { "par2", "r", "-v", "-v", szParFilename };
-		if (!commandLine.Parse(5, (char**)argv))
+		const char* argv[] = { "par2", "r", "-v", "-v", szMemParam, szParFilename };
+		if (!commandLine.Parse(6, (char**)argv))
 		{
 			return eInvalidCommandLineArguments;
 		}
@@ -122,6 +175,177 @@ Result Repairer::Process(bool dorepair)
 }
 
 
+bool Repairer::ScanDataFile(DiskFile *diskfile, Par2RepairerSourceFile* &sourcefile,
+	MatchType &matchtype, MD5Hash &hashfull, MD5Hash &hash16k, u32 &count)
+{
+	if (m_pOwner->GetParQuick() && sourcefile)
+	{
+		string path;
+		string name;
+		DiskFile::SplitFilename(diskfile->FileName(), path, name);
+
+		sig_filename(name);
+
+		int iAvailableBlocks = sourcefile->BlockCount();
+		ParChecker::EFileStatus eFileStatus = m_pOwner->VerifyDataFile(diskfile, sourcefile, &iAvailableBlocks);
+		if (eFileStatus != ParChecker::fsUnknown)
+		{
+			sig_done(name, iAvailableBlocks, sourcefile->BlockCount());
+			sig_progress(1000.0);
+			matchtype = eFileStatus == ParChecker::fsSuccess ? eFullMatch : ParChecker::fsPartial ? ePartialMatch : eNoMatch;
+			return true;
+		}
+	}
+
+	return Par2Repairer::ScanDataFile(diskfile, sourcefile, matchtype, hashfull, hash16k, count);
+}
+
+void Repairer::BeginRepair()
+{
+	int iMaxThreads = g_pOptions->GetParThreads() > 0 ? g_pOptions->GetParThreads() : Util::NumberOfCpuCores();
+	iMaxThreads = iMaxThreads > 0 ? iMaxThreads : 1;
+
+	int iThreads = iMaxThreads > (int)missingblockcount ? (int)missingblockcount : iMaxThreads;
+
+	m_pOwner->PrintMessage(Message::mkInfo, "Using %i of max %i thread(s) to repair %i block(s) for %s",
+		iThreads, iMaxThreads, (int)missingblockcount, m_pOwner->m_szNZBName);
+
+	m_bParallel = iThreads > 1;
+
+	if (m_bParallel)
+	{
+		for (int i = 0; i < iThreads; i++)
+		{
+			RepairThread* pRepairThread = new RepairThread(this);
+			m_Threads.push_back(pRepairThread);
+			pRepairThread->SetAutoDestroy(true);
+			pRepairThread->Start();
+		}
+
+#ifdef WIN32
+		timeBeginPeriod(1);
+#endif
+	}
+}
+
+void Repairer::EndRepair()
+{
+	if (m_bParallel)
+	{
+		for (Threads::iterator it = m_Threads.begin(); it != m_Threads.end(); it++)
+		{
+			RepairThread* pRepairThread = (RepairThread*)*it;
+			pRepairThread->Stop();
+		}
+
+#ifdef WIN32
+		timeEndPeriod(1);
+#endif
+	}
+}
+
+bool Repairer::RepairData(u32 inputindex, size_t blocklength)
+{
+	if (!m_bParallel)
+	{
+		return false;
+	}
+
+	for (u32 outputindex = 0; outputindex < missingblockcount; )
+	{
+		bool bJobAdded = false;
+		for (Threads::iterator it = m_Threads.begin(); it != m_Threads.end(); it++)
+		{
+			RepairThread* pRepairThread = (RepairThread*)*it;
+			if (!pRepairThread->IsWorking())
+			{
+				pRepairThread->RepairBlock(inputindex, outputindex, blocklength);
+				outputindex++;
+				bJobAdded = true;
+				break;
+			}
+		}
+
+		if (cancelled)
+		{
+			break;
+		}
+
+		if (!bJobAdded)
+		{
+			usleep(SYNC_SLEEP_INTERVAL);
+		}
+	}
+
+	// Wait until all m_Threads complete their jobs
+	bool bWorking = true;
+	while (bWorking)
+	{
+		bWorking = false;
+		for (Threads::iterator it = m_Threads.begin(); it != m_Threads.end(); it++)
+		{
+			RepairThread* pRepairThread = (RepairThread*)*it;
+			if (pRepairThread->IsWorking())
+			{
+				bWorking = true;
+				usleep(SYNC_SLEEP_INTERVAL);
+				break;
+			}
+		}
+	}
+
+	return true;
+}
+
+void Repairer::RepairBlock(u32 inputindex, u32 outputindex, size_t blocklength)
+{
+	// Select the appropriate part of the output buffer
+	void *outbuf = &((u8*)outputbuffer)[chunksize * outputindex];
+
+	// Process the data
+	rs.Process(blocklength, inputindex, inputbuffer, outputindex, outbuf);
+
+	if (noiselevel > CommandLine::nlQuiet)
+	{
+		// Update a progress indicator
+		progresslock.Lock();
+		u32 oldfraction = (u32)(1000 * progress / totaldata);
+		progress += blocklength;
+		u32 newfraction = (u32)(1000 * progress / totaldata);
+		progresslock.Unlock();
+
+		if (oldfraction != newfraction)
+		{
+			sig_progress(newfraction);
+		}
+	}
+}
+
+void RepairThread::Run()
+{
+	while (!IsStopped())
+	{
+		if (m_bWorking)
+		{
+			m_pOwner->RepairBlock(m_inputindex, m_outputindex, m_blocklength);
+			m_bWorking = false;
+		}
+		else
+		{
+			usleep(SYNC_SLEEP_INTERVAL);
+		}
+	}
+}
+
+void RepairThread::RepairBlock(u32 inputindex, u32 outputindex, size_t blocklength)
+{
+	m_inputindex = inputindex;
+	m_outputindex = outputindex;
+	m_blocklength = blocklength;
+	m_bWorking = true;
+}
+
+
 class MissingFilesComparator
 {
 private:
@@ -152,6 +376,24 @@ bool MissingFilesComparator::operator()(CommandLine::ExtraFile* pFile1, CommandL
 }
 
 
+ParChecker::Segment::Segment(bool bSuccess, long long iOffset, int iSize, unsigned long lCrc)
+{
+	m_bSuccess = bSuccess;
+	m_iOffset = iOffset;
+	m_iSize = iSize;
+	m_lCrc = lCrc;
+}
+
+
+ParChecker::SegmentList::~SegmentList()
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		delete *it;
+	}
+}
+
+
 ParChecker::ParChecker()
 {
     debug("Creating ParChecker");
@@ -170,6 +412,8 @@ ParChecker::ParChecker()
 	m_bVerifyingExtraFiles = false;
 	m_bCancelled = false;
 	m_eStage = ptLoadingPars;
+	m_bParQuick = false;
+	m_bForceRepair = false;
 }
 
 ParChecker::~ParChecker()
@@ -226,17 +470,29 @@ void ParChecker::SetInfoName(const char * szInfoName)
 }
 
 void ParChecker::Run()
+{
+	m_eStatus = RunParCheckAll();
+
+	if (m_eStatus == psRepairNotNeeded && m_bParQuick && m_bForceRepair && !m_bCancelled)
+	{
+		PrintMessage(Message::mkInfo, "Performing full par-check for %s", m_szNZBName);
+		m_bParQuick = false;
+		m_eStatus = RunParCheckAll();
+	}
+
+	Completed();
+}
+
+ParChecker::EStatus ParChecker::RunParCheckAll()
 {
 	ParCoordinator::ParFileList fileList;
 	if (!ParCoordinator::FindMainPars(m_szDestDir, &fileList))
 	{
 		PrintMessage(Message::mkError, "Could not start par-check for %s. Could not find any par-files", m_szNZBName);
-		m_eStatus = psFailed;
-		Completed();
-		return;
+		return psFailed;
 	}
 
-	m_eStatus = psRepairNotNeeded;
+	EStatus eAllStatus = psRepairNotNeeded;
 	m_bCancelled = false;
 
 	for (ParCoordinator::ParFileList::iterator it = fileList.begin(); it != fileList.end(); it++)
@@ -266,9 +522,9 @@ void ParChecker::Run()
 			EStatus eStatus = RunParCheck(szFullParFilename);
 
 			// accumulate total status, the worst status has priority
-			if (m_eStatus > eStatus)
+			if (eAllStatus > eStatus)
 			{
-				m_eStatus = eStatus;
+				eAllStatus = eStatus;
 			}
 
 			if (g_pOptions->GetCreateBrokenLog())
@@ -280,7 +536,7 @@ void ParChecker::Run()
 		free(szParFilename);
 	}
 
-	Completed();
+	return eAllStatus;
 }
 
 ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
@@ -291,6 +547,7 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 	m_iProcessedFiles = 0;
 	m_iExtraFiles = 0;
 	m_bVerifyingExtraFiles = false;
+	m_bHasDamagedFiles = false;
 	EStatus eStatus = psFailed;
 
 	PrintMessage(Message::mkInfo, "Verifying %s", m_szInfoName);
@@ -316,7 +573,7 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
     debug("ParChecker: Process-result=%i", res);
 
 	bool bAddedSplittedFragments = false;
-	if (!IsStopped() && res == eRepairNotPossible)
+	if (m_bHasDamagedFiles && !IsStopped() && res == eRepairNotPossible)
 	{
 		bAddedSplittedFragments = AddSplittedFragments();
 		if (bAddedSplittedFragments)
@@ -326,7 +583,7 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 		}
 	}
 
-	if (!IsStopped() && pRepairer->missingfilecount > 0 && 
+	if (m_bHasDamagedFiles && !IsStopped() && pRepairer->missingfilecount > 0 && 
 		!(bAddedSplittedFragments && res == eRepairPossible) &&
 		g_pOptions->GetParScan() == Options::psAuto)
 	{
@@ -337,7 +594,7 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 		}
 	}
 
-	if (!IsStopped() && res == eRepairNotPossible)
+	if (m_bHasDamagedFiles && !IsStopped() && res == eRepairNotPossible)
 	{
 		res = (Result)ProcessMorePars();
 	}
@@ -350,7 +607,7 @@ ParChecker::EStatus ParChecker::RunParCheck(const char* szParFilename)
 	
 	eStatus = psFailed;
 	
-	if (res == eSuccess)
+	if (res == eSuccess || !m_bHasDamagedFiles)
 	{
     	PrintMessage(Message::mkInfo, "Repair not needed for %s", m_szInfoName);
 		eStatus = psRepairNotNeeded;
@@ -422,11 +679,8 @@ int ParChecker::PreProcessPar()
 	{
 		Cleanup();
 
-		Repairer* pRepairer = new Repairer();
+		Repairer* pRepairer = new Repairer(this);
 		m_pRepairer = pRepairer;
-		pRepairer->sig_filename.connect(sigc::mem_fun(*this, &ParChecker::signal_filename));
-		pRepairer->sig_progress.connect(sigc::mem_fun(*this, &ParChecker::signal_progress));
-		pRepairer->sig_done.connect(sigc::mem_fun(*this, &ParChecker::signal_done));
 
 		res = pRepairer->PreProcess(m_szParFilename);
 		debug("ParChecker: PreProcess-result=%i", res);
@@ -811,14 +1065,26 @@ bool ParChecker::IsProcessedFile(const char* szFilename)
 
 void ParChecker::signal_filename(std::string str)
 {
-    const char* szStageMessage[] = { "Loading file", "Verifying file", "Repairing file", "Verifying repaired file" };
+	if (!m_lastFilename.compare(str))
+	{
+		return;
+	}
+
+	m_lastFilename = str;
+
+	const char* szStageMessage[] = { "Loading file", "Verifying file", "Repairing file", "Verifying repaired file" };
 
 	if (m_eStage == ptRepairing)
 	{
 		m_eStage = ptVerifyingRepaired;
 	}
 
-	PrintMessage(Message::mkInfo, "%s %s", szStageMessage[m_eStage], str.c_str());
+	// don't print progress messages when verifying repaired files in quick verification mode,
+	// because repaired files are not verified in this mode
+	if (!(m_eStage == ptVerifyingRepaired && m_bParQuick))
+	{
+		PrintMessage(Message::mkInfo, "%s %s", szStageMessage[m_eStage], str.c_str());
+	}
 
 	if (m_eStage == ptLoadingPars || m_eStage == ptVerifyingSources)
 	{
@@ -886,13 +1152,14 @@ void ParChecker::signal_done(std::string str, int available, int total)
 	{
 		if (available < total && !m_bVerifyingExtraFiles)
 		{
-			bool bFileExists = true;
+			const char* szFilename = str.c_str();
 
+			bool bFileExists = true;
 			for (std::vector<Par2RepairerSourceFile*>::iterator it = ((Repairer*)m_pRepairer)->sourcefiles.begin();
 				it != ((Repairer*)m_pRepairer)->sourcefiles.end(); it++)
 			{
 				Par2RepairerSourceFile *sourcefile = *it;
-				if (sourcefile && !strcmp(str.c_str(), Util::BaseFileName(sourcefile->TargetFileName().c_str())) &&
+				if (sourcefile && !strcmp(szFilename, Util::BaseFileName(sourcefile->TargetFileName().c_str())) &&
 					!sourcefile->GetTargetExists())
 				{
 					bFileExists = false;
@@ -900,13 +1167,19 @@ void ParChecker::signal_done(std::string str, int available, int total)
 				}
 			}
 
+			bool bIgnore = Util::MatchFileExt(szFilename, g_pOptions->GetParIgnoreExt(), ",;") ||
+				Util::MatchFileExt(szFilename, g_pOptions->GetExtCleanupDisk(), ",;");
+			m_bHasDamagedFiles |= !bIgnore;
+
 			if (bFileExists)
 			{
-				PrintMessage(Message::mkWarning, "File %s has %i bad block(s) of total %i block(s)", str.c_str(), total - available, total);
+				PrintMessage(Message::mkWarning, "File %s has %i bad block(s) of total %i block(s)%s",
+					szFilename, total - available, total, bIgnore ? ", ignoring" : "");
 			}
 			else
 			{
-				PrintMessage(Message::mkWarning, "File %s with %i block(s) is missing", str.c_str(), total);
+				PrintMessage(Message::mkWarning, "File %s with %i block(s) is missing%s",
+					szFilename, total, bIgnore ? ", ignoring" : "");
 			}
 		}
 	}
@@ -914,13 +1187,9 @@ void ParChecker::signal_done(std::string str, int available, int total)
 
 void ParChecker::Cancel()
 {
-#ifdef HAVE_PAR2_CANCEL
 	((Repairer*)m_pRepairer)->cancelled = true;
 	m_bCancelled = true;
 	QueueChanged();
-#else
-	PrintMessage(Message::mkError, "Could not cancel par-repair. The program was compiled using version of libpar2 which doesn't support cancelling of par-repair. Please apply libpar2-patches supplied with NZBGet and recompile libpar2 and NZBGet (see README for details).");
-#endif
 }
 
 void ParChecker::WriteBrokenLog(EStatus eStatus)
@@ -1018,4 +1287,324 @@ void ParChecker::DeleteLeftovers()
 	}
 }
 
+/**
+ * This function implements quick par verification replacing the standard verification routine
+ * from libpar2:
+ * - for successfully downloaded files the function compares CRC of the file computed during
+ *   download with CRC stored in PAR2-file;
+ * - for partially downloaded files the CRCs of articles are compared with block-CRCs stored
+ *   in PAR2-file;
+ * - for completely failed files (not a single successful artice) no verification is needed at all.
+ *
+ * Limitation of the function:
+ * This function requires every block in the file to have an unique CRC (across all blocks
+ * of the par-set). Otherwise the full verification is performed.
+ * The limitation can be avoided by using something more smart than "verificationhashtable.Lookup"
+ * but in the real life all blocks have unique CRCs and the simple "Lookup" works good enough.
+ */
+ParChecker::EFileStatus ParChecker::VerifyDataFile(void* pDiskfile, void* pSourcefile, int* pAvailableBlocks)
+{
+	if (m_eStage != ptVerifyingSources)
+	{
+		// skipping verification for repaired files, assuming the files were correctly repaired,
+		// the only reason for incorrect files after repair are hardware errors (memory, disk),
+		// but this isn't something NZBGet should care about.
+		return fsSuccess;
+	}
+
+	DiskFile* pDiskFile = (DiskFile*)pDiskfile;
+	Par2RepairerSourceFile* pSourceFile = (Par2RepairerSourceFile*)pSourcefile;
+	if (!pSourcefile || !pSourceFile->GetTargetExists())
+	{
+		return fsUnknown;
+	}
+
+	VerificationPacket* packet = pSourceFile->GetVerificationPacket();
+	if (!packet)
+	{
+		return fsUnknown;
+	}
+
+	std::string filename = pSourceFile->GetTargetFile()->FileName();
+	const char* szFilename = filename.c_str();
+
+	if (Util::FileSize(szFilename) == 0 && pSourceFile->BlockCount() > 0)
+	{
+		*pAvailableBlocks = 0;
+		return fsFailure;
+	}
+
+	// find file status and CRC computed during download
+	unsigned long lDownloadCrc;
+	SegmentList segments;
+	EFileStatus	eFileStatus = FindFileCrc(Util::BaseFileName(szFilename), &lDownloadCrc, &segments);
+	ValidBlocks validBlocks;
+
+	if (eFileStatus == fsFailure || eFileStatus == fsUnknown)
+	{
+		return eFileStatus;
+	}
+	else if ((eFileStatus == fsSuccess && !VerifySuccessDataFile(pDiskfile, pSourcefile, lDownloadCrc)) ||
+		(eFileStatus == fsPartial && !VerifyPartialDataFile(pDiskfile, pSourcefile, &segments, &validBlocks)))
+	{
+		PrintMessage(Message::mkWarning, "Quick verification failed for %s file %s, performing full verification instead",
+			eFileStatus == fsSuccess ? "good" : "damaged", Util::BaseFileName(szFilename));
+		return fsUnknown; // let libpar2 do the full verification of the file
+	}
+
+	// attach verification blocks to the file
+	*pAvailableBlocks = 0;
+	u64 blocksize = ((Repairer*)m_pRepairer)->mainpacket->BlockSize();
+	std::deque<const VerificationHashEntry*> undoList;
+	for (unsigned int i = 0; i < packet->BlockCount(); i++)
+	{
+		if (eFileStatus == fsSuccess || validBlocks.at(i))
+		{
+			const FILEVERIFICATIONENTRY* entry = packet->VerificationEntry(i);
+			u32 blockCrc = entry->crc;
+
+			// Look for a match
+			const VerificationHashEntry* pHashEntry = ((Repairer*)m_pRepairer)->verificationhashtable.Lookup(blockCrc);
+			if (!pHashEntry || pHashEntry->SourceFile() != pSourceFile || pHashEntry->IsSet())
+			{
+				// no match found, revert back the changes made by "pHashEntry->SetBlock"
+				for (std::deque<const VerificationHashEntry*>::iterator it = undoList.begin(); it != undoList.end(); it++)
+				{
+					const VerificationHashEntry* pUndoEntry = *it;
+					pUndoEntry->SetBlock(NULL, 0);
+				}
+				return fsUnknown;
+			}
+
+			undoList.push_back(pHashEntry);
+			pHashEntry->SetBlock(pDiskFile, i*blocksize);
+			(*pAvailableBlocks)++;
+		}
+	}
+
+	PrintMessage(Message::mkDetail, "Quickly verified %s file %s",
+		eFileStatus == fsSuccess ? "good" : "damaged", Util::BaseFileName(szFilename));
+
+	return eFileStatus;
+}
+
+bool ParChecker::VerifySuccessDataFile(void* pDiskfile, void* pSourcefile, unsigned long lDownloadCrc)
+{
+	Par2RepairerSourceFile* pSourceFile = (Par2RepairerSourceFile*)pSourcefile;
+	u64 blocksize = ((Repairer*)m_pRepairer)->mainpacket->BlockSize();
+	VerificationPacket* packet = pSourceFile->GetVerificationPacket();
+
+	// extend lDownloadCrc to block size
+	lDownloadCrc = CRCUpdateBlock(lDownloadCrc ^ 0xFFFFFFFF,
+		(size_t)(blocksize * packet->BlockCount() > pSourceFile->GetTargetFile()->FileSize() ?
+			blocksize * packet->BlockCount() - pSourceFile->GetTargetFile()->FileSize() : 0)
+		) ^ 0xFFFFFFFF;
+	debug("Download-CRC: %.8x", lDownloadCrc);
+
+	// compute file CRC using CRCs of blocks
+	unsigned long lParCrc = 0;
+	for (unsigned int i = 0; i < packet->BlockCount(); i++)
+	{
+		const FILEVERIFICATIONENTRY* entry = packet->VerificationEntry(i);
+		u32 blockCrc = entry->crc;
+		lParCrc = i == 0 ? blockCrc : Util::Crc32Combine(lParCrc, blockCrc, (unsigned long)blocksize);
+	}
+	debug("Block-CRC: %x, filename: %s", lParCrc, Util::BaseFileName(pSourceFile->GetTargetFile()->FileName().c_str()));
+
+	return lParCrc == lDownloadCrc;
+}
+
+bool ParChecker::VerifyPartialDataFile(void* pDiskfile, void* pSourcefile, SegmentList* pSegments, ValidBlocks* pValidBlocks)
+{
+	Par2RepairerSourceFile* pSourceFile = (Par2RepairerSourceFile*)pSourcefile;
+	VerificationPacket* packet = pSourceFile->GetVerificationPacket();
+	long long blocksize = ((Repairer*)m_pRepairer)->mainpacket->BlockSize();
+	std::string filename = pSourceFile->GetTargetFile()->FileName();
+	const char* szFilename = filename.c_str();
+	long long iFileSize = pSourceFile->GetTargetFile()->FileSize();
+
+	// determine presumably valid and bad blocks based on article download status
+	pValidBlocks->resize(packet->BlockCount(), false);
+	for (int i = 0; i < (int)pValidBlocks->size(); i++)
+	{
+		long long blockStart = i * blocksize;
+		long long blockEnd = blockStart + blocksize < iFileSize - 1 ? blockStart + blocksize : iFileSize - 1;
+		bool bBlockOK = false;
+		bool bBlockEnd = false;
+		u64 iCurOffset = 0;
+		for (SegmentList::iterator it = pSegments->begin(); it != pSegments->end(); it++)
+		{
+			Segment* pSegment = *it;
+			if (!bBlockOK && pSegment->GetSuccess() && pSegment->GetOffset() <= blockStart &&
+				pSegment->GetOffset() + pSegment->GetSize() >= blockStart)
+			{
+				bBlockOK = true;
+				iCurOffset = pSegment->GetOffset();
+			}
+			if (bBlockOK)
+			{
+				if (!(pSegment->GetSuccess() && pSegment->GetOffset() == iCurOffset))
+				{
+					bBlockOK = false;
+					break;
+				}
+				if (pSegment->GetOffset() + pSegment->GetSize() >= blockEnd)
+				{
+					bBlockEnd = true;
+					break;
+				}
+				iCurOffset = pSegment->GetOffset() + pSegment->GetSize();
+			}
+		}
+		pValidBlocks->at(i) = bBlockOK && bBlockEnd;
+	}
+
+	char szErrBuf[256];
+	FILE* infile = fopen(szFilename, FOPEN_RB);
+	if (!infile)
+	{
+		error("Could not open file %s: %s", szFilename, Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf)));
+	}
+
+	// For each sequential range of presumably valid blocks:
+	// - compute par-CRC of the range of blocks using block CRCs;
+	// - compute download-CRC for the same byte range using CRCs of articles; if articles and block
+	//   overlap - read a little bit of data from the file and calculate its CRC;
+	// - compare two CRCs - they must match; if not - the file is more damaged than we thought -
+	//   let libpar2 do the full verification of the file in this case.
+	unsigned long lParCrc = 0;
+	int iBlockStart = -1;
+	pValidBlocks->push_back(false); // end marker
+	for (int i = 0; i < (int)pValidBlocks->size(); i++)
+	{
+		bool bValidBlock = pValidBlocks->at(i);
+		if (bValidBlock)
+		{
+			if (iBlockStart == -1)
+			{
+				iBlockStart = i;
+			}
+			const FILEVERIFICATIONENTRY* entry = packet->VerificationEntry(i);
+			u32 blockCrc = entry->crc;
+			lParCrc = iBlockStart == i ? blockCrc : Util::Crc32Combine(lParCrc, blockCrc, (unsigned long)blocksize);
+		}
+		else
+		{
+			if (iBlockStart > -1)
+			{
+				int iBlockEnd = i - 1;
+				long long iBytesStart = iBlockStart * blocksize;
+				long long iBytesEnd = iBlockEnd * blocksize + blocksize - 1;
+				unsigned long lDownloadCrc = 0;
+				bool bOK = SmartCalcFileRangeCrc(infile, iBytesStart,
+					iBytesEnd < iFileSize - 1 ? iBytesEnd : iFileSize - 1, pSegments, &lDownloadCrc);
+				if (bOK && iBytesEnd > iFileSize - 1)
+				{
+					// for the last block: extend lDownloadCrc to block size
+					lDownloadCrc = CRCUpdateBlock(lDownloadCrc ^ 0xFFFFFFFF, (size_t)(iBytesEnd - (iFileSize - 1))) ^ 0xFFFFFFFF;
+				}
+
+				if (!bOK || lDownloadCrc != lParCrc)
+				{
+					fclose(infile);
+					return false;
+				}
+			}
+			iBlockStart = -1;
+		}
+	}
+
+	fclose(infile);
+
+	return true;
+}
+
+/*
+ * Compute CRC of bytes range of file using CRCs of segments and reading some data directly
+ * from file if necessary
+ */
+bool ParChecker::SmartCalcFileRangeCrc(FILE* pFile, long long lStart, long long lEnd, SegmentList* pSegments,
+	unsigned long* pDownloadCrc)
+{
+	unsigned long lDownloadCrc = 0;
+	bool bStarted = false;
+	for (SegmentList::iterator it = pSegments->begin(); it != pSegments->end(); it++)
+	{
+		Segment* pSegment = *it;
+
+		if (!bStarted && pSegment->GetOffset() > lStart)
+		{
+			// read start of range from file
+			if (!DumbCalcFileRangeCrc(pFile, lStart, pSegment->GetOffset() - 1, &lDownloadCrc))
+			{
+				return false;
+			}
+			if (pSegment->GetOffset() + pSegment->GetSize() >= lEnd)
+			{
+				break;
+			}
+			bStarted = true;
+		}
+
+		if (pSegment->GetOffset() >= lStart && pSegment->GetOffset() + pSegment->GetSize() <= lEnd)
+		{
+			lDownloadCrc = !bStarted ? pSegment->GetCrc() : Util::Crc32Combine(lDownloadCrc, pSegment->GetCrc(), (unsigned long)pSegment->GetSize());
+			bStarted = true;
+		}
+
+		if (pSegment->GetOffset() + pSegment->GetSize() == lEnd)
+		{
+			break;
+		}
+
+		if (pSegment->GetOffset() + pSegment->GetSize() > lEnd)
+		{
+			// read end of range from file
+			unsigned long lPartialCrc = 0;
+			if (!DumbCalcFileRangeCrc(pFile, pSegment->GetOffset(), lEnd, &lPartialCrc))
+			{
+				return false;
+			}
+
+			lDownloadCrc = Util::Crc32Combine(lDownloadCrc, (unsigned long)lPartialCrc, (unsigned long)(lEnd - pSegment->GetOffset() + 1));
+
+			break;
+		}
+	}
+
+	*pDownloadCrc = lDownloadCrc;
+	return true;
+}
+
+/*
+ * Compute CRC of bytes range of file reading the data directly from file
+ */
+bool ParChecker::DumbCalcFileRangeCrc(FILE* pFile, long long lStart, long long lEnd, unsigned long* pDownloadCrc)
+{
+	if (fseek(pFile, lStart, SEEK_SET))
+	{
+		return false;
+	}
+
+	static const int BUFFER_SIZE = 1024 * 64;
+	unsigned char* buffer = (unsigned char*)malloc(BUFFER_SIZE);
+	unsigned long lDownloadCrc = 0xFFFFFFFF;
+
+	int cnt = BUFFER_SIZE;
+	while (cnt == BUFFER_SIZE && lStart < lEnd)
+	{
+		int iNeedBytes = lEnd - lStart + 1 > BUFFER_SIZE ? BUFFER_SIZE : (int)(lEnd - lStart + 1);
+		cnt = (int)fread(buffer, 1, iNeedBytes, pFile);
+		lDownloadCrc = Util::Crc32m(lDownloadCrc, buffer, cnt);
+		lStart += cnt;
+	}
+
+	free(buffer);
+
+	lDownloadCrc ^= 0xFFFFFFFF;
+
+	*pDownloadCrc = lDownloadCrc;
+	return true;
+}
+
 #endif
