@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1039 $
- * $Date: 2014-06-06 23:49:11 +0200 (Fri, 06 Jun 2014) $
+ * $Revision: 1139 $
+ * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
  *
  */
 
@@ -45,10 +45,12 @@
 #include "nzbget.h"
 #include "ParCoordinator.h"
 #include "Options.h"
+#include "DiskState.h"
 #include "Log.h"
 #include "Util.h"
 
 extern Options* g_pOptions;
+extern DiskState* g_pDiskState;
 
 #ifndef DISABLE_PARCHECK
 bool ParCoordinator::PostParChecker::RequestMorePars(int iBlockNeeded, int* pBlockFound)
@@ -91,6 +93,59 @@ bool ParCoordinator::PostParChecker::IsParredFile(const char* szFilename)
 	return false;
 }
 
+ParChecker::EFileStatus ParCoordinator::PostParChecker::FindFileCrc(const char* szFilename,
+	unsigned long* lCrc, SegmentList* pSegments)
+{
+	CompletedFile* pCompletedFile = NULL;
+
+	for (CompletedFiles::iterator it = m_pPostInfo->GetNZBInfo()->GetCompletedFiles()->begin(); it != m_pPostInfo->GetNZBInfo()->GetCompletedFiles()->end(); it++)
+	{
+		CompletedFile* pCompletedFile2 = *it;
+		if (!strcasecmp(pCompletedFile2->GetFileName(), szFilename))
+		{
+			pCompletedFile = pCompletedFile2;
+			break;
+		}
+	}
+	if (!pCompletedFile)
+	{
+		return ParChecker::fsUnknown;
+	}
+
+	debug("Found completed file: %s, CRC: %.8x, Status: %i", Util::BaseFileName(pCompletedFile->GetFileName()), pCompletedFile->GetCrc(), (int)pCompletedFile->GetStatus());
+
+	*lCrc = pCompletedFile->GetCrc();
+
+	if (pCompletedFile->GetStatus() == CompletedFile::cfPartial && pCompletedFile->GetID() > 0 &&
+		!m_pPostInfo->GetNZBInfo()->GetReprocess())
+	{
+		FileInfo* pTmpFileInfo = new FileInfo(pCompletedFile->GetID());
+
+		if (!g_pDiskState->LoadFileState(pTmpFileInfo, NULL, true))
+		{
+			delete pTmpFileInfo;
+			return ParChecker::fsUnknown;
+		}
+
+		for (FileInfo::Articles::iterator it = pTmpFileInfo->GetArticles()->begin(); it != pTmpFileInfo->GetArticles()->end(); it++)
+		{
+			ArticleInfo* pa = *it;
+			ParChecker::Segment* pSegment = new Segment(pa->GetStatus() == ArticleInfo::aiFinished,
+				pa->GetSegmentOffset(), pa->GetSegmentSize(), pa->GetCrc());
+			pSegments->push_back(pSegment);
+		}
+
+		delete pTmpFileInfo;
+	}
+
+	return pCompletedFile->GetStatus() == CompletedFile::cfSuccess ? ParChecker::fsSuccess :
+		pCompletedFile->GetStatus() == CompletedFile::cfFailure &&
+			!m_pPostInfo->GetNZBInfo()->GetReprocess() ? ParChecker::fsFailure :
+		pCompletedFile->GetStatus() == CompletedFile::cfPartial && pSegments->size() > 0 &&
+			!m_pPostInfo->GetNZBInfo()->GetReprocess()? ParChecker::fsPartial :
+		ParChecker::fsUnknown;
+}
+
 void ParCoordinator::PostParRenamer::UpdateProgress()
 {
 	m_pOwner->UpdateParRenameProgress();
@@ -113,6 +168,22 @@ void ParCoordinator::PostParRenamer::RegisterParredFile(const char* szFilename)
 	m_pPostInfo->GetParredFiles()->push_back(strdup(szFilename));
 }
 
+/**
+ *  Update file name in the CompletedFiles-list of NZBInfo
+ */
+void ParCoordinator::PostParRenamer::RegisterRenamedFile(const char* szOldFilename, const char* szNewFileName)
+{
+	for (CompletedFiles::iterator it = m_pPostInfo->GetNZBInfo()->GetCompletedFiles()->begin(); it != m_pPostInfo->GetNZBInfo()->GetCompletedFiles()->end(); it++)
+	{
+		CompletedFile* pCompletedFile = *it;
+		if (!strcasecmp(pCompletedFile->GetFileName(), szOldFilename))
+		{
+			pCompletedFile->SetFileName(szNewFileName);
+			break;
+		}
+	}
+}
+
 #endif
 
 ParCoordinator::ParCoordinator()
@@ -280,6 +351,10 @@ void ParCoordinator::StartParCheckJob(PostInfo* pPostInfo)
 	m_ParChecker.SetPostInfo(pPostInfo);
 	m_ParChecker.SetDestDir(pPostInfo->GetNZBInfo()->GetDestDir());
 	m_ParChecker.SetNZBName(pPostInfo->GetNZBInfo()->GetName());
+	m_ParChecker.SetParTime(time(NULL));
+	m_ParChecker.SetDownloadSec(pPostInfo->GetNZBInfo()->GetDownloadSec());
+	m_ParChecker.SetParQuick(g_pOptions->GetParQuick() && !pPostInfo->GetForceParFull());
+	m_ParChecker.SetForceRepair(pPostInfo->GetForceRepair());
 	m_ParChecker.PrintMessage(Message::mkInfo, "Checking pars for %s", pPostInfo->GetNZBInfo()->GetName());
 	pPostInfo->SetWorking(true);
 	m_ParChecker.Start();
@@ -314,16 +389,12 @@ bool ParCoordinator::Cancel()
 {
 	if (m_eCurrentJob == jkParCheck)
 	{
-#ifdef HAVE_PAR2_CANCEL
 		if (!m_ParChecker.GetCancelled())
 		{
 			debug("Cancelling par-repair for %s", m_ParChecker.GetInfoName());
 			m_ParChecker.Cancel();
 			return true;
 		}
-#else
-		warn("Cannot cancel par-repair for %s, used version of libpar2 does not support cancelling", m_ParChecker.GetInfoName());
-#endif
 	}
 	else if (m_eCurrentJob == jkParRename)
 	{
@@ -381,6 +452,13 @@ void ParCoordinator::ParCheckCompleted()
 		pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psFailure);
 	}
 
+	int iWaitTime = pPostInfo->GetNZBInfo()->GetDownloadSec() - m_ParChecker.GetDownloadSec();
+	pPostInfo->SetStartTime(pPostInfo->GetStartTime() + (time_t)iWaitTime);
+	int iParSec = (int)(time(NULL) - m_ParChecker.GetParTime()) - iWaitTime;
+	pPostInfo->GetNZBInfo()->SetParSec(pPostInfo->GetNZBInfo()->GetParSec() + iParSec);
+
+	pPostInfo->GetNZBInfo()->SetParFull(!m_ParChecker.GetParQuick());
+
 	pPostInfo->SetWorking(false);
 	pPostInfo->SetStage(PostInfo::ptQueued);
 
@@ -587,19 +665,22 @@ void ParCoordinator::UpdateParCheckProgress()
 	PostInfo::EStage eStage = StageKind[m_ParChecker.GetStage()];
 	time_t tCurrent = time(NULL);
 
-	if (!pPostInfo->GetStartTime())
-	{
-		pPostInfo->SetStartTime(tCurrent);
-	}
-
 	if (pPostInfo->GetStage() != eStage)
 	{
 		pPostInfo->SetStage(eStage);
 		pPostInfo->SetStageTime(tCurrent);
+		if (pPostInfo->GetStage() == PostInfo::ptRepairing)
+		{
+			m_ParChecker.SetRepairTime(tCurrent);
+		}
+		else if (pPostInfo->GetStage() == PostInfo::ptVerifyingRepaired)
+		{
+			int iRepairSec = (int)(tCurrent - m_ParChecker.GetRepairTime());
+			pPostInfo->GetNZBInfo()->SetRepairSec(pPostInfo->GetNZBInfo()->GetRepairSec() + iRepairSec);
+		}
 	}
 
 	bool bParCancel = false;
-#ifdef HAVE_PAR2_CANCEL
 	if (!m_ParChecker.GetCancelled())
 	{
 		if ((g_pOptions->GetParTimeLimit() > 0) &&
@@ -618,7 +699,6 @@ void ParCoordinator::UpdateParCheckProgress()
 			}
 		}
 	}
-#endif
 
 	if (bParCancel)
 	{
@@ -636,12 +716,14 @@ void ParCoordinator::CheckPauseState(PostInfo* pPostInfo)
 	{
 		time_t tStageTime = pPostInfo->GetStageTime();
 		time_t tStartTime = pPostInfo->GetStartTime();
+		time_t tParTime = m_ParChecker.GetParTime();
+		time_t tRepairTime = m_ParChecker.GetRepairTime();
 		time_t tWaitTime = time(NULL);
 		
 		// wait until Post-processor is unpaused
 		while (g_pOptions->GetPausePostProcess() && !pPostInfo->GetNZBInfo()->GetForcePriority() && !m_bStopped)
 		{
-			usleep(100 * 1000);
+			usleep(50 * 1000);
 			
 			// update time stamps
 			
@@ -651,11 +733,18 @@ void ParCoordinator::CheckPauseState(PostInfo* pPostInfo)
 			{
 				pPostInfo->SetStageTime(tStageTime + tDelta);
 			}
-			
 			if (tStartTime > 0)
 			{
 				pPostInfo->SetStartTime(tStartTime + tDelta);
 			}
+			if (tParTime > 0)
+			{
+				m_ParChecker.SetParTime(tParTime + tDelta);
+			}
+			if (tRepairTime > 0)
+			{
+				m_ParChecker.SetRepairTime(tRepairTime + tDelta);
+			}
 		}
 	}
 }
@@ -672,11 +761,6 @@ void ParCoordinator::ParRenameCompleted()
 		PrintMessage(pPostInfo, Message::mkInfo, "Requesting par-check/repair for %s to restore missing files ", m_ParRenamer.GetInfoName());
 		pPostInfo->SetRequestParCheck(true);
 	}
-	else if (m_ParRenamer.HasSplittedFragments() && pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped)
-	{
-		PrintMessage(pPostInfo, Message::mkInfo, "Requesting par-check/repair for %s to join splitted fragments", m_ParRenamer.GetInfoName());
-		pPostInfo->SetRequestParCheck(true);
-	}
 
 	pPostInfo->SetWorking(false);
 	pPostInfo->SetStage(PostInfo::ptQueued);
@@ -695,11 +779,6 @@ void ParCoordinator::UpdateParRenameProgress()
 	pPostInfo->SetStageProgress(m_ParRenamer.GetStageProgress());
 	time_t tCurrent = time(NULL);
 	
-	if (!pPostInfo->GetStartTime())
-	{
-		pPostInfo->SetStartTime(tCurrent);
-	}
-	
 	if (pPostInfo->GetStage() != PostInfo::ptRenaming)
 	{
 		pPostInfo->SetStage(PostInfo::ptRenaming);
