@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1039 $
- * $Date: 2014-06-06 23:49:11 +0200 (Fri, 06 Jun 2014) $
+ * $Revision: 1126 $
+ * $Date: 2014-09-17 00:15:02 +0200 (Wed, 17 Sep 2014) $
  *
  */
 
@@ -37,17 +37,14 @@
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
-#ifdef WIN32
-#include <par2cmdline.h>
-#include <par2repairer.h>
-#include <md5.h>
-#else
+#ifndef WIN32
 #include <unistd.h>
-#include <libpar2/par2cmdline.h>
-#include <libpar2/par2repairer.h>
-#include <libpar2/md5.h>
 #endif
 
+#include "par2cmdline.h"
+#include "par2repairer.h"
+#include "md5.h"
+
 #include "nzbget.h"
 #include "ParRenamer.h"
 #include "ParCoordinator.h"
@@ -86,7 +83,6 @@ ParRenamer::ParRenamer()
 	m_szProgressLabel = (char*)malloc(1024);
 	m_iStageProgress = 0;
 	m_bCancelled = false;
-	m_bHasSplittedFragments = false;
 	m_bHasMissedFiles = false;
 	m_bDetectMissing = false;
 }
@@ -146,7 +142,6 @@ void ParRenamer::Run()
 	m_iFileCount = 0;
 	m_iCurFile = 0;
 	m_iRenamedCount = 0;
-	m_bHasSplittedFragments = false;
 	m_bHasMissedFiles = false;
 	m_eStatus = psFailed;
 
@@ -344,12 +339,10 @@ bool ParRenamer::IsSplittedFragment(const char* szFilename, const char* szCorrec
 		{
 			for (p++; *p && strchr("0123456789", *p); p++) ;
 			bSplittedFragement = !*p;
-			bSplittedFragement = bSplittedFragement && atoi(szDiskBasename + iBaseLen + 1) == 1;
+			bSplittedFragement = bSplittedFragement && atoi(szDiskBasename + iBaseLen + 1) <= 1; // .000 or .001
 		}
 	}
 
-	m_bHasSplittedFragments = m_bHasSplittedFragments || bSplittedFragement;
-
 	return bSplittedFragement;
 }
 
@@ -403,17 +396,7 @@ void ParRenamer::CheckRegularFile(const char* szDestDir, const char* szFilename)
 
 			if (!Util::FileExists(szDstFilename) && !IsSplittedFragment(szFilename, pFileHash->GetFilename()))
 			{
-				PrintMessage(Message::mkInfo, "Renaming %s to %s", Util::BaseFileName(szFilename), pFileHash->GetFilename());
-				if (Util::MoveFile(szFilename, szDstFilename))
-				{
-					m_iRenamedCount++;
-				}
-				else
-				{
-					char szErrBuf[256];
-					PrintMessage(Message::mkError, "Could not rename %s to %s: %s", szFilename, szDstFilename,
-						Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf)));
-				}
+				RenameFile(szFilename, szDstFilename);
 			}
 			
 			break;
@@ -483,17 +466,24 @@ void ParRenamer::CheckParFile(const char* szDestDir, const char* szFilename)
 		iNum++;
 	}
 
-	PrintMessage(Message::mkInfo, "Renaming %s to %s", Util::BaseFileName(szFilename), Util::BaseFileName(szDestFileName));
-	if (Util::MoveFile(szFilename, szDestFileName))
-	{
-		m_iRenamedCount++;
-	}
-	else
+	RenameFile(szFilename, szDestFileName);
+}
+
+void ParRenamer::RenameFile(const char* szSrcFilename, const char* szDestFileName)
+{
+	PrintMessage(Message::mkInfo, "Renaming %s to %s", Util::BaseFileName(szSrcFilename), Util::BaseFileName(szDestFileName));
+	if (!Util::MoveFile(szSrcFilename, szDestFileName))
 	{
 		char szErrBuf[256];
-		PrintMessage(Message::mkError, "Could not rename %s to %s: %s", szFilename, szDestFileName,
+		PrintMessage(Message::mkError, "Could not rename %s to %s: %s", szSrcFilename, szDestFileName,
 			Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf)));
+		return;
 	}
+
+	m_iRenamedCount++;
+
+	// notify about new file name
+	RegisterRenamedFile(Util::BaseFileName(szSrcFilename), Util::BaseFileName(szDestFileName));
 }
 
 #endif
