@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1047 $
- * $Date: 2014-06-29 23:24:58 +0200 (Sun, 29 Jun 2014) $
+ * $Revision: 1139 $
+ * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
  *
  */
 
@@ -55,6 +55,7 @@
 #include "Unpack.h"
 #include "NZBFile.h"
 #include "StatMeter.h"
+#include "QueueScript.h"
 
 extern HistoryCoordinator* g_pHistoryCoordinator;
 extern DupeCoordinator* g_pDupeCoordinator;
@@ -62,6 +63,7 @@ extern Options* g_pOptions;
 extern Scheduler* g_pScheduler;
 extern Scanner* g_pScanner;
 extern StatMeter* g_pStatMeter;
+extern QueueScriptCoordinator* g_pQueueScriptCoordinator;
 
 PrePostProcessor::PrePostProcessor()
 {
@@ -188,9 +190,25 @@ void PrePostProcessor::DownloadQueueUpdate(Subject* Caller, void* Aspect)
 	{
 		NZBAdded(pQueueAspect->pDownloadQueue, pQueueAspect->pNZBInfo);
 	}
+	else if (pQueueAspect->eAction == DownloadQueue::eaNzbDeleted &&
+		pQueueAspect->pNZBInfo->GetDeleting() &&
+		!pQueueAspect->pNZBInfo->GetPostInfo() &&
+		!pQueueAspect->pNZBInfo->GetParCleanup() &&
+		pQueueAspect->pNZBInfo->GetFileList()->empty())
+	{
+		// the deleting of nzbs is usually handled via eaFileDeleted-event, but when deleting nzb without
+		// any files left the eaFileDeleted-event is not fired and we need to process eaNzbDeleted-event instead
+		info("Collection %s deleted from queue", pQueueAspect->pNZBInfo->GetName());
+		NZBDeleted(pQueueAspect->pDownloadQueue, pQueueAspect->pNZBInfo);
+	}
 	else if ((pQueueAspect->eAction == DownloadQueue::eaFileCompleted ||
 		pQueueAspect->eAction == DownloadQueue::eaFileDeleted))
 	{
+		if (pQueueAspect->eAction == DownloadQueue::eaFileCompleted && !pQueueAspect->pNZBInfo->GetPostInfo())
+		{
+			g_pQueueScriptCoordinator->EnqueueScript(pQueueAspect->pNZBInfo, QueueScriptCoordinator::qeFileDownloaded);
+		}
+
 		if (
 #ifndef DISABLE_PARCHECK
 			!m_ParCoordinator.AddPar(pQueueAspect->pFileInfo, pQueueAspect->eAction == DownloadQueue::eaFileDeleted) &&
@@ -205,6 +223,7 @@ void PrePostProcessor::DownloadQueueUpdate(Subject* Caller, void* Aspect)
 				 pQueueAspect->pFileInfo->GetNZBInfo()->GetDeleteStatus() != NZBInfo::dsHealth)
 			{
 				info("Collection %s completely downloaded", pQueueAspect->pNZBInfo->GetName());
+				g_pQueueScriptCoordinator->EnqueueScript(pQueueAspect->pNZBInfo, QueueScriptCoordinator::qeNzbDownloaded);
 				NZBDownloaded(pQueueAspect->pDownloadQueue, pQueueAspect->pNZBInfo);
 			}
 			else if ((pQueueAspect->eAction == DownloadQueue::eaFileDeleted ||
@@ -240,9 +259,9 @@ void PrePostProcessor::NZBAdded(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo
 	{
 		NZBCompleted(pDownloadQueue, pNZBInfo, false);
 	}
-	else if (!Util::EmptyStr(g_pOptions->GetQueueScript()))
+	else
 	{
-		QueueScriptController::StartScripts(pDownloadQueue, pNZBInfo);
+		g_pQueueScriptCoordinator->EnqueueScript(pNZBInfo, QueueScriptCoordinator::qeNzbAdded);
 	}
 }
 
@@ -267,14 +286,6 @@ void PrePostProcessor::NZBDownloaded(DownloadQueue* pDownloadQueue, NZBInfo* pNZ
 			pNZBInfo->SetRenameStatus(NZBInfo::rsSkipped);
 		}
 
-		if (pNZBInfo->GetDeleteStatus() != NZBInfo::dsNone)
-		{
-			pNZBInfo->SetParStatus(NZBInfo::psFailure);
-			pNZBInfo->SetUnpackStatus(NZBInfo::usFailure);
-			pNZBInfo->SetRenameStatus(NZBInfo::rsFailure);
-			pNZBInfo->SetMoveStatus(NZBInfo::msFailure);
-		}
-
 		pDownloadQueue->Save();
 	}
 	else
@@ -291,17 +302,70 @@ void PrePostProcessor::NZBDeleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBIn
 	}
 	pNZBInfo->SetDeleting(false);
 
+	DeleteCleanup(pNZBInfo);
+
+	if (pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth ||
+		pNZBInfo->GetDeleteStatus() == NZBInfo::dsBad)
+	{
+		NZBDownloaded(pDownloadQueue, pNZBInfo);
+	}
+	else
+	{
+		NZBCompleted(pDownloadQueue, pNZBInfo, true);
+	}
+}
+
+void PrePostProcessor::NZBCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, bool bSaveQueue)
+{
+	bool bAddToHistory = g_pOptions->GetKeepHistory() > 0 && !pNZBInfo->GetAvoidHistory();
+	if (bAddToHistory)
+	{
+		g_pHistoryCoordinator->AddToHistory(pDownloadQueue, pNZBInfo);
+	}
+	pNZBInfo->SetAvoidHistory(false);
+
+	bool bNeedSave = bAddToHistory;
+
+	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce &&
+		(pNZBInfo->GetDeleteStatus() == NZBInfo::dsNone ||
+		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth ||
+		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsBad))
+	{
+		g_pDupeCoordinator->NZBCompleted(pDownloadQueue, pNZBInfo);
+		bNeedSave = true;
+	}
+
+	if (!bAddToHistory)
+	{
+		g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
+		pDownloadQueue->GetQueue()->Remove(pNZBInfo);
+		delete pNZBInfo;
+	}
+
+	if (bSaveQueue && bNeedSave)
+	{
+		pDownloadQueue->Save();
+	}
+}
+
+void PrePostProcessor::DeleteCleanup(NZBInfo* pNZBInfo)
+{
 	if ((g_pOptions->GetDeleteCleanupDisk() && pNZBInfo->GetCleanupDisk()) ||
 		pNZBInfo->GetDeleteStatus() == NZBInfo::dsDupe)
 	{
 		// download was cancelled, deleting already downloaded files from disk
-		for (NZBInfo::Files::reverse_iterator it = pNZBInfo->GetCompletedFiles()->rbegin(); it != pNZBInfo->GetCompletedFiles()->rend(); it++)
+		for (CompletedFiles::reverse_iterator it = pNZBInfo->GetCompletedFiles()->rbegin(); it != pNZBInfo->GetCompletedFiles()->rend(); it++)
 		{
-			char* szFilename = *it;
-			if (Util::FileExists(szFilename))
+			CompletedFile* pCompletedFile = *it;
+
+			char szFullFileName[1024];
+			snprintf(szFullFileName, 1024, "%s%c%s", pNZBInfo->GetDestDir(), (int)PATH_SEPARATOR, pCompletedFile->GetFileName());
+			szFullFileName[1024-1] = '\0';
+
+			if (Util::FileExists(szFullFileName))
 			{
-				detail("Deleting file %s", Util::BaseFileName(szFilename));
-				remove(szFilename);
+				detail("Deleting file %s", pCompletedFile->GetFileName());
+				remove(szFullFileName);
 			}
 		}
 
@@ -327,47 +391,6 @@ void PrePostProcessor::NZBDeleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBIn
 			rmdir(pNZBInfo->GetDestDir());
 		}
 	}
-
-	if (pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth)
-	{
-		NZBDownloaded(pDownloadQueue, pNZBInfo);
-	}
-	else
-	{
-		NZBCompleted(pDownloadQueue, pNZBInfo, true);
-	}
-}
-
-void PrePostProcessor::NZBCompleted(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, bool bSaveQueue)
-{
-	bool bAddToHistory = g_pOptions->GetKeepHistory() > 0 && !pNZBInfo->GetAvoidHistory();
-	if (bAddToHistory)
-	{
-		g_pHistoryCoordinator->AddToHistory(pDownloadQueue, pNZBInfo);
-	}
-	pNZBInfo->SetAvoidHistory(false);
-
-	bool bNeedSave = bAddToHistory;
-
-	if (g_pOptions->GetDupeCheck() && pNZBInfo->GetDupeMode() != dmForce &&
-		(pNZBInfo->GetDeleteStatus() == NZBInfo::dsNone ||
-		 pNZBInfo->GetDeleteStatus() == NZBInfo::dsHealth))
-	{
-		g_pDupeCoordinator->NZBCompleted(pDownloadQueue, pNZBInfo);
-		bNeedSave = true;
-	}
-
-	if (!bAddToHistory)
-	{
-		g_pHistoryCoordinator->DeleteQueuedFile(pNZBInfo->GetQueuedFilename());
-		pDownloadQueue->GetQueue()->Remove(pNZBInfo);
-		delete pNZBInfo;
-	}
-
-	if (bSaveQueue && bNeedSave)
-	{
-		pDownloadQueue->Save();
-	}
 }
 
 void PrePostProcessor::CheckDiskSpace()
@@ -397,11 +420,6 @@ void PrePostProcessor::CheckPostQueue()
 	if (!m_pCurJob && m_iJobCount > 0)
 	{
 		m_pCurJob = GetNextJob(pDownloadQueue);
-		if (!m_pCurJob && !g_pOptions->GetPausePostProcess())
-		{
-			error("Internal error: no jobs found in queue");
-			m_iJobCount = 0;
-		}
 	}
 
 	if (m_pCurJob)
@@ -410,9 +428,12 @@ void PrePostProcessor::CheckPostQueue()
 		if (!pPostInfo->GetWorking() && !IsNZBFileDownloading(m_pCurJob))
 		{
 #ifndef DISABLE_PARCHECK
-			if (pPostInfo->GetRequestParCheck() && pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped &&
+			if (pPostInfo->GetRequestParCheck() &&
+				(pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped ||
+				 (pPostInfo->GetForceRepair() && !pPostInfo->GetNZBInfo()->GetParFull())) &&
 				g_pOptions->GetParCheck() != Options::pcManual)
 			{
+				pPostInfo->SetForceParFull(pPostInfo->GetNZBInfo()->GetParStatus() > NZBInfo::psSkipped);
 				pPostInfo->GetNZBInfo()->SetParStatus(NZBInfo::psNone);
 				pPostInfo->SetRequestParCheck(false);
 				pPostInfo->SetStage(PostInfo::ptQueued);
@@ -474,7 +495,8 @@ NZBInfo* PrePostProcessor::GetNextJob(DownloadQueue* pDownloadQueue)
 	for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
 	{
 		NZBInfo* pNZBInfo1 = *it;
-		if (pNZBInfo1->GetPostInfo() && (!pNZBInfo || pNZBInfo1->GetPriority() > pNZBInfo->GetPriority()) &&
+		if (pNZBInfo1->GetPostInfo() && !g_pQueueScriptCoordinator->HasJob(pNZBInfo1->GetID()) &&
+			(!pNZBInfo || pNZBInfo1->GetPriority() > pNZBInfo->GetPriority()) &&
 			(!g_pOptions->GetPausePostProcess() || pNZBInfo1->GetForcePriority()))
 		{
 			pNZBInfo = pNZBInfo1;
@@ -519,14 +541,21 @@ void PrePostProcessor::DeletePostThread(PostInfo* pPostInfo)
 
 void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostInfo)
 {
+	if (!pPostInfo->GetStartTime())
+	{
+		pPostInfo->SetStartTime(time(NULL));
+	}
+
 #ifndef DISABLE_PARCHECK
-	if (pPostInfo->GetNZBInfo()->GetRenameStatus() == NZBInfo::rsNone)
+	if (pPostInfo->GetNZBInfo()->GetRenameStatus() == NZBInfo::rsNone &&
+		pPostInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone)
 	{
 		UpdatePauseState(g_pOptions->GetParPauseQueue(), "par-rename");
 		m_ParCoordinator.StartParRenameJob(pPostInfo);
 		return;
 	}
-	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psNone)
+	else if (pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psNone &&
+		pPostInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone)
 	{
 		if (m_ParCoordinator.FindMainPars(pPostInfo->GetNZBInfo()->GetDestDir(), NULL))
 		{
@@ -565,7 +594,8 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 
 	NZBParameter* pUnpackParameter = pPostInfo->GetNZBInfo()->GetParameters()->Find("*Unpack:", false);
 	bool bUnpackParam = !(pUnpackParameter && !strcasecmp(pUnpackParameter->GetValue(), "no"));
-	bool bUnpack = bUnpackParam && (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usNone);
+	bool bUnpack = bUnpackParam && pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usNone &&
+		pPostInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone;
 
 	bool bParFailed = pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psFailure ||
 		pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psRepairPossible ||
@@ -573,10 +603,13 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 
 	bool bCleanup = !bUnpack &&
 		pPostInfo->GetNZBInfo()->GetCleanupStatus() == NZBInfo::csNone &&
-		(pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSuccess ||
+		((pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSuccess &&
+		  pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usFailure &&
+		  pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usSpace &&
+		  pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usPassword) ||
 		 (pPostInfo->GetNZBInfo()->GetUnpackStatus() == NZBInfo::usSuccess &&
 		  pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure)) &&
-		strlen(g_pOptions->GetExtCleanupDisk()) > 0;
+		!Util::EmptyStr(g_pOptions->GetExtCleanupDisk());
 
 	bool bMoveInter = !bUnpack &&
 		pPostInfo->GetNZBInfo()->GetMoveStatus() == NZBInfo::msNone &&
@@ -585,7 +618,8 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 		pPostInfo->GetNZBInfo()->GetUnpackStatus() != NZBInfo::usPassword &&
 		pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psFailure &&
 		pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psManual &&
-		strlen(g_pOptions->GetInterDir()) > 0 &&
+		pPostInfo->GetNZBInfo()->GetDeleteStatus() == NZBInfo::dsNone &&
+		!Util::EmptyStr(g_pOptions->GetInterDir()) &&
 		!strncmp(pPostInfo->GetNZBInfo()->GetDestDir(), g_pOptions->GetInterDir(), strlen(g_pOptions->GetInterDir()));
 
 	bool bPostScript = true;
@@ -612,10 +646,6 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 
 	pDownloadQueue->Save();
 
-	if (!pPostInfo->GetStartTime())
-	{
-		pPostInfo->SetStartTime(time(NULL));
-	}
 	pPostInfo->SetStageTime(time(NULL));
 
 	if (bUnpack)
@@ -643,27 +673,44 @@ void PrePostProcessor::StartJob(DownloadQueue* pDownloadQueue, PostInfo* pPostIn
 void PrePostProcessor::JobCompleted(DownloadQueue* pDownloadQueue, PostInfo* pPostInfo)
 {
 	NZBInfo* pNZBInfo = pPostInfo->GetNZBInfo();
+
+	if (pPostInfo->GetStartTime() > 0)
+	{
+		pNZBInfo->SetPostTotalSec((int)(time(NULL) - pPostInfo->GetStartTime()));
+		pPostInfo->SetStartTime(0);
+	}
+
 	DeletePostThread(pPostInfo);
 	pNZBInfo->LeavePostProcess();
 
 	if (IsNZBFileCompleted(pNZBInfo, true, false))
 	{
 		// Cleaning up queue if par-check was successful or unpack was successful or
-		// script was successful (if unpack was not performed)
-		bool bCanCleanupQueue = pNZBInfo->GetParStatus() == NZBInfo::psSuccess ||
-			 pNZBInfo->GetParStatus() == NZBInfo::psRepairPossible ||
-			 pNZBInfo->GetUnpackStatus() == NZBInfo::usSuccess ||
-			 (pNZBInfo->GetUnpackStatus() == NZBInfo::usNone &&
-			  pNZBInfo->GetScriptStatuses()->CalcTotalStatus() == ScriptStatus::srSuccess);
-		if (g_pOptions->GetParCleanupQueue() && bCanCleanupQueue)
+		// health is 100% (if unpack and par-check were not performed)
+		// or health is below critical health
+		bool bCanCleanupQueue =
+			((pNZBInfo->GetParStatus() == NZBInfo::psSuccess ||
+			  pNZBInfo->GetParStatus() == NZBInfo::psRepairPossible) &&
+			 pNZBInfo->GetUnpackStatus() != NZBInfo::usFailure &&
+			 pNZBInfo->GetUnpackStatus() != NZBInfo::usSpace &&
+			 pNZBInfo->GetUnpackStatus() != NZBInfo::usPassword) ||
+			(pNZBInfo->GetUnpackStatus() == NZBInfo::usSuccess &&
+			 pNZBInfo->GetParStatus() != NZBInfo::psFailure) ||
+			(pNZBInfo->GetUnpackStatus() <= NZBInfo::usSkipped &&
+			 pNZBInfo->GetParStatus() != NZBInfo::psFailure &&
+			 pNZBInfo->GetFailedSize() - pNZBInfo->GetParFailedSize() == 0) ||
+			(pNZBInfo->CalcHealth() < pNZBInfo->CalcCriticalHealth(false) &&
+			 pNZBInfo->CalcCriticalHealth(false) < 1000);
+		if (g_pOptions->GetParCleanupQueue() && bCanCleanupQueue && !pNZBInfo->GetFileList()->empty())
 		{
-			if (!pNZBInfo->GetFileList()->empty())
-			{
-				info("Cleaning up download queue for %s", pNZBInfo->GetName());
-				pNZBInfo->ClearCompletedFiles();
-				pNZBInfo->SetParCleanup(true);
-				pDownloadQueue->EditEntry(pNZBInfo->GetID(), DownloadQueue::eaGroupDelete, 0, NULL);
-			}
+			info("Cleaning up download queue for %s", pNZBInfo->GetName());
+			pNZBInfo->SetParCleanup(true);
+			pDownloadQueue->EditEntry(pNZBInfo->GetID(), DownloadQueue::eaGroupDelete, 0, NULL);
+		}
+
+		if (pNZBInfo->GetUnpackCleanedUpDisk())
+		{
+			pNZBInfo->ClearCompletedFiles();
 		}
 
 		NZBCompleted(pDownloadQueue, pNZBInfo, false);
