@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1032 $
- * $Date: 2014-05-29 23:38:27 +0200 (Thu, 29 May 2014) $
+ * $Revision: 1173 $
+ * $Date: 2014-11-26 22:33:57 +0100 (Wed, 26 Nov 2014) $
  *
  */
 
@@ -85,6 +85,8 @@ void UnpackController::StartJob(PostInfo* pPostInfo)
 
 void UnpackController::Run()
 {
+	time_t tStart = time(NULL);
+
 	// the locking is needed for accessing the members of NZBInfo
 	DownloadQueue::Lock();
 
@@ -98,7 +100,7 @@ void UnpackController::Run()
 	m_szPassword[0] = '\0';
 	m_szFinalDir[0] = '\0';
 	m_bFinalDirCreated = false;
-	
+
 	NZBParameter* pParameter = m_pPostInfo->GetNZBInfo()->GetParameters()->Find("*Unpack:", false);
 	bool bUnpack = !(pParameter && !strcasecmp(pParameter->GetValue(), "no"));
 
@@ -127,20 +129,24 @@ void UnpackController::Run()
 		CheckArchiveFiles(bScanNonStdFiles);
 	}
 
-	if (bUnpack && (m_bHasRarFiles || m_bHasNonStdRarFiles || m_bHasSevenZipFiles || m_bHasSevenZipMultiFiles))
+	SetInfoName(m_szInfoName);
+	SetWorkingDir(m_szDestDir);
+
+	bool bHasFiles = m_bHasRarFiles || m_bHasNonStdRarFiles || m_bHasSevenZipFiles || m_bHasSevenZipMultiFiles || m_bHasSplittedFiles;
+
+	if (bUnpack && bHasFiles)
 	{
-		SetInfoName(m_szInfoName);
-		SetWorkingDir(m_szDestDir);
+		m_bUnpackOK = true;
+		m_bUnpackStartError = false;
+		m_bUnpackSpaceError = false;
+		m_bUnpackPasswordError4 = false;
+		m_bUnpackPasswordError5 = false;
+		m_bAutoTerminated = false;
 
 		PrintMessage(Message::mkInfo, "Unpacking %s", m_szName);
 
 		CreateUnpackDir();
 
-		m_bUnpackOK = true;
-		m_bUnpackStartError = false;
-		m_bUnpackSpaceError = false;
-		m_bUnpackPasswordError = false;
-
 		if (m_bHasRarFiles || m_bHasNonStdRarFiles)
 		{
 			ExecuteUnrar();
@@ -156,7 +162,14 @@ void UnpackController::Run()
 			ExecuteSevenZip(true);
 		}
 
+		if (m_bHasSplittedFiles && m_bUnpackOK)
+		{
+			JoinSplittedFiles();
+		}
+
 		Completed();
+
+		m_JoinedFiles.Clear();
 	}
 	else
 	{
@@ -166,7 +179,7 @@ void UnpackController::Run()
 		if (bUnpack && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped && 
 			m_pPostInfo->GetNZBInfo()->GetRenameStatus() <= NZBInfo::rsSkipped && m_bHasParFiles)
 		{
-			RequestParCheck();
+			RequestParCheck(false);
 		}
 		else
 #endif
@@ -176,6 +189,9 @@ void UnpackController::Run()
 		}
 	}
 
+	int iUnpackSec = (int)(time(NULL) - tStart);
+	m_pPostInfo->GetNZBInfo()->SetUnpackSec(m_pPostInfo->GetNZBInfo()->GetUnpackSec() + iUnpackSec);
+
 	m_pPostInfo->SetWorking(false);
 }
 
@@ -193,11 +209,17 @@ void UnpackController::ExecuteUnrar()
 	if (strlen(m_szPassword) > 0)
 	{
 		snprintf(szPasswordParam, 1024, "-p%s", m_szPassword);
+		szPasswordParam[1024-1] = '\0';
 		szArgs[3] = szPasswordParam;
 	}
 	szArgs[4] = "-o+";
 	szArgs[5] = m_bHasNonStdRarFiles ? "*.*" : "*.rar";
-	szArgs[6] = m_szUnpackDir;
+
+	char szUnpackDirParam[1024];
+	snprintf(szUnpackDirParam, 1024, "%s%c", m_szUnpackDir, PATH_SEPARATOR);
+	szUnpackDirParam[1024-1] = '\0';
+	szArgs[6] = szUnpackDirParam;
+
 	szArgs[7] = NULL;
 	SetArgs(szArgs, false);
 
@@ -215,7 +237,7 @@ void UnpackController::ExecuteUnrar()
 	m_bUnpackOK = iExitCode == 0 && m_bAllOKMessageReceived && !GetTerminated();
 	m_bUnpackStartError = iExitCode == -1;
 	m_bUnpackSpaceError = iExitCode == 5;
-	m_bUnpackPasswordError = iExitCode == 11; // only for rar5-archives
+	m_bUnpackPasswordError5 |= iExitCode == 11; // only for rar5-archives
 
 	if (!m_bUnpackOK && iExitCode > 0)
 	{
@@ -240,11 +262,13 @@ void UnpackController::ExecuteSevenZip(bool bMultiVolumes)
 	if (strlen(m_szPassword) > 0)
 	{
 		snprintf(szPasswordParam, 1024, "-p%s", m_szPassword);
+		szPasswordParam[1024-1] = '\0';
 		szArgs[3] = szPasswordParam;
 	}
 
 	char szUnpackDirParam[1024];
 	snprintf(szUnpackDirParam, 1024, "-o%s", m_szUnpackDir);
+	szUnpackDirParam[1024-1] = '\0';
 	szArgs[4] = szUnpackDirParam;
 
 	szArgs[5] = bMultiVolumes ? "*.7z.001" : "*.7z";
@@ -272,6 +296,179 @@ void UnpackController::ExecuteSevenZip(bool bMultiVolumes)
 	}
 }
 
+void UnpackController::JoinSplittedFiles()
+{
+	SetLogPrefix("Join");
+	SetProgressLabel("");
+	m_pPostInfo->SetStageProgress(0);
+
+	// determine groups
+
+	FileList groups;
+	RegEx regExSplitExt(".*\\.[a-z,0-9]{3}\\.001$");
+
+	DirBrowser dir(m_szDestDir);
+	while (const char* filename = dir.Next())
+	{
+		char szFullFilename[1024];
+		snprintf(szFullFilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, filename);
+		szFullFilename[1024-1] = '\0';
+
+		if (strcmp(filename, ".") && strcmp(filename, "..") && !Util::DirectoryExists(szFullFilename))
+		{
+			if (regExSplitExt.Match(filename) && !FileHasRarSignature(szFullFilename))
+			{
+				if (!JoinFile(filename))
+				{
+					m_bUnpackOK = false;
+					break;
+				}
+			}
+		}
+	}
+
+	SetLogPrefix(NULL);
+	SetProgressLabel("");
+}
+
+bool UnpackController::JoinFile(const char* szFragBaseName)
+{
+	char szDestBaseName[1024];
+	strncpy(szDestBaseName, szFragBaseName, 1024);
+	szDestBaseName[1024-1] = '\0';
+
+	// trim extension
+	char* szExtension = strrchr(szDestBaseName, '.');
+	*szExtension = '\0';
+
+	char szFullFilename[1024];
+	snprintf(szFullFilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, szFragBaseName);
+	szFullFilename[1024-1] = '\0';
+	long long lFirstSegmentSize = Util::FileSize(szFullFilename);
+	long long lDifSegmentSize = 0;
+
+	// Validate joinable file:
+	//  - fragments have continuous numbers (no holes);
+	//  - fragments have the same size (except of the last fragment);
+	//  - the last fragment must be smaller than other fragments,
+	//  if it has the same size it is probably not the last and there are missing fragments.
+
+	RegEx regExSplitExt(".*\\.[a-z,0-9]{3}\\.[0-9]{3}$");
+	int iCount = 0;
+	int iMin = -1;
+	int iMax = -1;
+	int iDifSizeCount = 0;
+	int iDifSizeMin = 999999;
+	DirBrowser dir(m_szDestDir);
+	while (const char* filename = dir.Next())
+	{
+		snprintf(szFullFilename, 1024, "%s%c%s", m_szDestDir, PATH_SEPARATOR, filename);
+		szFullFilename[1024-1] = '\0';
+
+		if (strcmp(filename, ".") && strcmp(filename, "..") && !Util::DirectoryExists(szFullFilename) &&
+			regExSplitExt.Match(filename))
+		{
+			const char* szSegExt = strrchr(filename, '.');
+			int iSegNum = atoi(szSegExt + 1);
+			iCount++;
+			iMin = iSegNum < iMin || iMin == -1 ? iSegNum : iMin;
+			iMax = iSegNum > iMax ? iSegNum : iMax;
+			
+			long long lSegmentSize = Util::FileSize(szFullFilename);
+			if (lSegmentSize != lFirstSegmentSize)
+			{
+				iDifSizeCount++;
+				iDifSizeMin = iSegNum < iDifSizeMin ? iSegNum : iDifSizeMin;
+				lDifSegmentSize = lSegmentSize;
+			}
+		}
+	}
+
+	int iCorrectedCount = iCount - (iMin == 0 ? 1 : 0);
+	if ((iMin > 1) || iCorrectedCount != iMax ||
+		((iDifSizeMin != iCorrectedCount || iDifSizeMin > iMax) &&
+		 m_pPostInfo->GetNZBInfo()->GetParStatus() != NZBInfo::psSuccess))
+	{
+		PrintMessage(Message::mkWarning, "Could not join splitted file %s: missing fragments detected", szDestBaseName);
+		return false;
+	}
+
+	// Now can join
+	PrintMessage(Message::mkInfo, "Joining splitted file %s", szDestBaseName);
+	m_pPostInfo->SetStageProgress(0);
+
+	char szErrBuf[256];
+	char szDestFilename[1024];
+	snprintf(szDestFilename, 1024, "%s%c%s", m_szUnpackDir, PATH_SEPARATOR, szDestBaseName);
+	szDestFilename[1024-1] = '\0';
+
+	FILE* pOutFile = fopen(szDestFilename, FOPEN_WBP);
+	if (!pOutFile)
+	{
+		error("Could not create file %s: %s", szDestFilename, Util::GetLastErrorMessage(szErrBuf, sizeof(szErrBuf)));
+		return false;
+	}
+	if (g_pOptions->GetWriteBuffer() > 0)
+	{
+		setvbuf(pOutFile, NULL, _IOFBF, g_pOptions->GetWriteBuffer() * 1024);
+	}
+
+	long long lTotalSize = lFirstSegmentSize * (iCount - 1) + lDifSegmentSize;
+	long long lWritten = 0;
+
+	static const int BUFFER_SIZE = 1024 * 50;
+	char* buffer = (char*)malloc(BUFFER_SIZE);
+
+	bool bOK = true;
+	for (int i = iMin; i <= iMax; i++)
+	{
+		PrintMessage(Message::mkInfo, "Joining from %s.%.3i", szDestBaseName, i);
+
+		char szMessage[1024];
+		snprintf(szMessage, 1024, "Joining from %s.%.3i", szDestBaseName, i);
+		szMessage[1024-1] = '\0';
+		SetProgressLabel(szMessage);
+
+		char szFragFilename[1024];
+		snprintf(szFragFilename, 1024, "%s%c%s.%.3i", m_szDestDir, PATH_SEPARATOR, szDestBaseName, i);
+		szFragFilename[1024-1] = '\0';
+		if (!Util::FileExists(szFragFilename))
+		{
+			break;
+		}
+
+		FILE* pInFile = fopen(szFragFilename, FOPEN_RB);
+		if (pInFile)
+		{
+			int cnt = BUFFER_SIZE;
+			while (cnt == BUFFER_SIZE)
+			{
+				cnt = (int)fread(buffer, 1, BUFFER_SIZE, pInFile);
+				fwrite(buffer, 1, cnt, pOutFile);
+				lWritten += cnt;
+				m_pPostInfo->SetStageProgress(int(lWritten * 1000 / lTotalSize));
+			}
+			fclose(pInFile);
+
+			char szFragFilename[1024];
+			snprintf(szFragFilename, 1024, "%s.%.3i", szDestBaseName, i);
+			szFragFilename[1024-1] = '\0';
+			m_JoinedFiles.push_back(strdup(szFragFilename));
+		}
+		else
+		{
+			error("Could not open file %s", szFragFilename);
+			bOK = false;
+			break;
+		}
+	}
+
+	fclose(pOutFile);
+	free(buffer);
+
+	return bOK;
+}
+
 void UnpackController::Completed()
 {
 	bool bCleanupSuccess = Cleanup();
@@ -291,11 +488,14 @@ void UnpackController::Completed()
 	else
 	{
 #ifndef DISABLE_PARCHECK
-		if (!m_bUnpackOK && m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped &&
-			!m_bUnpackStartError && !m_bUnpackSpaceError && !m_bUnpackPasswordError &&
-			!GetTerminated() && m_bHasParFiles)
+		if (!m_bUnpackOK && 
+			(m_pPostInfo->GetNZBInfo()->GetParStatus() <= NZBInfo::psSkipped ||
+			 !m_pPostInfo->GetNZBInfo()->GetParFull()) &&
+			!m_bUnpackStartError && !m_bUnpackSpaceError &&
+			(!m_bUnpackPasswordError5 || m_bUnpackPasswordError4) &&
+			(!GetTerminated() || m_bAutoTerminated) && m_bHasParFiles)
 		{
-			RequestParCheck();
+			RequestParCheck(true);
 		}
 		else
 #endif
@@ -303,7 +503,8 @@ void UnpackController::Completed()
 			PrintMessage(Message::mkError, "%s failed", m_szInfoNameUp);
 			m_pPostInfo->GetNZBInfo()->SetUnpackStatus(
 				m_bUnpackSpaceError ? NZBInfo::usSpace :
-				m_bUnpackPasswordError ? NZBInfo::usPassword :
+				m_bUnpackPasswordError5 || (m_bUnpackPasswordError4 && 
+					m_pPostInfo->GetNZBInfo()->GetParStatus() == NZBInfo::psSuccess) ? NZBInfo::usPassword :
 				NZBInfo::usFailure);
 			m_pPostInfo->SetStage(PostInfo::ptQueued);
 		}
@@ -311,10 +512,11 @@ void UnpackController::Completed()
 }
 
 #ifndef DISABLE_PARCHECK
-void UnpackController::RequestParCheck()
+void UnpackController::RequestParCheck(bool bForceRepair)
 {
 	PrintMessage(Message::mkInfo, "%s requested par-check/repair", m_szInfoNameUp);
 	m_pPostInfo->SetRequestParCheck(true);
+	m_pPostInfo->SetForceRepair(bForceRepair);
 	m_pPostInfo->SetStage(PostInfo::ptFinished);
 }
 #endif
@@ -350,12 +552,14 @@ void UnpackController::CheckArchiveFiles(bool bScanNonStdFiles)
 	m_bHasNonStdRarFiles = false;
 	m_bHasSevenZipFiles = false;
 	m_bHasSevenZipMultiFiles = false;
+	m_bHasSplittedFiles = false;
 
 	RegEx regExRar(".*\\.rar$");
 	RegEx regExRarMultiSeq(".*\\.(r|s)[0-9][0-9]$");
 	RegEx regExSevenZip(".*\\.7z$");
 	RegEx regExSevenZipMulti(".*\\.7z\\.[0-9]+$");
 	RegEx regExNumExt(".*\\.[0-9]+$");
+	RegEx regExSplitExt(".*\\.[a-z,0-9]{3}\\.[0-9]{3}$");
 
 	DirBrowser dir(m_szDestDir);
 	while (const char* filename = dir.Next())
@@ -366,6 +570,9 @@ void UnpackController::CheckArchiveFiles(bool bScanNonStdFiles)
 
 		if (strcmp(filename, ".") && strcmp(filename, "..") && !Util::DirectoryExists(szFullFilename))
 		{
+			const char* szExt = strchr(filename, '.');
+			int iExtNum = szExt ? atoi(szExt + 1) : -1;
+
 			if (regExRar.Match(filename))
 			{
 				m_bHasRarFiles = true;
@@ -378,12 +585,16 @@ void UnpackController::CheckArchiveFiles(bool bScanNonStdFiles)
 			{
 				m_bHasSevenZipMultiFiles = true;
 			}
-			else if (bScanNonStdFiles && !m_bHasNonStdRarFiles &&
+			else if (bScanNonStdFiles && !m_bHasNonStdRarFiles && iExtNum > 1 &&
 				!regExRarMultiSeq.Match(filename) && regExNumExt.Match(filename) &&
 				FileHasRarSignature(szFullFilename))
 			{
 				m_bHasNonStdRarFiles = true;
 			}
+			else if (regExSplitExt.Match(filename) && (iExtNum == 0 || iExtNum == 1))
+			{
+				m_bHasSplittedFiles = true;
+			}
 		}
 	}
 }
@@ -473,6 +684,7 @@ bool UnpackController::Cleanup()
 		RegEx regExRarMultiSeq(".*\\.[r-z][0-9][0-9]$");
 		RegEx regExSevenZip(".*\\.7z$|.*\\.7z\\.[0-9]+$");
 		RegEx regExNumExt(".*\\.[0-9]+$");
+		RegEx regExSplitExt(".*\\.[a-z,0-9]{3}\\.[0-9]{3}$");
 
 		DirBrowser dir(m_szDestDir);
 		while (const char* filename = dir.Next())
@@ -485,8 +697,9 @@ bool UnpackController::Cleanup()
 				!Util::DirectoryExists(szFullFilename) &&
 				(m_bInterDir || !extractedFiles.Exists(filename)) &&
 				(regExRar.Match(filename) || regExSevenZip.Match(filename) ||
-				(regExRarMultiSeq.Match(filename) && FileHasRarSignature(szFullFilename)) ||
-				(m_bHasNonStdRarFiles && regExNumExt.Match(filename) && FileHasRarSignature(szFullFilename))))
+				 (regExRarMultiSeq.Match(filename) && FileHasRarSignature(szFullFilename)) ||
+				 (m_bHasNonStdRarFiles && regExNumExt.Match(filename) && FileHasRarSignature(szFullFilename)) ||
+				 (m_bHasSplittedFiles && regExSplitExt.Match(filename) && m_JoinedFiles.Exists(filename))))
 			{
 				PrintMessage(Message::mkInfo, "Deleting file %s", filename);
 
@@ -607,6 +820,35 @@ void UnpackController::AddMessage(Message::EKind eKind, const char* szText)
 		SetProgressLabel(szText + 7);
 	}
 
+	if (m_eUnpacker == upUnrar &&
+		(!strncmp(szText, "Unrar: Checksum error in the encrypted file", 42) ||
+		 !strncmp(szText, "Unrar: CRC failed in the encrypted file", 39)))
+	{
+		m_bUnpackPasswordError4 = true;
+	}
+
+	if (m_eUnpacker == upUnrar && !strncmp(szText, "Unrar: The specified password is incorrect.'", 43))
+	{
+		m_bUnpackPasswordError5 = true;
+	}
+
+	int iLen = strlen(szText);
+	if (m_eUnpacker == upUnrar && !IsStopped() && (m_bUnpackPasswordError4 || m_bUnpackPasswordError5 ||
+		strstr(szText, " : packed data CRC failed in volume") ||
+		strstr(szText, " : packed data checksum error in volume") ||
+		(iLen > 13 && !strncmp(szText + iLen - 13, " - CRC failed", 13)) ||
+		(iLen > 18 && !strncmp(szText + iLen - 18, " - checksum failed", 18)) ||
+		!strncmp(szText, "Unrar: WARNING: You need to start extraction from a previous volume", 67)))
+	{
+		char szMsgText[1024];
+		snprintf(szMsgText, 1024, "Cancelling %s due to errors", m_szInfoName);
+		szMsgText[1024-1] = '\0';
+		ScriptController::AddMessage(Message::mkWarning, szMsgText);
+		m_pPostInfo->AppendMessage(Message::mkWarning, szMsgText);
+		m_bAutoTerminated = true;
+		Stop();
+	}
+
 	if ((m_eUnpacker == upUnrar && !strncmp(szText, "Unrar: All OK", 13)) ||
 		(m_eUnpacker == upSevenZip && !strncmp(szText, "7-Zip: Everything is Ok", 23)))
 	{
