@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1050 $
- * $Date: 2014-07-04 21:07:51 +0200 (Fri, 04 Jul 2014) $
+ * $Revision: 1139 $
+ * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
  *
  */
 
@@ -136,7 +136,7 @@ bool DiskState::SaveDownloadQueue(DownloadQueue* pDownloadQueue)
 		return false;
 	}
 
-	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 47);
+	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 51);
 
 	// save nzb-infos
 	SaveNZBQueue(pDownloadQueue, outfile);
@@ -179,7 +179,7 @@ bool DiskState::LoadDownloadQueue(DownloadQueue* pDownloadQueue, Servers* pServe
 	char FileSignatur[128];
 	fgets(FileSignatur, sizeof(FileSignatur), infile);
 	iFormatVersion = ParseFormatVersion(FileSignatur);
-	if (iFormatVersion < 3 || iFormatVersion > 47)
+	if (iFormatVersion < 3 || iFormatVersion > 51)
 	{
 		error("Could not load diskstate due to file version mismatch");
 		fclose(infile);
@@ -363,6 +363,9 @@ void DiskState::SaveNZBInfo(NZBInfo* pNZBInfo, FILE* outfile)
 		(int)pNZBInfo->GetAddUrlPaused());
 	fprintf(outfile, "%i,%i\n", pNZBInfo->GetFileCount(), pNZBInfo->GetParkedFileCount());
 	fprintf(outfile, "%i,%i\n", (int)pNZBInfo->GetMinTime(), (int)pNZBInfo->GetMaxTime());
+	fprintf(outfile, "%i,%i,%i\n", (int)pNZBInfo->GetParFull(), 
+		pNZBInfo->GetPostInfo() ? (int)pNZBInfo->GetPostInfo()->GetForceParFull() : 0,
+		pNZBInfo->GetPostInfo() ? (int)pNZBInfo->GetPostInfo()->GetForceRepair() : 0);
 
 	fprintf(outfile, "%u,%u\n", pNZBInfo->GetFullContentHash(), pNZBInfo->GetFilteredContentHash());
 
@@ -382,23 +385,19 @@ void DiskState::SaveNZBInfo(NZBInfo* pNZBInfo, FILE* outfile)
 	fprintf(outfile, "%s\n", pNZBInfo->GetDupeKey());
 	fprintf(outfile, "%i,%i\n", (int)pNZBInfo->GetDupeMode(), pNZBInfo->GetDupeScore());
 
+	Util::SplitInt64(pNZBInfo->GetDownloadedSize(), &High1, &Low1);
+	fprintf(outfile, "%lu,%lu,%i,%i,%i,%i,%i\n", High1, Low1, pNZBInfo->GetDownloadSec(), pNZBInfo->GetPostTotalSec(),
+		pNZBInfo->GetParSec(), pNZBInfo->GetRepairSec(), pNZBInfo->GetUnpackSec());
+
 	char DestDirSlash[1024];
 	snprintf(DestDirSlash, 1023, "%s%c", pNZBInfo->GetDestDir(), PATH_SEPARATOR);
-	int iDestDirLen = strlen(DestDirSlash);
 
 	fprintf(outfile, "%i\n", (int)pNZBInfo->GetCompletedFiles()->size());
-	for (NZBInfo::Files::iterator it = pNZBInfo->GetCompletedFiles()->begin(); it != pNZBInfo->GetCompletedFiles()->end(); it++)
+	for (CompletedFiles::iterator it = pNZBInfo->GetCompletedFiles()->begin(); it != pNZBInfo->GetCompletedFiles()->end(); it++)
 	{
-		char* szFilename = *it;
-		// do not save full path to reduce the size of queue-file
-		if (!strncmp(DestDirSlash, szFilename, iDestDirLen))
-		{
-			fprintf(outfile, "%s\n", szFilename + iDestDirLen);
-		}
-		else
-		{
-			fprintf(outfile, "%s\n", szFilename);
-		}
+		CompletedFile* pCompletedFile = *it;
+		fprintf(outfile, "%i,%i,%lu,%s\n", pCompletedFile->GetID(), (int)pCompletedFile->GetStatus(),
+			pCompletedFile->GetCrc(), pCompletedFile->GetFileName());
 	}
 
 	fprintf(outfile, "%i\n", (int)pNZBInfo->GetParameters()->size());
@@ -672,6 +671,18 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 		pNZBInfo->SetMaxTime((time_t)iMaxTime);
 	}
 
+	if (iFormatVersion >= 51)
+	{
+		int iParFull, iForceParFull, iForceRepair;
+		if (fscanf(infile, "%i,%i,%i\n", &iParFull, &iForceParFull, &iForceRepair) != 3) goto error;
+		pNZBInfo->SetParFull((bool)iParFull);
+		if (pNZBInfo->GetPostInfo())
+		{
+			pNZBInfo->GetPostInfo()->SetForceParFull((bool)iForceParFull);
+			pNZBInfo->GetPostInfo()->SetForceRepair((bool)iForceRepair);
+		}
+	}
+
 	if (true) // clang requires a block for goto to work
 	{
 		unsigned int iFullContentHash = 0, iFilteredContentHash = 0;
@@ -746,6 +757,18 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 		pNZBInfo->SetDupeScore(iDupeScore);
 	}
 
+	if (iFormatVersion >= 48)
+	{
+		unsigned long High1, Low1, iDownloadSec, iPostTotalSec, iParSec, iRepairSec, iUnpackSec;
+		if (fscanf(infile, "%lu,%lu,%i,%i,%i,%i,%i\n", &High1, &Low1, &iDownloadSec, &iPostTotalSec, &iParSec, &iRepairSec, &iUnpackSec) != 7) goto error;
+		pNZBInfo->SetDownloadedSize(Util::JoinInt64(High1, Low1));
+		pNZBInfo->SetDownloadSec(iDownloadSec);
+		pNZBInfo->SetPostTotalSec(iPostTotalSec);
+		pNZBInfo->SetParSec(iParSec);
+		pNZBInfo->SetRepairSec(iRepairSec);
+		pNZBInfo->SetUnpackSec(iUnpackSec);
+	}
+
 	if (iFormatVersion >= 4)
 	{
 		int iFileCount;
@@ -755,16 +778,32 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 			if (!fgets(buf, sizeof(buf), infile)) goto error;
 			if (buf[0] != 0) buf[strlen(buf)-1] = 0; // remove traling '\n'
 
-			// restore full file name.
+			int iID = 0;
 			char* szFileName = buf;
-			char FullFileName[1024];
-			if (!strchr(buf, PATH_SEPARATOR))
+			int iStatus = 0;
+			unsigned long lCrc = 0;
+
+			if (iFormatVersion >= 49)
 			{
-				snprintf(FullFileName, 1023, "%s%c%s", pNZBInfo->GetDestDir(), PATH_SEPARATOR, buf);
-				szFileName = FullFileName;
+				if (iFormatVersion >= 50)
+				{
+					if (sscanf(buf, "%i,%i,%lu", &iID, &iStatus, &lCrc) != 3) goto error;
+					szFileName = strchr(buf, ',');
+					if (szFileName) szFileName = strchr(szFileName+1, ',');
+					if (szFileName) szFileName = strchr(szFileName+1, ',');
+				}
+				else
+				{
+					if (sscanf(buf, "%i,%lu", &iStatus, &lCrc) != 2) goto error;
+					szFileName = strchr(buf + 2, ',');
+				}
+				if (szFileName)
+				{
+					szFileName++;
+				}
 			}
 
-			pNZBInfo->GetCompletedFiles()->push_back(strdup(szFileName));
+			pNZBInfo->GetCompletedFiles()->push_back(new CompletedFile(iID, szFileName, (CompletedFile::EStatus)iStatus, lCrc));
 		}
 	}
 
@@ -825,7 +864,8 @@ bool DiskState::LoadNZBInfo(NZBInfo* pNZBInfo, Servers* pServers, FILE* infile,
 			int iKind, iTime;
 			sscanf(buf, "%i,%i", &iKind, &iTime);
 			char* szText = strchr(buf + 2, ',');
-			if (szText) {
+			if (szText)
+			{
 				szText++;
 			}
 			pNZBInfo->AppendMessage((Message::EKind)iKind, (time_t)iTime, szText);
@@ -978,13 +1018,16 @@ bool DiskState::LoadServerStats(ServerStatList* pServerStatList, Servers* pServe
 		int iServerID, iSuccessArticles, iFailedArticles;
 		if (fscanf(infile, "%i,%i,%i\n", &iServerID, &iSuccessArticles, &iFailedArticles) != 3) goto error;
 
-		// find server (id could change if config file was edited)
-		for (Servers::iterator it = pServers->begin(); it != pServers->end(); it++)
+		if (pServers)
 		{
-			NewsServer* pNewsServer = *it;
-			if (pNewsServer->GetStateID() == iServerID)
+			// find server (id could change if config file was edited)
+			for (Servers::iterator it = pServers->begin(); it != pServers->end(); it++)
 			{
-				pServerStatList->StatOp(pNewsServer->GetID(), iSuccessArticles, iFailedArticles, ServerStatList::soSet);
+				NewsServer* pNewsServer = *it;
+				if (pNewsServer->GetStateID() == iServerID)
+				{
+					pServerStatList->StatOp(pNewsServer->GetID(), iSuccessArticles, iFailedArticles, ServerStatList::soSet);
+				}
 			}
 		}
 	}
@@ -1173,12 +1216,12 @@ error:
 	return false;
 }
 
-bool DiskState::SaveFileState(FileInfo* pFileInfo)
+bool DiskState::SaveFileState(FileInfo* pFileInfo, bool bCompleted)
 {
 	debug("Saving FileState to disk");
 
 	char szFilename[1024];
-	snprintf(szFilename, 1024, "%s%is", g_pOptions->GetQueueDir(), pFileInfo->GetID());
+	snprintf(szFilename, 1024, "%s%i%s", g_pOptions->GetQueueDir(), pFileInfo->GetID(), bCompleted ? "c" : "s");
 	szFilename[1024-1] = '\0';
 
 	FILE* outfile = fopen(szFilename, FOPEN_WB);
@@ -1189,7 +1232,7 @@ bool DiskState::SaveFileState(FileInfo* pFileInfo)
 		return false;
 	}
 
-	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 1);
+	fprintf(outfile, "%s%i\n", FORMATVERSION_SIGNATURE, 2);
 
 	fprintf(outfile, "%i,%i\n", pFileInfo->GetSuccessArticles(), pFileInfo->GetFailedArticles());
 
@@ -1205,19 +1248,22 @@ bool DiskState::SaveFileState(FileInfo* pFileInfo)
 	for (FileInfo::Articles::iterator it = pFileInfo->GetArticles()->begin(); it != pFileInfo->GetArticles()->end(); it++)
 	{
 		ArticleInfo* pArticleInfo = *it;
-		fprintf(outfile, "%i\n", (int)pArticleInfo->GetStatus());
+		fprintf(outfile, "%i,%lu,%i,%lu\n", (int)pArticleInfo->GetStatus(), (unsigned long)pArticleInfo->GetSegmentOffset(),
+			pArticleInfo->GetSegmentSize(), (unsigned long)pArticleInfo->GetCrc());
 	}
 
 	fclose(outfile);
 	return true;
 }
 
-bool DiskState::LoadFileState(FileInfo* pFileInfo, Servers* pServers)
+bool DiskState::LoadFileState(FileInfo* pFileInfo, Servers* pServers, bool bCompleted)
 {
 	char szFilename[1024];
-	snprintf(szFilename, 1024, "%s%is", g_pOptions->GetQueueDir(), pFileInfo->GetID());
+	snprintf(szFilename, 1024, "%s%i%s", g_pOptions->GetQueueDir(), pFileInfo->GetID(), bCompleted ? "c" : "s");
 	szFilename[1024-1] = '\0';
 
+	bool bHasArticles = !pFileInfo->GetArticles()->empty();
+
 	FILE* infile = fopen(szFilename, FOPEN_RB);
 
 	if (!infile)
@@ -1233,7 +1279,7 @@ bool DiskState::LoadFileState(FileInfo* pFileInfo, Servers* pServers)
 	{
 		if (buf[0] != 0) buf[strlen(buf)-1] = 0; // remove traling '\n'
 		iFormatVersion = ParseFormatVersion(buf);
-		if (iFormatVersion > 1)
+		if (iFormatVersion > 2)
 		{
 			error("Could not load diskstate due to file version mismatch");
 			goto error;
@@ -1264,8 +1310,28 @@ bool DiskState::LoadFileState(FileInfo* pFileInfo, Servers* pServers)
 	if (fscanf(infile, "%i\n", &size) != 1) goto error;
 	for (int i = 0; i < size; i++)
 	{
+		if (!bHasArticles)
+		{
+			pFileInfo->GetArticles()->push_back(new ArticleInfo());
+		}
+		ArticleInfo* pa = pFileInfo->GetArticles()->at(i);
+
 		int iStatus;
-		if (fscanf(infile, "%i\n", &iStatus) != 1) goto error;
+
+		if (iFormatVersion >= 2)
+		{
+			unsigned long iSegmentOffset, iCrc;
+			int iSegmentSize;
+			if (fscanf(infile, "%i,%lu,%i,%lu\n", &iStatus, &iSegmentOffset, &iSegmentSize, &iCrc) != 4) goto error;
+			pa->SetSegmentOffset(iSegmentOffset);
+			pa->SetSegmentSize(iSegmentSize);
+			pa->SetCrc(iCrc);
+		}
+		else
+		{
+			if (fscanf(infile, "%i\n", &iStatus) != 1) goto error;
+		}
+
 		ArticleInfo::EStatus eStatus = (ArticleInfo::EStatus)iStatus;
 
 		if (eStatus == ArticleInfo::aiRunning)
@@ -1275,7 +1341,7 @@ bool DiskState::LoadFileState(FileInfo* pFileInfo, Servers* pServers)
 
 		// don't allow all articles be completed or the file will stuck.
 		// such states should never be saved on disk but just in case.
-		if (iCompletedArticles == size - 1)
+		if (iCompletedArticles == size - 1 && !bCompleted)
 		{
 			eStatus = ArticleInfo::aiUndefined;
 		}
@@ -1284,7 +1350,7 @@ bool DiskState::LoadFileState(FileInfo* pFileInfo, Servers* pServers)
 			iCompletedArticles++;
 		}
 
-		pFileInfo->GetArticles()->at(i)->SetStatus(eStatus);
+		pa->SetStatus(eStatus);
 	}
 
 	pFileInfo->SetCompletedArticles(iCompletedArticles);
@@ -1298,6 +1364,35 @@ error:
 	return false;
 }
 
+void DiskState::DiscardFiles(NZBInfo* pNZBInfo)
+{
+	for (FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
+	{
+		FileInfo* pFileInfo = *it;
+		DiscardFile(pFileInfo, true, true, true);
+	}
+
+	for (CompletedFiles::iterator it = pNZBInfo->GetCompletedFiles()->begin(); it != pNZBInfo->GetCompletedFiles()->end(); it++)
+	{
+		CompletedFile* pCompletedFile = *it;
+		if (pCompletedFile->GetStatus() != CompletedFile::cfSuccess && pCompletedFile->GetID() > 0)
+		{
+			char szFilename[1024];
+			snprintf(szFilename, 1024, "%s%i", g_pOptions->GetQueueDir(), pCompletedFile->GetID());
+			szFilename[1024-1] = '\0';
+			remove(szFilename);
+
+			snprintf(szFilename, 1024, "%s%is", g_pOptions->GetQueueDir(), pCompletedFile->GetID());
+			szFilename[1024-1] = '\0';
+			remove(szFilename);
+
+			snprintf(szFilename, 1024, "%s%ic", g_pOptions->GetQueueDir(), pCompletedFile->GetID());
+			szFilename[1024-1] = '\0';
+			remove(szFilename);
+		}
+	}
+}
+
 bool DiskState::LoadPostQueue12(DownloadQueue* pDownloadQueue, NZBList* pNZBList, FILE* infile, int iFormatVersion)
 {
 	debug("Loading post-queue from disk");
@@ -1738,6 +1833,7 @@ bool DiskState::LoadHistory(DownloadQueue* pDownloadQueue, NZBList* pNZBList, Se
 			{
 				pNZBInfo = new NZBInfo();
 				if (!LoadNZBInfo(pNZBInfo, pServers, infile, iFormatVersion)) goto error;
+				pNZBInfo->LeavePostProcess();
 			}
 
 			pHistoryInfo = new HistoryInfo(pNZBInfo);
@@ -1853,6 +1949,16 @@ void DiskState::DiscardDownloadQueue()
 			snprintf(szFullFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), filename);
 			szFullFilename[1024-1] = '\0';
 			remove(szFullFilename);
+
+			// delete file state file
+			snprintf(szFullFilename, 1024, "%s%ss", g_pOptions->GetQueueDir(), filename);
+			szFullFilename[1024-1] = '\0';
+			remove(szFullFilename);
+
+			// delete failed info file
+			snprintf(szFullFilename, 1024, "%s%sc", g_pOptions->GetQueueDir(), filename);
+			szFullFilename[1024-1] = '\0';
+			remove(szFullFilename);
 		}
 	}
 }
@@ -1867,20 +1973,33 @@ bool DiskState::DownloadQueueExists()
 	return Util::FileExists(fileName);
 }
 
-bool DiskState::DiscardFile(FileInfo* pFileInfo)
+bool DiskState::DiscardFile(FileInfo* pFileInfo, bool bDeleteData, bool bDeletePartialState, bool bDeleteCompletedState)
 {
-	// delete diskstate-file for file-info
+	char fileName[1024];
 
 	// info and articles file
-	char fileName[1024];
-	snprintf(fileName, 1024, "%s%i", g_pOptions->GetQueueDir(), pFileInfo->GetID());
-	fileName[1024-1] = '\0';
-	remove(fileName);
+	if (bDeleteData)
+	{
+		snprintf(fileName, 1024, "%s%i", g_pOptions->GetQueueDir(), pFileInfo->GetID());
+		fileName[1024-1] = '\0';
+		remove(fileName);
+	}
 
-	// state file
-	snprintf(fileName, 1024, "%s%is", g_pOptions->GetQueueDir(), pFileInfo->GetID());
-	fileName[1024-1] = '\0';
-	remove(fileName);
+	// partial state file
+	if (bDeletePartialState)
+	{
+		snprintf(fileName, 1024, "%s%is", g_pOptions->GetQueueDir(), pFileInfo->GetID());
+		fileName[1024-1] = '\0';
+		remove(fileName);
+	}
+
+	// completed state file
+	if (bDeleteCompletedState)
+	{
+		snprintf(fileName, 1024, "%s%ic", g_pOptions->GetQueueDir(), pFileInfo->GetID());
+		fileName[1024-1] = '\0';
+		remove(fileName);
+	}
 
 	return true;
 }
@@ -2222,6 +2341,12 @@ void DiskState::CalcNZBFileStats(NZBInfo* pNZBInfo, int iFormatVersion)
 
 bool DiskState::LoadAllFileStates(DownloadQueue* pDownloadQueue, Servers* pServers)
 {
+	char szCacheFlagFilename[1024];
+	snprintf(szCacheFlagFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "acache");
+	szCacheFlagFilename[1024-1] = '\0';
+
+	bool bCacheWasActive = Util::FileExists(szCacheFlagFilename);
+
 	DirBrowser dir(g_pOptions->GetQueueDir());
 	while (const char* filename = dir.Next())
 	{
@@ -2229,7 +2354,7 @@ bool DiskState::LoadAllFileStates(DownloadQueue* pDownloadQueue, Servers* pServe
 		char suffix;
 		if (sscanf(filename, "%i%c", &id, &suffix) == 2 && suffix == 's')
 		{
-			if (g_pOptions->GetContinuePartial())
+			if (g_pOptions->GetContinuePartial() && !bCacheWasActive)
 			{
 				for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
 				{
@@ -2240,7 +2365,7 @@ bool DiskState::LoadAllFileStates(DownloadQueue* pDownloadQueue, Servers* pServe
 						if (pFileInfo->GetID() == id)
 						{
 							if (!LoadArticles(pFileInfo)) goto error;
-							if (!LoadFileState(pFileInfo, pServers)) goto error;
+							if (!LoadFileState(pFileInfo, pServers, false)) goto error;
 						}
 					}
 				}
@@ -2734,3 +2859,28 @@ error:
 
 	return false;
 }
+
+void DiskState::WriteCacheFlag()
+{
+	char szFlagFilename[1024];
+	snprintf(szFlagFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "acache");
+	szFlagFilename[1024-1] = '\0';
+
+	FILE* outfile = fopen(szFlagFilename, FOPEN_WB);
+	if (!outfile)
+	{
+		error("Error saving diskstate: Could not create file %s", szFlagFilename);
+		return;
+	}
+
+	fclose(outfile);
+}
+
+void DiskState::DeleteCacheFlag()
+{
+	char szFlagFilename[1024];
+	snprintf(szFlagFilename, 1024, "%s%s", g_pOptions->GetQueueDir(), "acache");
+	szFlagFilename[1024-1] = '\0';
+
+	remove(szFlagFilename);
+}
