@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1050 $
- * $Date: 2014-07-04 21:07:51 +0200 (Fri, 04 Jul 2014) $
+ * $Revision: 1073 $
+ * $Date: 2014-08-06 01:45:28 +0200 (Wed, 06 Aug 2014) $
  *                                                          
  */
 
@@ -58,7 +58,6 @@ private:
 	bool				LoadVolumeStat(Servers* pServers, ServerVolumes* pServerVolumes, FILE* infile, int iFormatVersion);
 	void				CalcFileStats(DownloadQueue* pDownloadQueue, int iFormatVersion);
 	void				CalcNZBFileStats(NZBInfo* pNZBInfo, int iFormatVersion);
-	bool				LoadFileState(FileInfo* pFileInfo, Servers* pServers);
 	bool				LoadAllFileStates(DownloadQueue* pDownloadQueue, Servers* pServers);
 	void				SaveServerStats(ServerStatList* pServerStatList, FILE* outfile);
 	bool				LoadServerStats(ServerStatList* pServerStatList, Servers* pServers, FILE* infile);
@@ -80,15 +79,19 @@ public:
 	bool				SaveDownloadQueue(DownloadQueue* pDownloadQueue);
 	bool				LoadDownloadQueue(DownloadQueue* pDownloadQueue, Servers* pServers);
 	bool				SaveFile(FileInfo* pFileInfo);
-	bool				SaveFileState(FileInfo* pFileInfo);
+	bool				SaveFileState(FileInfo* pFileInfo, bool bCompleted);
+	bool				LoadFileState(FileInfo* pFileInfo, Servers* pServers, bool bCompleted);
 	bool				LoadArticles(FileInfo* pFileInfo);
 	void				DiscardDownloadQueue();
-	bool				DiscardFile(FileInfo* pFileInfo);
+	bool				DiscardFile(FileInfo* pFileInfo, bool bDeleteData, bool bDeletePartialState, bool bDeleteCompletedState);
+	void				DiscardFiles(NZBInfo* pNZBInfo);
 	bool				SaveFeeds(Feeds* pFeeds, FeedHistory* pFeedHistory);
 	bool				LoadFeeds(Feeds* pFeeds, FeedHistory* pFeedHistory);
 	bool				SaveStats(Servers* pServers, ServerVolumes* pServerVolumes);
 	bool				LoadStats(Servers* pServers, ServerVolumes* pServerVolumes, bool* pPerfectMatch);
 	void				CleanupTempDir(DownloadQueue* pDownloadQueue);
+	void				WriteCacheFlag();
+	void				DeleteCacheFlag();
 };
 
 #endif
