@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1050 $
- * $Date: 2014-07-04 21:07:51 +0200 (Fri, 04 Jul 2014) $
+ * $Revision: 1139 $
+ * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
  *
  */
 
@@ -41,10 +41,12 @@
 
 #include "nzbget.h"
 #include "DownloadInfo.h"
+#include "ArticleWriter.h"
 #include "Options.h"
 #include "Util.h"
 
 extern Options* g_pOptions;
+extern ArticleCache* g_pArticleCache;
 
 int FileInfo::m_iIDGen = 0;
 int FileInfo::m_iIDMax = 0;
@@ -330,6 +332,16 @@ NZBInfo::NZBInfo() : m_FileList(true)
 	m_pPostInfo = NULL;
 	m_iIDMessageGen = 0;
 	m_iID = ++m_iIDGen;
+	m_lDownloadedSize = 0;
+	m_iDownloadSec = 0;
+	m_iPostTotalSec = 0;
+	m_iParSec = 0;
+	m_iRepairSec = 0;
+	m_iUnpackSec = 0;
+	m_tDownloadStartTime = 0;
+	m_bReprocess = false;
+	m_tQueueScriptTime = 0;
+	m_bParFull = false;
 }
 
 NZBInfo::~NZBInfo()
@@ -386,9 +398,9 @@ int NZBInfo::GenerateID()
 
 void NZBInfo::ClearCompletedFiles()
 {
-	for (Files::iterator it = m_completedFiles.begin(); it != m_completedFiles.end(); it++)
+	for (CompletedFiles::iterator it = m_completedFiles.begin(); it != m_completedFiles.end(); it++)
 	{
-		free(*it);
+		delete *it;
 	}
 	m_completedFiles.clear();
 }
@@ -725,6 +737,25 @@ void NZBInfo::LeavePostProcess()
 	m_pPostInfo = NULL;
 }
 
+void NZBInfo::SetActiveDownloads(int iActiveDownloads)
+{
+	if (((m_iActiveDownloads == 0 && iActiveDownloads > 0) ||
+		 (m_iActiveDownloads > 0 && iActiveDownloads == 0)) &&
+		m_eKind == NZBInfo::nkNzb)
+	{
+		if (iActiveDownloads > 0)
+		{
+			m_tDownloadStartTime = time(NULL);
+		}
+		else
+		{
+			m_iDownloadSec += time(NULL) - m_tDownloadStartTime;
+			m_tDownloadStartTime = 0;
+		}
+	}
+	m_iActiveDownloads = iActiveDownloads;
+}
+
 bool NZBInfo::IsDupeSuccess()
 {
 	bool bFailure =
@@ -769,6 +800,10 @@ const char* NZBInfo::MakeTextStatus(bool bIgnoreScriptStatus)
 		{
 			szStatus = "DELETED/DUPE";
 		}
+		else if (m_eDeleteStatus == NZBInfo::dsBad)
+		{
+			szStatus = "FAILURE/BAD";
+		}
 		else if (m_eParStatus == NZBInfo::psFailure)
 		{
 			szStatus = "FAILURE/PAR";
@@ -895,20 +930,38 @@ void NZBList::Remove(NZBInfo* pNZBInfo)
 	}
 }
 
+NZBInfo* NZBList::Find(int iID)
+{
+	for (iterator it = begin(); it != end(); it++)
+	{
+		NZBInfo* pNZBInfo = *it;
+		if (pNZBInfo->GetID() == iID)
+		{
+			return pNZBInfo;
+		}
+	}
+
+	return NULL;
+}
+
 
 ArticleInfo::ArticleInfo()
 {
 	//debug("Creating ArticleInfo");
-	m_szMessageID		= NULL;
-	m_iSize 			= 0;
-	m_eStatus			= aiUndefined;
-	m_szResultFilename	= NULL;
+	m_szMessageID = NULL;
+	m_iSize = 0;
+	m_pSegmentContent = NULL;
+	m_iSegmentOffset = 0;
+	m_iSegmentSize = 0;
+	m_eStatus = aiUndefined;
+	m_szResultFilename = NULL;
+	m_lCrc = 0;
 }
 
 ArticleInfo::~ ArticleInfo()
 {
 	//debug("Destroying ArticleInfo");
-
+	DiscardSegment();
 	free(m_szMessageID);
 	free(m_szResultFilename);
 }
@@ -925,8 +978,26 @@ void ArticleInfo::SetResultFilename(const char * v)
 	m_szResultFilename = strdup(v);
 }
 
+void ArticleInfo::AttachSegment(char* pContent, long long iOffset, int iSize)
+{
+	DiscardSegment();
+	m_pSegmentContent = pContent;
+	m_iSegmentOffset = iOffset;
+	m_iSegmentSize = iSize;
+}
 
-FileInfo::FileInfo()
+void ArticleInfo::DiscardSegment()
+{
+	if (m_pSegmentContent)
+	{
+		free(m_pSegmentContent);
+		m_pSegmentContent = NULL;
+		g_pArticleCache->Free(m_iSegmentSize);
+	}
+}
+
+
+FileInfo::FileInfo(int iID)
 {
 	debug("Creating FileInfo");
 
@@ -956,7 +1027,9 @@ FileInfo::FileInfo()
 	m_bExtraPriority = false;
 	m_iActiveDownloads = 0;
 	m_bAutoDeleted = false;
-	m_iID = ++m_iIDGen;
+	m_iCachedArticles = 0;
+	m_bPartialChanged = false;
+	m_iID = iID ? iID : ++m_iIDGen;
 }
 
 FileInfo::~ FileInfo()
@@ -1088,6 +1161,30 @@ void FileList::Remove(FileInfo* pFileInfo)
 	erase(std::find(begin(), end(), pFileInfo));
 }
 
+CompletedFile::CompletedFile(int iID, const char* szFileName, EStatus eStatus, unsigned long lCrc)
+{
+	m_iID = iID;
+
+	if (FileInfo::m_iIDMax < m_iID)
+	{
+		FileInfo::m_iIDMax = m_iID;
+	}
+
+	m_szFileName = strdup(szFileName);
+	m_eStatus = eStatus;
+	m_lCrc = lCrc;
+}
+
+void CompletedFile::SetFileName(const char* szFileName)
+{
+	free(m_szFileName);
+	m_szFileName = strdup(szFileName);
+}
+
+CompletedFile::~CompletedFile()
+{
+	free(m_szFileName);
+}
 
 PostInfo::PostInfo()
 {
@@ -1097,6 +1194,8 @@ PostInfo::PostInfo()
 	m_bWorking = false;
 	m_bDeleted = false;
 	m_bRequestParCheck = false;
+	m_bForceParFull = false;
+	m_bForceRepair = false;
 	m_szProgressLabel = strdup("");
 	m_iFileProgress = 0;
 	m_iStageProgress = 0;
