@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1050 $
- * $Date: 2014-07-04 21:07:51 +0200 (Fri, 04 Jul 2014) $
+ * $Revision: 1139 $
+ * $Date: 2014-10-09 23:11:42 +0200 (Thu, 09 Oct 2014) $
  *
  */
 
@@ -89,8 +89,12 @@ private:
 	int					m_iPartNumber;
 	char*				m_szMessageID;
 	int					m_iSize;
+	char*				m_pSegmentContent;
+	long long			m_iSegmentOffset;
+	int					m_iSegmentSize;
 	EStatus				m_eStatus;
 	char*				m_szResultFilename;
+	unsigned long		m_lCrc;
 
 public:
 						ArticleInfo();
@@ -99,12 +103,21 @@ public:
 	int 				GetPartNumber() { return m_iPartNumber; }
 	const char* 		GetMessageID() { return m_szMessageID; }
 	void 				SetMessageID(const char* szMessageID);
-	void 				SetSize(int s) { m_iSize = s; }
+	void 				SetSize(int iSize) { m_iSize = iSize; }
 	int 				GetSize() { return m_iSize; }
+	void				AttachSegment(char* pContent, long long iOffset, int iSize);
+	void				DiscardSegment();
+	const char* 		GetSegmentContent() { return m_pSegmentContent; }
+	void				SetSegmentOffset(long long iSegmentOffset) { m_iSegmentOffset = iSegmentOffset; }
+	long long			GetSegmentOffset() { return m_iSegmentOffset; }
+	void 				SetSegmentSize(int iSegmentSize) { m_iSegmentSize = iSegmentSize; }
+	int 				GetSegmentSize() { return m_iSegmentSize; }
 	EStatus				GetStatus() { return m_eStatus; }
 	void				SetStatus(EStatus Status) { m_eStatus = Status; }
 	const char*			GetResultFilename() { return m_szResultFilename; }
 	void 				SetResultFilename(const char* v);
+	unsigned long		GetCrc() { return m_lCrc; }
+	void				SetCrc(unsigned long lCrc) { m_lCrc = lCrc; }
 };
 
 class FileInfo
@@ -142,12 +155,16 @@ private:
 	bool				m_bExtraPriority;
 	int					m_iActiveDownloads;
 	bool				m_bAutoDeleted;
+	int					m_iCachedArticles;
+	bool				m_bPartialChanged;
 
 	static int			m_iIDGen;
 	static int			m_iIDMax;
 
+	friend class CompletedFile;
+
 public:
-						FileInfo();
+						FileInfo(int iID = 0);
 						~FileInfo();
 	int					GetID() { return m_iID; }
 	void				SetID(int iID);
@@ -199,11 +216,15 @@ public:
 	bool				GetOutputInitialized() { return m_bOutputInitialized; }
 	void				SetOutputInitialized(bool bOutputInitialized) { m_bOutputInitialized = bOutputInitialized; }
 	bool				GetExtraPriority() { return m_bExtraPriority; }
-	void				SetExtraPriority(bool bExtraPriority) { m_bExtraPriority = bExtraPriority; };
+	void				SetExtraPriority(bool bExtraPriority) { m_bExtraPriority = bExtraPriority; }
 	int					GetActiveDownloads() { return m_iActiveDownloads; }
 	void				SetActiveDownloads(int iActiveDownloads);
 	bool				GetAutoDeleted() { return m_bAutoDeleted; }
 	void				SetAutoDeleted(bool bAutoDeleted) { m_bAutoDeleted = bAutoDeleted; }
+	int					GetCachedArticles() { return m_iCachedArticles; }
+	void				SetCachedArticles(int iCachedArticles) { m_iCachedArticles = iCachedArticles; }
+	bool				GetPartialChanged() { return m_bPartialChanged; }
+	void				SetPartialChanged(bool bPartialChanged) { m_bPartialChanged = bPartialChanged; }
 	ServerStatList*		GetServerStats() { return &m_ServerStats; }
 };
                               
@@ -220,6 +241,34 @@ public:
 	void				Remove(FileInfo* pFileInfo);
 };
 
+class CompletedFile
+{
+public:
+	enum EStatus
+	{
+		cfUnknown,
+		cfSuccess,
+		cfPartial,
+		cfFailure
+	};
+
+private:
+	int					m_iID;
+	char*				m_szFileName;
+	EStatus				m_eStatus;
+	unsigned long		m_lCrc;
+
+public:
+						CompletedFile(int iID, const char* szFileName, EStatus eStatus, unsigned long lCrc);
+						~CompletedFile();
+	int					GetID() { return m_iID; }
+	void				SetFileName(const char* szFileName);
+	const char*			GetFileName() { return m_szFileName; }
+	EStatus				GetStatus() { return m_eStatus; }
+	unsigned long		GetCrc() { return m_lCrc; }
+};
+
+typedef std::deque<CompletedFile*>	CompletedFiles;
 
 class NZBParameter
 {
@@ -263,7 +312,7 @@ public:
 private:
 	char* 				m_szName;
 	EStatus				m_eStatus;
-	
+
 	friend class ScriptStatusList;
 	
 public:
@@ -341,7 +390,8 @@ public:
 		dsNone,
 		dsManual,
 		dsHealth,
-		dsDupe
+		dsDupe,
+		dsBad
 	};
 
 	enum EMarkStatus
@@ -368,7 +418,6 @@ public:
 		nkUrl
 	};
 
-	typedef std::vector<char*>			Files;
 	typedef std::deque<Message*>		Messages;
 
 	static const int FORCE_PRIORITY = 900;
@@ -409,7 +458,7 @@ private:
 	time_t				m_tMinTime;
 	time_t				m_tMaxTime;
 	int					m_iPriority;
-	Files				m_completedFiles;
+	CompletedFiles		m_completedFiles;
 	ERenameStatus		m_eRenameStatus;
 	EParStatus			m_eParStatus;
 	EUnpackStatus		m_eUnpackStatus;
@@ -443,6 +492,16 @@ private:
 	Messages			m_Messages;
 	int					m_iIDMessageGen;
 	PostInfo*			m_pPostInfo;
+	long long 			m_lDownloadedSize;
+	time_t				m_tDownloadStartTime;
+	int					m_iDownloadSec;
+	int					m_iPostTotalSec;
+	int					m_iParSec;
+	int					m_iRepairSec;
+	int					m_iUnpackSec;
+	bool				m_bReprocess;
+	time_t				m_tQueueScriptTime;
+	bool				m_bParFull;
 
 	static int			m_iIDGen;
 	static int			m_iIDMax;
@@ -485,7 +544,7 @@ public:
 	int					GetRemainingParCount() { return m_iRemainingParCount; }
 	void 				SetRemainingParCount(int iRemainingParCount) { m_iRemainingParCount = iRemainingParCount; }
 	int					GetActiveDownloads() { return m_iActiveDownloads; }
-	void				SetActiveDownloads(int iActiveDownloads) { m_iActiveDownloads = iActiveDownloads; }
+	void				SetActiveDownloads(int iActiveDownloads);
 	long long			GetSuccessSize() { return m_lSuccessSize; }
 	void 				SetSuccessSize(long long lSuccessSize) { m_lSuccessSize = lSuccessSize; }
 	long long			GetFailedSize() { return m_lFailedSize; }
@@ -523,7 +582,7 @@ public:
 	void				SetMaxTime(time_t tMaxTime) { m_tMaxTime = tMaxTime; }
 	void				BuildDestDirName();
 	void				BuildFinalDirName(char* szFinalDirBuf, int iBufSize);
-	Files*				GetCompletedFiles() { return &m_completedFiles; }		// needs locking (for shared objects)
+	CompletedFiles*		GetCompletedFiles() { return &m_completedFiles; }		// needs locking (for shared objects)
 	void				ClearCompletedFiles();
 	ERenameStatus		GetRenameStatus() { return m_eRenameStatus; }
 	void				SetRenameStatus(ERenameStatus eRenameStatus) { m_eRenameStatus = eRenameStatus; }
@@ -578,6 +637,27 @@ public:
 	void				SetFullContentHash(unsigned int iFullContentHash) { m_iFullContentHash = iFullContentHash; }
 	unsigned int		GetFilteredContentHash() { return m_iFilteredContentHash; }
 	void				SetFilteredContentHash(unsigned int iFilteredContentHash) { m_iFilteredContentHash = iFilteredContentHash; }
+	long long 			GetDownloadedSize() { return m_lDownloadedSize; }
+	void 				SetDownloadedSize(long long lDownloadedSize) { m_lDownloadedSize = lDownloadedSize; }
+	int					GetDownloadSec() { return m_iDownloadSec; }
+	void 				SetDownloadSec(int iDownloadSec) { m_iDownloadSec = iDownloadSec; }
+	int					GetPostTotalSec() { return m_iPostTotalSec; }
+	void 				SetPostTotalSec(int iPostTotalSec) { m_iPostTotalSec = iPostTotalSec; }
+	int					GetParSec() { return m_iParSec; }
+	void 				SetParSec(int iParSec) { m_iParSec = iParSec; }
+	int					GetRepairSec() { return m_iRepairSec; }
+	void 				SetRepairSec(int iRepairSec) { m_iRepairSec = iRepairSec; }
+	int					GetUnpackSec() { return m_iUnpackSec; }
+	void 				SetUnpackSec(int iUnpackSec) { m_iUnpackSec = iUnpackSec; }
+	time_t				GetDownloadStartTime() { return m_tDownloadStartTime; }
+	void 				SetDownloadStartTime(time_t tDownloadStartTime) { m_tDownloadStartTime = tDownloadStartTime; }
+	void				SetReprocess(bool bReprocess) { m_bReprocess = bReprocess; }
+	bool				GetReprocess() { return m_bReprocess; }
+	time_t				GetQueueScriptTime() { return m_tQueueScriptTime; }
+	void 				SetQueueScriptTime(time_t tQueueScriptTime) { m_tQueueScriptTime = tQueueScriptTime; }
+	void				SetParFull(bool bParFull) { m_bParFull = bParFull; }
+	bool				GetParFull() { return m_bParFull; }
+
 	void				CopyFileList(NZBInfo* pSrcNZBInfo);
 	void				UpdateMinMaxTime();
 	PostInfo*			GetPostInfo() { return m_pPostInfo; }
@@ -603,6 +683,7 @@ public:
 	void				Clear();
 	void				Add(NZBInfo* pNZBInfo, bool bAddTop);
 	void				Remove(NZBInfo* pNZBInfo);
+	NZBInfo*			Find(int iID);
 };
 
 class PostInfo
@@ -630,6 +711,8 @@ private:
 	bool				m_bWorking;
 	bool				m_bDeleted;
 	bool				m_bRequestParCheck;
+	bool				m_bForceParFull;
+	bool				m_bForceRepair;
 	EStage				m_eStage;
 	char*				m_szProgressLabel;
 	int					m_iFileProgress;
@@ -666,6 +749,10 @@ public:
 	void				SetDeleted(bool bDeleted) { m_bDeleted = bDeleted; }
 	bool				GetRequestParCheck() { return m_bRequestParCheck; }
 	void				SetRequestParCheck(bool bRequestParCheck) { m_bRequestParCheck = bRequestParCheck; }
+	bool				GetForceParFull() { return m_bForceParFull; }
+	void				SetForceParFull(bool bForceParFull) { m_bForceParFull = bForceParFull; }
+	bool				GetForceRepair() { return m_bForceRepair; }
+	void				SetForceRepair(bool bForceRepair) { m_bForceRepair = bForceRepair; }
 	Thread*				GetPostThread() { return m_pPostThread; }
 	void				SetPostThread(Thread* pPostThread) { m_pPostThread = pPostThread; }
 	void				AppendMessage(Message::EKind eKind, const char* szText);
@@ -765,6 +852,7 @@ public:
 	{
 		eaNzbFound,
 		eaNzbAdded,
+		eaNzbDeleted,
 		eaFileCompleted,
 		eaFileDeleted,
 		eaUrlCompleted
@@ -846,7 +934,7 @@ protected:
 	static void				Loaded() { g_bLoaded = true; }
 
 public:
-	virtual					~DownloadQueue() {};
+	virtual					~DownloadQueue() {}
 	static bool				IsLoaded() { return g_bLoaded; }
 	static DownloadQueue*	Lock();
 	static void				Unlock();
