@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 966 $
- * $Date: 2014-03-18 23:35:58 +0100 (Tue, 18 Mar 2014) $
+ * $Revision: 1123 $
+ * $Date: 2014-09-13 23:30:42 +0200 (Sat, 13 Sep 2014) $
  *
  */
 
@@ -103,11 +103,58 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 			}
 			// Flag saying QueueCoordinator to skip nzb-file
 			pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
-			g_pHistoryCoordinator->DeleteQueuedFile(pNZBInfo->GetQueuedFilename());
+			g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
 			return;
 		}
 	}
 
+	// if download has empty dupekey and empty dupescore - check if download queue
+	// or history have an item with the same name and non empty dupekey or dupescore and
+	// take these properties from this item
+	if (Util::EmptyStr(pNZBInfo->GetDupeKey()) && pNZBInfo->GetDupeScore() == 0)
+	{
+		for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
+		{
+			NZBInfo* pQueuedNZBInfo = *it;
+			if (!strcmp(pQueuedNZBInfo->GetName(), pNZBInfo->GetName()) &&
+				(!Util::EmptyStr(pQueuedNZBInfo->GetDupeKey()) || pQueuedNZBInfo->GetDupeScore() != 0))
+			{
+				pNZBInfo->SetDupeKey(pQueuedNZBInfo->GetDupeKey());
+				pNZBInfo->SetDupeScore(pQueuedNZBInfo->GetDupeScore());
+				info("Assigning dupekey %s and dupescore %i to %s from existing queue item with the same name",
+					 pNZBInfo->GetDupeKey(), pNZBInfo->GetDupeScore(), pNZBInfo->GetName());
+				break;
+			}
+		}
+	}
+	if (Util::EmptyStr(pNZBInfo->GetDupeKey()) && pNZBInfo->GetDupeScore() == 0)
+	{
+		for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
+		{
+			HistoryInfo* pHistoryInfo = *it;
+			if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb &&
+				!strcmp(pHistoryInfo->GetNZBInfo()->GetName(), pNZBInfo->GetName()) &&
+				(!Util::EmptyStr(pHistoryInfo->GetNZBInfo()->GetDupeKey()) || pHistoryInfo->GetNZBInfo()->GetDupeScore() != 0))
+			{
+				pNZBInfo->SetDupeKey(pHistoryInfo->GetNZBInfo()->GetDupeKey());
+				pNZBInfo->SetDupeScore(pHistoryInfo->GetNZBInfo()->GetDupeScore());
+				info("Assigning dupekey %s and dupescore %i to %s from existing history item with the same name",
+					 pNZBInfo->GetDupeKey(), pNZBInfo->GetDupeScore(), pNZBInfo->GetName());
+				break;
+			}
+			if (pHistoryInfo->GetKind() == HistoryInfo::hkDup &&
+				!strcmp(pHistoryInfo->GetDupInfo()->GetName(), pNZBInfo->GetName()) &&
+				(!Util::EmptyStr(pHistoryInfo->GetDupInfo()->GetDupeKey()) || pHistoryInfo->GetDupInfo()->GetDupeScore() != 0))
+			{
+				pNZBInfo->SetDupeKey(pHistoryInfo->GetDupInfo()->GetDupeKey());
+				pNZBInfo->SetDupeScore(pHistoryInfo->GetDupInfo()->GetDupeScore());
+				info("Assigning dupekey %s and dupescore %i to %s from existing history item with the same name",
+					 pNZBInfo->GetDupeKey(), pNZBInfo->GetDupeScore(), pNZBInfo->GetName());
+				break;
+			}
+		}
+	}
+
 	// find duplicates in history
 
 	bool bSkip = false;
@@ -211,7 +258,7 @@ void DupeCoordinator::NZBFound(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo)
 
 		// Flag saying QueueCoordinator to skip nzb-file
 		pNZBInfo->SetDeleteStatus(NZBInfo::dsManual);
-		g_pHistoryCoordinator->DeleteQueuedFile(pNZBInfo->GetQueuedFilename());
+		g_pHistoryCoordinator->DeleteDiskFiles(pNZBInfo);
 		return;
 	}
 
@@ -451,3 +498,67 @@ void DupeCoordinator::HistoryCleanup(DownloadQueue* pDownloadQueue, HistoryInfo*
 		pDownloadQueue->Save();
 	}
 }
+
+DupeCoordinator::EDupeStatus DupeCoordinator::GetDupeStatus(DownloadQueue* pDownloadQueue,
+	const char* szName, const char* szDupeKey)
+{
+	EDupeStatus eStatuses = dsNone;
+
+	// find duplicates in download queue
+	for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
+	{
+		NZBInfo* pNZBInfo = *it;
+		if (SameNameOrKey(szName, szDupeKey, pNZBInfo->GetName(), pNZBInfo->GetDupeKey()))
+		{
+			if (pNZBInfo->GetSuccessArticles() + pNZBInfo->GetFailedArticles() > 0)
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsDownloading);
+			}
+			else
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsQueued);
+			}
+		}
+	}
+
+	// find duplicates in history
+	for (HistoryList::iterator it = pDownloadQueue->GetHistory()->begin(); it != pDownloadQueue->GetHistory()->end(); it++)
+	{
+		HistoryInfo* pHistoryInfo = *it;
+
+		if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb &&
+			SameNameOrKey(szName, szDupeKey, pHistoryInfo->GetNZBInfo()->GetName(), pHistoryInfo->GetNZBInfo()->GetDupeKey()))
+		{
+			const char* szTextStatus = pHistoryInfo->GetNZBInfo()->MakeTextStatus(true);
+			if (!strncasecmp(szTextStatus, "SUCCESS", 7))
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsSuccess);
+			}
+			else if (!strncasecmp(szTextStatus, "FAILURE", 7))
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsFailure);
+			}
+			else if (!strncasecmp(szTextStatus, "WARNING", 7))
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsWarning);
+			}
+		}
+
+		if (pHistoryInfo->GetKind() == HistoryInfo::hkDup &&
+			SameNameOrKey(szName, szDupeKey, pHistoryInfo->GetDupInfo()->GetName(), pHistoryInfo->GetDupInfo()->GetDupeKey()))
+		{
+			if (pHistoryInfo->GetDupInfo()->GetStatus() == DupInfo::dsSuccess ||
+				pHistoryInfo->GetDupInfo()->GetStatus() == DupInfo::dsGood)
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsSuccess);
+			}
+			else if (pHistoryInfo->GetDupInfo()->GetStatus() == DupInfo::dsFailed ||
+					 pHistoryInfo->GetDupInfo()->GetStatus() == DupInfo::dsBad)
+			{
+				eStatuses = (EDupeStatus)(eStatuses | dsFailure);
+			}
+		}
+	}
+
+	return eStatuses;
+}
