@@ -18,7 +18,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  * $Revision: 951 $
- * $Date: 2014-07-04 21:07:51 +0200 (Fri, 04 Jul 2014) $
+ * $Date: 2014-10-14 18:12:25 +0200 (Tue, 14 Oct 2014) $
  *
  */
 
@@ -119,7 +119,7 @@ void HistoryCoordinator::IntervalCheck()
 				
 				if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb)
 				{
-					DeleteQueuedFile(pHistoryInfo->GetNZBInfo()->GetQueuedFilename());
+					DeleteDiskFiles(pHistoryInfo->GetNZBInfo());
 				}
 				info("Collection %s removed from history", szNiceName);
 
@@ -144,16 +144,24 @@ void HistoryCoordinator::IntervalCheck()
 	DownloadQueue::Unlock();
 }
 
-void HistoryCoordinator::DeleteQueuedFile(const char* szQueuedFile)
+void HistoryCoordinator::DeleteDiskFiles(NZBInfo* pNZBInfo)
 {
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+	{
+		// delete parked files
+		g_pDiskState->DiscardFiles(pNZBInfo);
+	}
+	pNZBInfo->GetFileList()->Clear();
+
+	// delete nzb-file
 	if (!g_pOptions->GetNzbCleanupDisk())
 	{
 		return;
 	}
 
-	// szQueuedFile may contain one filename or several filenames separated
+	// QueuedFile may contain one filename or several filenames separated
 	// with "|"-character (for merged groups)
-	char* szFilename = strdup(szQueuedFile);
+	char* szFilename = strdup(pNZBInfo->GetQueuedFilename());
 	char* szEnd = szFilename - 1;
 	
 	while (szEnd)
@@ -251,7 +259,7 @@ void HistoryCoordinator::HistoryHide(DownloadQueue* pDownloadQueue, HistoryInfo*
 	pNewHistoryInfo->SetTime(pHistoryInfo->GetTime());
 	(*pDownloadQueue->GetHistory())[pDownloadQueue->GetHistory()->size() - 1 - rindex] = pNewHistoryInfo;
 
-	DeleteQueuedFile(pHistoryInfo->GetNZBInfo()->GetQueuedFilename());
+	DeleteDiskFiles(pHistoryInfo->GetNZBInfo());
 
 	delete pHistoryInfo;
 	info("Collection %s removed from history", szNiceName);
@@ -329,20 +337,7 @@ void HistoryCoordinator::HistoryDelete(DownloadQueue* pDownloadQueue, HistoryLis
 
 	if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb)
 	{
-		NZBInfo* pNZBInfo = pHistoryInfo->GetNZBInfo();
-
-		// delete parked files
-		if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
-		{
-			for (FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
-			{
-				FileInfo* pFileInfo = *it;
-				g_pDiskState->DiscardFile(pFileInfo);
-			}
-		}
-		pNZBInfo->GetFileList()->Clear();
-
-		DeleteQueuedFile(pNZBInfo->GetQueuedFilename());
+		DeleteDiskFiles(pHistoryInfo->GetNZBInfo());
 	}
 
 	if (pHistoryInfo->GetKind() == HistoryInfo::hkNzb &&
@@ -382,7 +377,6 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 	char szNiceName[1024];
 	pHistoryInfo->GetName(szNiceName, 1024);
 	debug("Returning %s from history back to download queue", szNiceName);
-	bool bUnparked = false;
 	NZBInfo* pNZBInfo = NULL;
 
 	if (bReprocess && pHistoryInfo->GetKind() != HistoryInfo::hkNzb)
@@ -396,6 +390,7 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 		pNZBInfo = pHistoryInfo->GetNZBInfo();
 
 		// unpark files
+		bool bUnparked = false;
 		for (FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
 		{
 			FileInfo* pFileInfo = *it;
@@ -403,6 +398,12 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 			bUnparked = true;
 		}
 
+		if (!(bUnparked || bReprocess))
+		{
+			warn("Could not return %s back from history to download queue: history item does not have any files left for download", szNiceName);
+			return;
+		}
+
 		pDownloadQueue->GetQueue()->push_front(pNZBInfo);
 		pHistoryInfo->DiscardNZBInfo();
 
@@ -413,10 +414,16 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 			pNZBInfo->SetUnpackStatus(NZBInfo::usNone);
 			pNZBInfo->SetCleanupStatus(NZBInfo::csNone);
 			pNZBInfo->SetRenameStatus(NZBInfo::rsNone);
+			pNZBInfo->SetPostTotalSec(pNZBInfo->GetPostTotalSec() - pNZBInfo->GetUnpackSec());
+			pNZBInfo->SetUnpackSec(0);
 
 			if (ParCoordinator::FindMainPars(pNZBInfo->GetDestDir(), NULL))
 			{
 				pNZBInfo->SetParStatus(NZBInfo::psNone);
+				pNZBInfo->SetPostTotalSec(pNZBInfo->GetPostTotalSec() - pNZBInfo->GetParSec());
+				pNZBInfo->SetParSec(0);
+				pNZBInfo->SetRepairSec(0);
+				pNZBInfo->SetParFull(false);
 			}
 		}
 		pNZBInfo->SetDeleteStatus(NZBInfo::dsNone);
@@ -428,6 +435,7 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 		{
 			pNZBInfo->SetMoveStatus(NZBInfo::msNone);
 		}
+		pNZBInfo->SetReprocess(bReprocess);
 	}
 
 	if (pHistoryInfo->GetKind() == HistoryInfo::hkUrl)
@@ -437,19 +445,11 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 		pNZBInfo->SetUrlStatus(NZBInfo::lsNone);
 		pNZBInfo->SetDeleteStatus(NZBInfo::dsNone);
 		pDownloadQueue->GetQueue()->push_front(pNZBInfo);
-		bUnparked = true;
 	}
 
-	if (bUnparked || bReprocess)
-	{
-		pDownloadQueue->GetHistory()->erase(itHistory);
-		// the object "pHistoryInfo" is released few lines later, after the call to "NZBDownloaded"
-		info("%s returned from history back to download queue", szNiceName);
-	}
-	else
-	{
-		warn("Could not return %s back from history to download queue: history item does not have any files left for download", szNiceName);
-	}
+	pDownloadQueue->GetHistory()->erase(itHistory);
+	// the object "pHistoryInfo" is released few lines later, after the call to "NZBDownloaded"
+	info("%s returned from history back to download queue", szNiceName);
 
 	if (bReprocess)
 	{
@@ -458,10 +458,7 @@ void HistoryCoordinator::HistoryReturn(DownloadQueue* pDownloadQueue, HistoryLis
 		g_pPrePostProcessor->NZBDownloaded(pDownloadQueue, pNZBInfo);
 	}
 
-	if (bUnparked || bReprocess)
-	{
-		delete pHistoryInfo;
-	}
+	delete pHistoryInfo;
 }
 
 void HistoryCoordinator::HistoryRedownload(DownloadQueue* pDownloadQueue, HistoryList::iterator itHistory,
@@ -514,11 +511,19 @@ void HistoryCoordinator::HistoryRedownload(DownloadQueue* pDownloadQueue, Histor
 		}
 	}
 
+	g_pDiskState->DiscardFiles(pNZBInfo);
+
 	// reset status fields (which are not reset by "HistoryReturn")
 	pNZBInfo->SetMoveStatus(NZBInfo::msNone);
 	pNZBInfo->SetUnpackCleanedUpDisk(false);
 	pNZBInfo->SetParStatus(NZBInfo::psNone);
 	pNZBInfo->SetRenameStatus(NZBInfo::rsNone);
+	pNZBInfo->SetDownloadedSize(0);
+	pNZBInfo->SetDownloadSec(0);
+	pNZBInfo->SetPostTotalSec(0);
+	pNZBInfo->SetParSec(0);
+	pNZBInfo->SetRepairSec(0);
+	pNZBInfo->SetUnpackSec(0);
 	pNZBInfo->ClearCompletedFiles();
 	pNZBInfo->GetServerStats()->Clear();
 	pNZBInfo->GetCurrentServerStats()->Clear();
@@ -530,11 +535,7 @@ void HistoryCoordinator::HistoryRedownload(DownloadQueue* pDownloadQueue, Histor
 
 	HistoryReturn(pDownloadQueue, itHistory, pHistoryInfo, false);
 
-	if (!bPaused && g_pOptions->GetParCheck() != Options::pcForce)
-	{
-		pDownloadQueue->EditEntry(pNZBInfo->GetID(), 
-			DownloadQueue::eaGroupPauseExtraPars, 0, NULL);
-	}
+	g_pPrePostProcessor->NZBAdded(pDownloadQueue, pNZBInfo);
 }
 
 void HistoryCoordinator::HistorySetParameter(HistoryInfo* pHistoryInfo, const char* szText)
