@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1032 $
- * $Date: 2014-05-29 23:38:27 +0200 (Thu, 29 May 2014) $
+ * $Revision: 1122 $
+ * $Date: 2014-09-08 21:35:11 +0200 (Mon, 08 Sep 2014) $
  *
  */
 
@@ -461,7 +461,7 @@ void NZBFile::ReadPassword()
 
     // obtain file size.
     fseek(pFile , 0 , SEEK_END);
-    int iSize  = ftell(pFile);
+    int iSize  = (int)ftell(pFile);
     rewind(pFile);
 
 	// reading first 4KB of the file
