@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1045 $
- * $Date: 2014-06-19 17:00:46 +0200 (Thu, 19 Jun 2014) $
+ * $Revision: 1140 $
+ * $Date: 2014-10-10 23:13:02 +0200 (Fri, 10 Oct 2014) $
  *
  */
 
@@ -47,6 +47,7 @@
 #include "Options.h"
 #include "ServerPool.h"
 #include "ArticleDownloader.h"
+#include "ArticleWriter.h"
 #include "DiskState.h"
 #include "Util.h"
 #include "Decoder.h"
@@ -56,6 +57,7 @@ extern Options* g_pOptions;
 extern ServerPool* g_pServerPool;
 extern DiskState* g_pDiskState;
 extern StatMeter* g_pStatMeter;
+extern ArticleCache* g_pArticleCache;
 
 bool QueueCoordinator::CoordinatorDownloadQueue::EditEntry(
 	int ID, EEditAction eAction, int iOffset, const char* szText)
@@ -88,7 +90,6 @@ QueueCoordinator::QueueCoordinator()
 
 	m_DownloadQueue.m_pOwner = this;
 	CoordinatorDownloadQueue::Init(&m_DownloadQueue);
-	YDecoder::Init();
 }
 
 QueueCoordinator::~QueueCoordinator()
@@ -106,7 +107,6 @@ QueueCoordinator::~QueueCoordinator()
 	m_ActiveDownloads.clear();
 
 	CoordinatorDownloadQueue::Final();
-	YDecoder::Final();
 
 	debug("QueueCoordinator destroyed");
 }
@@ -149,17 +149,35 @@ void QueueCoordinator::Load()
 		pDownloadQueue->Save();
 
 		// re-save file states into diskstate to update server ids
-		if (g_pOptions->GetServerMode() && g_pOptions->GetSaveQueue() && g_pOptions->GetContinuePartial())
+		if (g_pOptions->GetServerMode() && g_pOptions->GetSaveQueue())
 		{
 			for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
 			{
 				NZBInfo* pNZBInfo = *it;
-				for (FileList::iterator it2 = pNZBInfo->GetFileList()->begin(); it2 != pNZBInfo->GetFileList()->end(); it2++)
+
+				if (g_pOptions->GetContinuePartial())
 				{
-					FileInfo* pFileInfo = *it2;
-					if (!pFileInfo->GetArticles()->empty())
+					for (FileList::iterator it2 = pNZBInfo->GetFileList()->begin(); it2 != pNZBInfo->GetFileList()->end(); it2++)
 					{
-						g_pDiskState->SaveFileState(pFileInfo);
+						FileInfo* pFileInfo = *it2;
+						if (!pFileInfo->GetArticles()->empty())
+						{
+							g_pDiskState->SaveFileState(pFileInfo, false);
+						}
+					}
+				}
+
+				for (CompletedFiles::iterator it2 = pNZBInfo->GetCompletedFiles()->begin(); it2 != pNZBInfo->GetCompletedFiles()->end(); it2++)
+				{
+					CompletedFile* pCompletedFile = *it2;
+					if (pCompletedFile->GetStatus() != CompletedFile::cfSuccess && pCompletedFile->GetID() > 0)
+					{
+						FileInfo* pFileInfo = new FileInfo(pCompletedFile->GetID());
+						if (g_pDiskState->LoadFileState(pFileInfo, g_pServerPool->GetServers(), false))
+						{
+							g_pDiskState->SaveFileState(pFileInfo, true);
+						}
+						delete pFileInfo;
 					}
 				}
 			}
@@ -229,6 +247,10 @@ void QueueCoordinator::Run()
 		{
 			g_pStatMeter->EnterLeaveStandBy(bStandBy);
 			bWasStandBy = bStandBy;
+			if (bStandBy)
+			{
+				SavePartialState();
+			}
 		}
 
 		// sleep longer in StandBy
@@ -248,6 +270,10 @@ void QueueCoordinator::Run()
 			// this code should not be called too often, once per second is OK
 			g_pServerPool->CloseUnusedConnections();
 			ResetHangingDownloads();
+			if (!bStandBy)
+			{
+				SavePartialState();
+			}
 			iResetCounter = 0;
 			g_pStatMeter->IntervalCheck();
 			AdjustDownloadsLimit();
@@ -267,6 +293,8 @@ void QueueCoordinator::Run()
 	}
 	debug("QueueCoordinator: Downloads are completed");
 
+	SavePartialState();
+
 	debug("Exiting QueueCoordinator-loop");
 }
 
@@ -318,7 +346,7 @@ void QueueCoordinator::AddNZBFileToQueue(NZBFile* pNZBFile, NZBInfo* pUrlInfo, b
 			bAllPaused &= pFileInfo->GetPaused();
 			if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
 			{
-				g_pDiskState->DiscardFile(pFileInfo);
+				g_pDiskState->DiscardFile(pFileInfo, true, false, false);
 			}
 		}
 		pNZBInfo->SetDeletePaused(bAllPaused);
@@ -431,7 +459,7 @@ void QueueCoordinator::CheckDupeFileInfos(NZBInfo* pNZBInfo)
 		pNZBInfo->GetFileList()->Remove(pFileInfo);
 		if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
 		{
-			g_pDiskState->DiscardFile(pFileInfo);
+			g_pDiskState->DiscardFile(pFileInfo, true, false, false);
 		}
 	}
 }
@@ -533,11 +561,17 @@ bool QueueCoordinator::GetNextArticle(DownloadQueue* pDownloadQueue, FileInfo* &
 					iTotalFileCount += pNZBInfo->GetFileList()->size();
 				}
 
-				int iArrSize = sizeof(bool) * iTotalFileCount;
-				pCheckedFiles = (bool*)malloc(iArrSize);
-				memset(pCheckedFiles, false, iArrSize);
+				if (iTotalFileCount > 0)
+				{
+					int iArrSize = sizeof(bool) * iTotalFileCount;
+					pCheckedFiles = (bool*)malloc(iArrSize);
+					memset(pCheckedFiles, false, iArrSize);
+				}
+			}
+			if (pCheckedFiles)
+			{
+				pCheckedFiles[iFileNum] = true;
 			}
-			pCheckedFiles[iFileNum] = true;
 		}
 	}
 
@@ -574,7 +608,7 @@ void QueueCoordinator::Update(Subject* Caller, void* Aspect)
 {
 	debug("Notification from ArticleDownloader received");
 
-	ArticleDownloader* pArticleDownloader = (ArticleDownloader*) Caller;
+	ArticleDownloader* pArticleDownloader = (ArticleDownloader*)Caller;
 	if ((pArticleDownloader->GetStatus() == ArticleDownloader::adFinished) ||
 		(pArticleDownloader->GetStatus() == ArticleDownloader::adFailed) ||
 		(pArticleDownloader->GetStatus() == ArticleDownloader::adRetry))
@@ -631,10 +665,7 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 		fileCompleted = (int)pFileInfo->GetArticles()->size() == pFileInfo->GetCompletedArticles();
 		pFileInfo->GetServerStats()->ListOp(pArticleDownloader->GetServerStats(), ServerStatList::soAdd);
 		pNZBInfo->GetCurrentServerStats()->ListOp(pArticleDownloader->GetServerStats(), ServerStatList::soAdd);
-		if (g_pOptions->GetServerMode() && g_pOptions->GetSaveQueue() && g_pOptions->GetContinuePartial())
-		{
-			g_pDiskState->SaveFileState(pFileInfo);
-		}
+		pFileInfo->SetPartialChanged(true);
 	}
 
 	if (!pFileInfo->GetFilenameConfirmed() &&
@@ -655,6 +686,8 @@ void QueueCoordinator::ArticleCompleted(ArticleDownloader* pArticleDownloader)
 		}
 	}
 
+	pNZBInfo->SetDownloadedSize(pNZBInfo->GetDownloadedSize() + pArticleDownloader->GetDownloadedSize());
+
 	bool deleteFileObj = false;
 
 	if (fileCompleted && !pFileInfo->GetDeleted())
@@ -754,9 +787,10 @@ void QueueCoordinator::DeleteFileInfo(DownloadQueue* pDownloadQueue, FileInfo* p
 
 	StatFileInfo(pFileInfo, bCompleted);
 
-	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode())
+	if (g_pOptions->GetSaveQueue() && g_pOptions->GetServerMode() &&
+		(!bCompleted || (pFileInfo->GetMissedArticles() == 0 && pFileInfo->GetFailedArticles() == 0)))
 	{
-		g_pDiskState->DiscardFile(pFileInfo);
+		g_pDiskState->DiscardFile(pFileInfo, true, true, false);
 	}
 
 	if (!bCompleted)
@@ -802,6 +836,33 @@ void QueueCoordinator::DiscardDiskFile(FileInfo* pFileInfo)
 	}
 }
 
+void QueueCoordinator::SavePartialState()
+{
+	if (!(g_pOptions->GetServerMode() && g_pOptions->GetSaveQueue() && g_pOptions->GetContinuePartial()))
+	{
+		return;
+	}
+
+	DownloadQueue* pDownloadQueue = DownloadQueue::Lock();
+
+	for (NZBList::iterator it = pDownloadQueue->GetQueue()->begin(); it != pDownloadQueue->GetQueue()->end(); it++)
+	{
+		NZBInfo* pNZBInfo = *it;
+		for (FileList::iterator it2 = pNZBInfo->GetFileList()->begin(); it2 != pNZBInfo->GetFileList()->end(); it2++)
+		{
+			FileInfo* pFileInfo = *it2;
+			if (pFileInfo->GetPartialChanged())
+			{
+				debug("Saving partial state for %s", pFileInfo->GetFilename());
+				g_pDiskState->SaveFileState(pFileInfo, false);
+				pFileInfo->SetPartialChanged(false);
+			}
+		}
+	}
+
+	DownloadQueue::Unlock();
+}
+
 void QueueCoordinator::CheckHealth(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo)
 {
 	if (g_pOptions->GetHealthCheck() == Options::hcNone ||
@@ -853,7 +914,7 @@ void QueueCoordinator::LogDebugInfo()
 
 void QueueCoordinator::ResetHangingDownloads()
 {
-	if (g_pOptions->GetTerminateTimeout() == 0 && g_pOptions->GetConnectionTimeout() == 0)
+	if (g_pOptions->GetTerminateTimeout() == 0 && g_pOptions->GetArticleTimeout() == 0)
 	{
 		return;
 	}
@@ -864,11 +925,12 @@ void QueueCoordinator::ResetHangingDownloads()
 	for (ActiveDownloads::iterator it = m_ActiveDownloads.begin(); it != m_ActiveDownloads.end();)
 	{
 		ArticleDownloader* pArticleDownloader = *it;
-
-		if (tm - pArticleDownloader->GetLastUpdateTime() > g_pOptions->GetConnectionTimeout() + 1 &&
+																		   
+		if (tm - pArticleDownloader->GetLastUpdateTime() > g_pOptions->GetArticleTimeout() + 1 &&
 		   pArticleDownloader->GetStatus() == ArticleDownloader::adRunning)
 		{
-			error("Cancelling hanging download %s", pArticleDownloader->GetInfoName());
+			error("Cancelling hanging download %s @ %s", pArticleDownloader->GetInfoName(),
+				pArticleDownloader->GetConnectionName());
 			pArticleDownloader->Stop();
 		}
 		
@@ -879,16 +941,21 @@ void QueueCoordinator::ResetHangingDownloads()
 			debug("Terminating hanging download %s", pArticleDownloader->GetInfoName());
 			if (pArticleDownloader->Terminate())
 			{
-				error("Terminated hanging download %s", pArticleDownloader->GetInfoName());
+				error("Terminated hanging download %s @ %s", pArticleDownloader->GetInfoName(),
+					pArticleDownloader->GetConnectionName());
 				pArticleInfo->SetStatus(ArticleInfo::aiUndefined);
 			}
 			else
 			{
-				error("Could not terminate hanging download %s", Util::BaseFileName(pArticleInfo->GetResultFilename()));
+				error("Could not terminate hanging download %s @ %s", pArticleDownloader->GetInfoName(),
+					  pArticleDownloader->GetConnectionName());
 			}
 			m_ActiveDownloads.erase(it);
+
 			pArticleDownloader->GetFileInfo()->SetActiveDownloads(pArticleDownloader->GetFileInfo()->GetActiveDownloads() - 1);
 			pArticleDownloader->GetFileInfo()->GetNZBInfo()->SetActiveDownloads(pArticleDownloader->GetFileInfo()->GetNZBInfo()->GetActiveDownloads() - 1);
+			pArticleDownloader->GetFileInfo()->GetNZBInfo()->SetDownloadedSize(pArticleDownloader->GetFileInfo()->GetNZBInfo()->GetDownloadedSize() + pArticleDownloader->GetDownloadedSize());
+
 			// it's not safe to destroy pArticleDownloader, because the state of object is unknown
 			delete pArticleDownloader;
 			it = m_ActiveDownloads.begin();
@@ -907,21 +974,27 @@ void QueueCoordinator::ResetHangingDownloads()
 bool QueueCoordinator::DeleteQueueEntry(DownloadQueue* pDownloadQueue, FileInfo* pFileInfo)
 {
 	pFileInfo->SetDeleted(true);
-	bool hasDownloads = false;
+	bool bDownloading = false;
 	for (ActiveDownloads::iterator it = m_ActiveDownloads.begin(); it != m_ActiveDownloads.end(); it++)
 	{
 		ArticleDownloader* pArticleDownloader = *it;
 		if (pArticleDownloader->GetFileInfo() == pFileInfo)
 		{
-			hasDownloads = true;
+			bDownloading = true;
 			pArticleDownloader->Stop();
 		}
 	}
-	if (!hasDownloads)
+
+	while (g_pArticleCache->FileBusy(pFileInfo))
+	{
+		usleep(20*1000);
+	}
+
+	if (!bDownloading)
 	{
 		DeleteFileInfo(pDownloadQueue, pFileInfo, false);
 	}
-	return hasDownloads;
+	return bDownloading;
 }
 
 bool QueueCoordinator::SetQueueEntryCategory(DownloadQueue* pDownloadQueue, NZBInfo* pNZBInfo, const char* szCategory)
@@ -940,7 +1013,7 @@ bool QueueCoordinator::SetQueueEntryCategory(DownloadQueue* pDownloadQueue, NZBI
 	pNZBInfo->BuildDestDirName();
 
 	bool bDirUnchanged = !strcmp(pNZBInfo->GetDestDir(), szOldDestDir);
-	bool bOK = bDirUnchanged || ArticleDownloader::MoveCompletedFiles(pNZBInfo, szOldDestDir);
+	bool bOK = bDirUnchanged || ArticleWriter::MoveCompletedFiles(pNZBInfo, szOldDestDir);
 
 	return bOK;
 }
@@ -979,7 +1052,7 @@ bool QueueCoordinator::SetQueueEntryName(DownloadQueue* pDownloadQueue, NZBInfo*
 	pNZBInfo->BuildDestDirName();
 
 	bool bDirUnchanged = !strcmp(pNZBInfo->GetDestDir(), szOldDestDir);
-	bool bOK = bDirUnchanged || ArticleDownloader::MoveCompletedFiles(pNZBInfo, szOldDestDir);
+	bool bOK = bDirUnchanged || ArticleWriter::MoveCompletedFiles(pNZBInfo, szOldDestDir);
 
 	return bOK;
 }
@@ -1045,11 +1118,17 @@ bool QueueCoordinator::MergeQueueEntries(DownloadQueue* pDownloadQueue, NZBInfo*
 	pDestNZBInfo->SetMinTime(pSrcNZBInfo->GetMinTime() < pDestNZBInfo->GetMinTime() ? pSrcNZBInfo->GetMinTime() : pDestNZBInfo->GetMinTime());
 	pDestNZBInfo->SetMaxTime(pSrcNZBInfo->GetMaxTime() > pDestNZBInfo->GetMaxTime() ? pSrcNZBInfo->GetMaxTime() : pDestNZBInfo->GetMaxTime());
 
+	pDestNZBInfo->SetDownloadedSize(pDestNZBInfo->GetDownloadedSize() + pSrcNZBInfo->GetDownloadedSize());
+	pDestNZBInfo->SetDownloadSec(pDestNZBInfo->GetDownloadSec() + pSrcNZBInfo->GetDownloadSec());
+	pDestNZBInfo->SetDownloadStartTime((pDestNZBInfo->GetDownloadStartTime() > 0 &&
+		pDestNZBInfo->GetDownloadStartTime() < pSrcNZBInfo->GetDownloadStartTime()) || pSrcNZBInfo->GetDownloadStartTime() == 0 ?
+		pDestNZBInfo->GetDownloadStartTime() : pSrcNZBInfo->GetDownloadStartTime());
+
 	// reattach completed file items to new NZBInfo-object
-	for (NZBInfo::Files::iterator it = pSrcNZBInfo->GetCompletedFiles()->begin(); it != pSrcNZBInfo->GetCompletedFiles()->end(); it++)
+	for (CompletedFiles::iterator it = pSrcNZBInfo->GetCompletedFiles()->begin(); it != pSrcNZBInfo->GetCompletedFiles()->end(); it++)
     {
-		char* szFileName = *it;
-		pDestNZBInfo->GetCompletedFiles()->push_back(szFileName);
+		CompletedFile* pCompletedFile = *it;
+		pDestNZBInfo->GetCompletedFiles()->push_back(pCompletedFile);
 	}
 	pSrcNZBInfo->GetCompletedFiles()->clear();
 
