@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1043 $
- * $Date: 2014-06-13 23:53:27 +0200 (Fri, 13 Jun 2014) $
+ * $Revision: 1108 $
+ * $Date: 2014-08-28 21:15:42 +0200 (Thu, 28 Aug 2014) $
  *
  */
 
@@ -234,8 +234,6 @@ bool QueueEditor::EditList(DownloadQueue* pDownloadQueue, IDList* pIDList, NameL
 bool QueueEditor::InternEditList(ItemList* pItemList, 
 	IDList* pIDList, DownloadQueue::EEditAction eAction, int iOffset, const char* szText)
 {
-	std::set<NZBInfo*> uniqueNzbs;
-
 	ItemList itemList;
 	if (!pItemList)
 	{
@@ -624,6 +622,7 @@ bool QueueEditor::EditGroup(NZBInfo* pNZBInfo, DownloadQueue::EEditAction eActio
 {
 	ItemList itemList;
 	bool bAllPaused = true;
+	int iID = pNZBInfo->GetID();
 
 	// collecting files belonging to group
 	for (FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
@@ -672,7 +671,17 @@ bool QueueEditor::EditGroup(NZBInfo* pNZBInfo, DownloadQueue::EEditAction eActio
 		(DownloadQueue::EEditAction)0,
 		(DownloadQueue::EEditAction)0 };
 
-	return InternEditList(&itemList, NULL, GroupToFileMap[eAction], iOffset, szText);
+	bool bOK = InternEditList(&itemList, NULL, GroupToFileMap[eAction], iOffset, szText);
+
+	if ((eAction == DownloadQueue::eaGroupDelete || eAction == DownloadQueue::eaGroupDupeDelete || eAction == DownloadQueue::eaGroupFinalDelete) &&
+		// NZBInfo could have been destroyed already
+		m_pDownloadQueue->GetQueue()->Find(iID))
+	{
+		DownloadQueue::Aspect deleteAspect = { DownloadQueue::eaNzbDeleted, m_pDownloadQueue, pNZBInfo, NULL };
+		m_pDownloadQueue->Notify(&deleteAspect);
+	}
+
+	return bOK;
 }
 
 void QueueEditor::PauseParsInGroups(ItemList* pItemList, bool bExtraParsOnly)
@@ -901,6 +910,11 @@ void QueueEditor::SetNZBName(NZBInfo* pNZBInfo, const char* szName)
 */
 bool QueueEditor::CanCleanupDisk(NZBInfo* pNZBInfo)
 {
+	if (pNZBInfo->GetDeleteStatus() != NZBInfo::dsNone)
+	{
+		return true;
+	}
+
     for (FileList::iterator it = pNZBInfo->GetFileList()->begin(); it != pNZBInfo->GetFileList()->end(); it++)
     {
         FileInfo* pFileInfo = *it;
