@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1045 $
- * $Date: 2014-06-19 17:00:46 +0200 (Thu, 19 Jun 2014) $
+ * $Revision: 1098 $
+ * $Date: 2014-08-19 22:53:00 +0200 (Tue, 19 Aug 2014) $
  *
  */
 
@@ -80,7 +80,7 @@ void UrlDownloader::ProcessHeader(const char* szLine)
 		char* szValue = strchr(szModLine, ':');
 		if (szValue)
 		{
-			*szValue = NULL;
+			*szValue = '\0';
 			szValue++;
 			while (*szValue == ' ') szValue++;
 			Util::Trim(szValue);
