@@ -18,8 +18,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1018 $
- * $Date: 2014-05-19 23:28:35 +0200 (Mon, 19 May 2014) $
+ * $Revision: 1111 $
+ * $Date: 2014-08-28 22:29:51 +0200 (Thu, 28 Aug 2014) $
  *
  */
 
@@ -101,19 +101,12 @@ void RemoteClient::perror(const char * msg)
 bool RemoteClient::InitConnection()
 {
 	// Create a connection to the server
-
-	const char *szControlIP = g_pOptions->GetControlIP();
-	if (!strcmp(szControlIP, "0.0.0.0"))
-	{
-		szControlIP = "127.0.0.1";
-	}
-
-	m_pConnection = new Connection(szControlIP, g_pOptions->GetControlPort(), false);
+	m_pConnection = new Connection(g_pOptions->GetControlIP(), g_pOptions->GetControlPort(), false);
 
 	bool OK = m_pConnection->Connect();
 	if (!OK)
 	{
-		printf("Unable to send request to nzbserver at %s (port %i)\n", szControlIP, g_pOptions->GetControlPort());
+		printf("Unable to send request to nzbget-server at %s (port %i)\n", g_pOptions->GetControlIP(), g_pOptions->GetControlPort());
 	}
 	return OK;
 }
@@ -242,6 +235,7 @@ void RemoteClient::BuildFileList(SNZBListResponse* pListResponse, const char* pT
 			pNZBInfo->SetRemainingSize(Util::JoinInt64(ntohl(pListAnswer->m_iRemainingSizeHi), ntohl(pListAnswer->m_iRemainingSizeLo)));
 			pNZBInfo->SetPausedSize(Util::JoinInt64(ntohl(pListAnswer->m_iPausedSizeHi), ntohl(pListAnswer->m_iPausedSizeLo)));
 			pNZBInfo->SetPausedFileCount(ntohl(pListAnswer->m_iPausedCount));
+			pNZBInfo->SetRemainingParCount(ntohl(pListAnswer->m_iRemainingParCount));
 			pNZBInfo->SetFilename(szFileName);
 			pNZBInfo->SetName(szName);
 			pNZBInfo->SetDestDir(szDestDir);
