@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1011 $
- * $Date: 2014-05-04 15:18:28 +0200 (Sun, 04 May 2014) $
+ * $Revision: 1168 $
+ * $Date: 2014-11-22 23:11:54 +0100 (Sat, 22 Nov 2014) $
  *
  */
 
@@ -38,12 +38,15 @@
 #include <unistd.h>
 #include <sys/wait.h>
 #include <signal.h>
+#else
+#include <io.h>
 #endif
 #include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <algorithm>
 
 #include "nzbget.h"
 #include "Script.h"
@@ -57,6 +60,9 @@ extern char** environ;
 extern Options* g_pOptions;
 extern char* (*g_szEnvironmentVariables)[];
 
+ScriptController::RunningScripts ScriptController::m_RunningScripts;
+Mutex ScriptController::m_mutexRunning;
+
 #ifndef WIN32
 #define CHILD_WATCHDOG 1
 #endif
@@ -195,7 +201,12 @@ ScriptController::ScriptController()
 	m_szLogPrefix = NULL;
 	m_bTerminated = false;
 	m_bDetached = false;
+	m_hProcess = 0;
 	m_environmentStrings.InitFromCurrentProcess();
+
+	m_mutexRunning.Lock();
+	m_RunningScripts.push_back(this);
+	m_mutexRunning.Unlock();
 }
 
 ScriptController::~ScriptController()
@@ -208,6 +219,10 @@ ScriptController::~ScriptController()
 		}
 		free(m_szArgs);
 	}
+
+	m_mutexRunning.Lock();
+	m_RunningScripts.erase(std::find(m_RunningScripts.begin(), m_RunningScripts.end(), this));
+	m_mutexRunning.Unlock();
 }
 
 void ScriptController::ResetEnv()
@@ -612,6 +627,11 @@ void ScriptController::Terminate()
 
 #ifdef WIN32
 	BOOL bOK = TerminateProcess(m_hProcess, -1);
+	if (bOK)
+	{
+		// wait 60 seconds for process to terminate
+		WaitForSingleObject(m_hProcess, 60 * 1000);
+	}
 #else
 	pid_t hKillProcess = m_hProcess;
 	if (getpgid(hKillProcess) == hKillProcess)
@@ -619,7 +639,7 @@ void ScriptController::Terminate()
 		// if the child process has its own group (setsid() was successful), kill the whole group
 		hKillProcess = -hKillProcess;
 	}
-	bool bOK = kill(hKillProcess, SIGKILL) == 0;
+	bool bOK = hKillProcess && kill(hKillProcess, SIGKILL) == 0;
 #endif
 
 	if (bOK)
@@ -634,6 +654,20 @@ void ScriptController::Terminate()
 	debug("Stopped %s", m_szInfoName);
 }
 
+void ScriptController::TerminateAll()
+{
+	m_mutexRunning.Lock();
+	for (RunningScripts::iterator it = m_RunningScripts.begin(); it != m_RunningScripts.end(); it++)
+	{
+		ScriptController* pScript = *it;
+		if (pScript->m_hProcess && !pScript->m_bDetached)
+		{
+			pScript->Terminate();
+		}
+	}
+	m_mutexRunning.Unlock();
+}
+
 void ScriptController::Detach()
 {
 	debug("Detaching %s", m_szInfoName);
