@@ -17,8 +17,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1040 $
- * $Date: 2014-06-09 22:52:23 +0200 (Mon, 09 Jun 2014) $
+ * $Revision: 1136 $
+ * $Date: 2014-10-04 21:34:03 +0200 (Sat, 04 Oct 2014) $
  *
  */
 
@@ -26,13 +26,9 @@
 #ifndef UTIL_H
 #define UTIL_H
 
-#ifdef WIN32
-#include <stdio.h>
-#include <io.h>
-#else
-#include <dirent.h>
+#ifdef DIRBROWSER_SNAPSHOT
+#include <deque>
 #endif
-
 #include <time.h>
 
 #ifdef WIN32
@@ -49,12 +45,23 @@ private:
 	HANDLE				m_hFile;
 	bool				m_bFirst;
 #else
-	DIR*				m_pDir;
+	void*				m_pDir;    // DIR*, declared as void* to avoid including of <dirent>
 	struct dirent*		m_pFindData;
 #endif
 
+#ifdef DIRBROWSER_SNAPSHOT
+	bool				m_bSnapshot;
+	typedef std::deque<char*>	FileList;
+	FileList			m_Snapshot;
+	FileList::iterator	m_itSnapshot;
+#endif
+
 public:
+#ifdef DIRBROWSER_SNAPSHOT
+						DirBrowser(const char* szPath, bool bSnapshot = true);
+#else
 						DirBrowser(const char* szPath);
+#endif
 						~DirBrowser();
 	const char*			Next();
 };
@@ -75,12 +82,11 @@ public:
 class Util
 {
 public:
-
 	static char* BaseFileName(const char* filename);
 	static void NormalizePathSeparators(char* szPath);
 	static bool LoadFileIntoBuffer(const char* szFileName, char** pBuffer, int* pBufferLength);
 	static bool SaveBufferIntoFile(const char* szFileName, const char* szBuffer, int iBufLen);
-	static bool CreateSparseFile(const char* szFilename, int iSize);
+	static bool CreateSparseFile(const char* szFilename, long long iSize);
 	static bool TruncateFile(const char* szFilename, int iSize);
 	static void MakeValidFilename(char* szFilename, char cReplaceChar, bool bAllowSlashes);
 	static bool MakeUniqueFilename(char* szDestBufFilename, int iDestBufSize, const char* szDestDir, const char* szBasename);
@@ -157,14 +163,18 @@ public:
 	 */
 	static const char* VersionRevision() { return VersionRevisionBuf; };
 	
-	/*
-	 * Initialize buffer for program version and revision number.
-	 * This function must be called during program initialization before any
-	 * call to "VersionRevision()".
-	 */
-	static void InitVersionRevision();
-	
 	static char VersionRevisionBuf[40];
+
+	static void InitVersionRevision();
+
+	static unsigned long Crc32(unsigned char *block, unsigned long length);
+	static unsigned long Crc32m(unsigned long startCrc, unsigned char *block, unsigned long length);
+	static unsigned long Crc32Combine(unsigned long crc1, unsigned long crc2, unsigned long len2);
+
+	/*
+	 * Returns number of available CPU cores or -1 if it could not be determined
+	 */
+	static int NumberOfCpuCores();
 };
 
 class WebUtil
