@@ -1,7 +1,7 @@
 /*
  * This file is part of nzbget
  *
- * Copyright (C) 2012-2013 Andrey Prygunkov <hugbug@users.sourceforge.net>
+ * Copyright (C) 2012-2014 Andrey Prygunkov <hugbug@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 881 $
- * $Date: 2013-10-17 21:35:43 +0200 (Thu, 17 Oct 2013) $
+ * $Revision: 1097 $
+ * $Date: 2014-08-19 21:56:09 +0200 (Tue, 19 Aug 2014) $
  *
  */
 
@@ -65,7 +65,7 @@ var Options = (new function($)
 
 			// loading config templates and build list of post-processing parameters
 			_this.postParamConfig = [];
-			RPC.call('configtemplates', [], function(data)
+			RPC.call('configtemplates', [false], function(data)
 				{
 					initPostParamConfig(data);
 					RPC.next();
@@ -116,7 +116,7 @@ var Options = (new function($)
 	function serverValuesLoaded(data)
 	{
 		serverValues = data;
-		RPC.call('configtemplates', [], serverTemplateLoaded, loadServerTemplateError);
+		RPC.call('configtemplates', [true], serverTemplateLoaded, loadServerTemplateError);
 	}
 
 	function serverTemplateLoaded(data)
@@ -152,6 +152,10 @@ var Options = (new function($)
 			scriptConfig.name = scriptConfig.name.replace(/\\/, ' \\ ').replace(/\//, ' / ');
 			scriptConfig.shortName = shortScriptName(scriptName);
 			scriptConfig.shortName = scriptConfig.shortName.replace(/\\/, ' \\ ').replace(/\//, ' / ');
+			scriptConfig.post = serverTemplateData[i].PostScript;
+			scriptConfig.scan = serverTemplateData[i].ScanScript;
+			scriptConfig.queue = serverTemplateData[i].QueueScript;
+			scriptConfig.scheduler = serverTemplateData[i].SchedulerScript;
 			mergeValues(scriptConfig.sections, serverValues);
 			config.push(scriptConfig);
 		}
@@ -416,32 +420,35 @@ var Options = (new function($)
 
 		for (var i=1; i < data.length; i++)
 		{
-			var scriptName = data[i].Name;
-			var sectionId = (scriptName + ':').replace(/ |\/|[\.|$|\:|\*]/g, '_');
-			var option = {};
-			option.name = scriptName + ':';
-			option.caption = shortScriptName(scriptName);
-			option.caption = option.caption.replace(/\\/, ' \\ ').replace(/\//, ' / ');
-
-			option.defvalue = 'no';
-			option.description = (data[i].Template.trim().split('\n')[0].substr(1, 1000).trim() || 'Post-processing script ' + scriptName + '.');
-			option.value = null;
-			option.sectionId = sectionId;
-			option.select = ['yes', 'no'];
-			section.options.push(option);
-
-			var templateData = data[i].Template;
-			var postConfig = readConfigTemplate(templateData, POSTPARAM_SECTIONS, undefined, scriptName + ':');
-			for (var j=0; j < postConfig.sections.length; j++)
+			if (data[i].PostScript)
 			{
-				var sec = postConfig.sections[j];
-				if (!sec.hidden)
+				var scriptName = data[i].Name;
+				var sectionId = (scriptName + ':').replace(/ |\/|[\.|$|\:|\*]/g, '_');
+				var option = {};
+				option.name = scriptName + ':';
+				option.caption = shortScriptName(scriptName);
+				option.caption = option.caption.replace(/\\/, ' \\ ').replace(/\//, ' / ');
+
+				option.defvalue = 'no';
+				option.description = (data[i].Template.trim().split('\n')[0].substr(1, 1000).trim() || 'Post-processing script ' + scriptName + '.');
+				option.value = null;
+				option.sectionId = sectionId;
+				option.select = ['yes', 'no'];
+				section.options.push(option);
+
+				var templateData = data[i].Template;
+				var postConfig = readConfigTemplate(templateData, POSTPARAM_SECTIONS, undefined, scriptName + ':');
+				for (var j=0; j < postConfig.sections.length; j++)
 				{
-					for (var n=0; n < sec.options.length; n++)
+					var sec = postConfig.sections[j];
+					if (!sec.hidden)
 					{
-						var option = sec.options[n];
-						option.sectionId = sectionId;
-						section.options.push(option);
+						for (var n=0; n < sec.options.length; n++)
+						{
+							var option = sec.options[n];
+							option.sectionId = sectionId;
+							section.options.push(option);
+						}
 					}
 				}
 			}
@@ -785,7 +792,7 @@ var Config = (new function($)
 			html += '<table class="editor"><tr><td>';
 			html += '<input type="text" id="' + option.formId + '" value="' + Util.textToAttr(value) + '">';
 			html += '</td><td>';
-			html += '<button class="btn" onclick="' + option.editor.click + '($(\'input\', $(this).closest(\'table\')).attr(\'id\'))">' + option.editor.caption + '</button>';
+			html += '<button id="' + option.formId + '_Editor" class="btn" onclick="' + option.editor.click + '($(\'input\', $(this).closest(\'table\')).attr(\'id\'))">' + option.editor.caption + '</button>';
 			html += '</td></tr></table>';
 		}
 		else
@@ -921,6 +928,8 @@ var Config = (new function($)
 				$ConfigNav.append(html);
 			}
 		}
+		
+		notifyChanges();
 
 		$ConfigNav.append('<li class="divider hide ConfigSearch"></li>');
 		$ConfigNav.append('<li class="hide ConfigSearch"><a href="#Search">SEARCH RESULTS</a></li>');
@@ -989,13 +998,12 @@ var Config = (new function($)
 			option.sectionId = firstVisibleSection.id;
 			option.select = [];
 			var description = conf.description;
-			option.description = description !== '' ? description : 'No description available.\n\nNOTE: The script doesn\'t have a description section. '+
-				'It\'s either not NZBGet script or a script created for an older NZBGet version and might not work properly.';
+			option.description = description !== '' ? description : 'No description available.';
 			option.nocontent = true;
 			firstVisibleSection.options.unshift(option);
 		}
 
-		// register editors for options "DefScript", "ScriptOrder" and FeedX.Filter
+		// register editors for certain options
 		var conf = config[0];
 		for (var j=0; j < conf.sections.length; j++)
 		{
@@ -1008,9 +1016,25 @@ var Config = (new function($)
 				{
 					option.editor = { caption: 'Reorder', click: 'Config.editScriptOrder' };
 				}
-				if (optname.indexOf('defscript') > -1)
+				if (optname.indexOf('postscript') > -1)
 				{
-					option.editor = { caption: 'Choose', click: 'Config.editDefScript' };
+					option.editor = { caption: 'Choose', click: 'Config.editPostScript' };
+				}
+				if (optname.indexOf('scanscript') > -1)
+				{
+					option.editor = { caption: 'Choose', click: 'Config.editScanScript' };
+				}
+				if (optname.indexOf('queuescript') > -1)
+				{
+					option.editor = { caption: 'Choose', click: 'Config.editQueueScript' };
+				}
+				if (optname.indexOf('task') > -1 && optname.indexOf('.param') > -1)
+				{
+					option.editor = { caption: 'Choose', click: 'Config.editSchedulerScript' };
+				}
+				if (optname.indexOf('task') > -1 && optname.indexOf('.command') > -1)
+				{
+					option.onchange = Config.schedulerCommandChanged;
 				}
 				if (optname.indexOf('.filter') > -1)
 				{
@@ -1020,6 +1044,26 @@ var Config = (new function($)
 		}
 	}
 
+	function notifyChanges()
+	{
+		for (var k=0; k < config.length; k++)
+		{
+			var sections = config[k].sections;
+			for (var i=0; i < sections.length; i++)
+			{
+				var section = sections[i];
+				for (var j=0; j < section.options.length; j++)
+				{
+					var option = section.options[j];
+					if (option.onchange && !option.template)
+					{
+						option.onchange(option);
+					}
+				}
+			}
+		}
+	}
+	
 	function scrollOptionIntoView(optFormId)
 	{
 		var option = findOptionById(optFormId);
@@ -1043,14 +1087,31 @@ var Config = (new function($)
 		var state = $(control).val().toLowerCase();
 		$('.btn', $(control).parent()).removeClass('btn-primary');
 		$(control).addClass('btn-primary');
+
+		// not for page Postprocess in download details
+		if (config)
+		{
+			var optFormId = $(control).parent().attr('id');
+			var option = findOptionById(optFormId);
+			if (option.onchange)
+			{
+				option.onchange(option);
+			}
+		}
 	}
 
 	function switchGetValue(control)
 	{
-		var state = $('.btn-primary', $(control).parent()).val();
+		var state = $('.btn-primary', control).val();
 		return state;
 	}
 
+	function switchSetValue(control, value)
+	{
+		$('.btn', control).removeClass('btn-primary');
+		$('.btn@[value=' + value + ']', control).addClass('btn-primary');
+	}
+
 	/*** CHANGE/ADD/REMOVE OPTIONS *************************************************************/
 
 	function navClick(event)
@@ -1179,6 +1240,9 @@ var Config = (new function($)
 				option.formId = option.formId.replace(new RegExp(option.multiid), newMultiId);
 				$('#' + oldFormId).attr('id', option.formId);
 
+				// update editor id
+				$('#' + oldFormId + '_Editor').attr('id', option.formId + '_Editor');
+
 				// update name
 				option.name = option.name.replace(new RegExp(option.multiid), newMultiId);
 
@@ -1209,6 +1273,7 @@ var Config = (new function($)
 		multiid++;
 
 		// create new multi set
+		var addedOptions = [];
 		for (var j=0; j < section.options.length; j++)
 		{
 			var option = section.options[j];
@@ -1222,6 +1287,7 @@ var Config = (new function($)
 				newoption.template = false;
 				newoption.multiid = multiid;
 				section.options.push(newoption);
+				addedOptions.push(newoption);
 			}
 		}
 
@@ -1238,6 +1304,15 @@ var Config = (new function($)
 		div.hide();
 		addButton.parent().before(div);
 
+		for (var j=0; j < addedOptions.length; j++)
+		{
+			var option = addedOptions[j];
+			if (option.onchange)
+			{
+				option.onchange(option);
+			}
+		}
+		
 		div.slideDown('normal', function()
 		{
 			var opts = div.children();
@@ -1265,15 +1340,46 @@ var Config = (new function($)
 	this.editScriptOrder = function(optFormId)
 	{
 		var option = findOptionById(optFormId);
-		ScriptListDialog.showModal(option, config);
+		ScriptListDialog.showModal(option, config, null);
 	}
 
-	this.editDefScript = function(optFormId)
+	this.editPostScript = function(optFormId)
 	{
 		var option = findOptionById(optFormId);
-		ScriptListDialog.showModal(option, config);
+		ScriptListDialog.showModal(option, config, 'post');
 	}
 
+	this.editScanScript = function(optFormId)
+	{
+		var option = findOptionById(optFormId);
+		ScriptListDialog.showModal(option, config, 'scan');
+	}
+
+	this.editQueueScript = function(optFormId)
+	{
+		var option = findOptionById(optFormId);
+		ScriptListDialog.showModal(option, config, 'queue');
+	}
+
+	this.editSchedulerScript = function(optFormId)
+	{
+		var option = findOptionById(optFormId);
+		var command = getOptionValue(findOptionById(optFormId.replace(/Param/, 'Command')));
+		if (command !== 'Script')
+		{
+			alert('This button is to choose scheduler scripts when option TaskX.Command is set to "Script".');
+			return;
+		}
+		ScriptListDialog.showModal(option, config, 'scheduler');
+	}
+
+	this.schedulerCommandChanged = function(option)
+	{
+		var command = getOptionValue(option);
+		var btnId = option.formId.replace(/Command/, 'Param_Editor');
+		Util.show('#' + btnId, command === 'Script');
+	}
+	
 	/*** RSS FEEDS ********************************************************************/
 
 	this.editFilter = function(optFormId)
@@ -1321,6 +1427,20 @@ var Config = (new function($)
 	}
 	this.getOptionValue = getOptionValue;
 
+	function setOptionValue(option, value)
+	{
+		var control = $('#' + option.formId);
+		if (option.type === 'switch')
+		{
+			switchSetValue(control, value);
+		}
+		else
+		{
+			control.val(value);
+		}
+	}
+	this.setOptionValue = setOptionValue;
+	
 	// Checks if there are obsolete or invalid options
 	function invalidOptionsExist()
 	{
@@ -1719,7 +1839,9 @@ var ScriptListDialog = (new function($)
 	var $ScriptTable;
 	var option;
 	var config;
+	var kind;
 	var scriptList;
+	var allScripts;
 	var orderChanged;
 	var orderMode;
 
@@ -1751,10 +1873,11 @@ var ScriptListDialog = (new function($)
 		});
 	}
 
-	this.showModal = function(_option, _config)
+	this.showModal = function(_option, _config, _kind)
 	{
 		option = _option;
 		config = _config;
+		kind = _kind;
 		orderChanged = false;
 		orderMode = option.name === 'ScriptOrder';
 
@@ -1775,7 +1898,7 @@ var ScriptListDialog = (new function($)
 		Util.show('#ScriptListDialog_OrderInfo', orderMode, 'inline-block');
 
 		buildScriptList();
-		var selectedList = parseCommaList(Config.getOptionValue(option));
+		var selectedList = Util.parseCommaList(Config.getOptionValue(option));
 		updateTable(selectedList);
 
 		$ScriptListDialog.modal({backdrop: 'static'});
@@ -1807,33 +1930,25 @@ var ScriptListDialog = (new function($)
 		$ScriptTable.fasttable('update', data);
 	}
 
-	function parseCommaList(commaList)
-	{
-		var valueList = commaList.split(/[,;]+/);
-		for (var i=0; i < valueList.length; i++)
-		{
-			valueList[i] = valueList[i].trim();
-			if (valueList[i] === '')
-			{
-				valueList.splice(i, 1);
-				i--;
-			}
-		}
-		return valueList;
-	}
-
 	function buildScriptList()
 	{
-		var orderList = parseCommaList(Config.getOptionValue(Config.findOptionByName('ScriptOrder')));
+		var orderList = Util.parseCommaList(Config.getOptionValue(Config.findOptionByName('ScriptOrder')));
 
 		var availableScripts = [];
+		var availableAllScripts = [];
 		for (var i=1; i < config.length; i++)
 		{
-			availableScripts.push(config[i].scriptName);
+			availableAllScripts.push(config[i].scriptName);
+			if (!kind || config[i][kind])
+			{
+				availableScripts.push(config[i].scriptName);
+			}
 		}
 		availableScripts.sort();
+		availableAllScripts.sort();
 
 		scriptList = [];
+		allScripts = [];
 
 		// first add all scripts from orderList
 		for (var i=0; i < orderList.length; i++)
@@ -1843,18 +1958,32 @@ var ScriptListDialog = (new function($)
 			{
 				scriptList.push(scriptName);
 			}
+			if (availableAllScripts.indexOf(scriptName) > -1)
+			{
+				allScripts.push(scriptName);
+			}
 		}
 
-		// second add all other scripts from script list
+		// add all other scripts of this kind from script list
 		for (var i=0; i < availableScripts.length; i++)
 		{
 			var scriptName = availableScripts[i];
-			if (scriptList.indexOf(scriptName) == -1)
+			if (scriptList.indexOf(scriptName) === -1)
 			{
 				scriptList.push(scriptName);
 			}
 		}
 
+		// add all other scripts of other kinds from script list
+		for (var i=0; i < availableAllScripts.length; i++)
+		{
+			var scriptName = availableAllScripts[i];
+			if (allScripts.indexOf(scriptName) === -1)
+			{
+				allScripts.push(scriptName);
+			}
+		}
+
 		return scriptList;
 	}
 
@@ -1884,7 +2013,26 @@ var ScriptListDialog = (new function($)
 		{
 			var scriptOrderOption = Config.findOptionByName('ScriptOrder');
 			var control = $('#' + scriptOrderOption.formId);
-			control.val(scriptList.join(', '));
+
+			// preserving order of scripts of other kinds which were not visible in the dialog
+			var orderList = [];
+			for (var i=0; i < allScripts.length; i++)
+			{
+				var scriptName = allScripts[i];
+				if (orderList.indexOf(scriptName) === -1)
+				{
+					if (scriptList.indexOf(scriptName) > -1)
+					{
+						orderList = orderList.concat(scriptList);
+					}
+					else
+					{
+						orderList.push(scriptName);
+					}
+				}
+			}
+			
+			control.val(orderList.join(', '));
 		}
 
 		$ScriptListDialog.modal('hide');
@@ -2339,7 +2487,7 @@ var UpdateDialog = (new function($)
 
 		$UpdateDialog.modal({backdrop: 'static'});
 
-		RPC.call('readurl', ['http://nzbget.sourceforge.net/info/nzbget-version.php?nocache=' + new Date().getTime(), 'version info'], loadedUpstreamInfo, error);
+		RPC.call('readurl', ['http://nzbget.net/info/nzbget-version.php?nocache=' + new Date().getTime(), 'version info'], loadedUpstreamInfo, error);
 	}
 	
 	function error(e)
