@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1043 $
- * $Date: 2014-06-13 23:53:27 +0200 (Fri, 13 Jun 2014) $
+ * $Revision: 1108 $
+ * $Date: 2014-08-28 21:15:42 +0200 (Thu, 28 Aug 2014) $
  *
  */
 
@@ -166,6 +166,10 @@ var DownloadsEditDialog = (new function($)
 		var pausedSize = Util.formatSizeMB(group.PausedSizeMB, group.PausedSizeLo);
 		var estimated = group.Status === 'PAUSED' ? '' : (Status.status.DownloadRate > 0 ? Util.formatTimeHMS((group.RemainingSizeMB-group.PausedSizeMB)*1024/(Status.status.DownloadRate/1024)) : '');
 		var completion = group.SuccessArticles + group.FailedArticles > 0 ? Util.round0(group.SuccessArticles * 100.0 / (group.SuccessArticles +  group.FailedArticles)) + '%' : '--';
+		if (group.FailedArticles > 0 && completion === '100%')
+		{
+			completion = '99.9%';
+		}
 
 		var table = '';
 		//table += '<tr><td>Age</td><td class="text-right">' + age + '</td></tr>';
@@ -493,20 +497,7 @@ var DownloadsEditDialog = (new function($)
 		e.preventDefault();
 		disableAllButtons();
 		notification = '#Notif_Downloads_PostCanceled';
-
-		var postDelete = function()
-		{
-			RPC.call('editqueue', ['PostDelete', 0, '', [curGroup.NZBID]], completed);
-		};
-
-		if (curGroup.NZBID > 0)
-		{
-			RPC.call('editqueue', ['GroupDelete', 0, '', [curGroup.NZBID]], postDelete);
-		}
-		else
-		{
-			postDelete();
-		}
+		RPC.call('editqueue', ['PostDelete', 0, '', [curGroup.NZBID]], completed);
 	}
 
 	function categoryChange()
@@ -997,6 +988,11 @@ var EditUI = (new function($)
 					artquota = Util.round0(articles * 100.0 / (editItem.SuccessArticles + editItem.FailedArticles)) + '%';
 					success = Util.round0(stat.SuccessArticles * 100.0 / articles) + '%';
 					failures = Util.round0(stat.FailedArticles * 100.0 / articles) + '%';
+					if (stat.FailedArticles > 0 && failures === '0%')
+					{
+						success = '99.9%';
+						failures = '0.1%';
+					}
 					break;
 				}
 			}
@@ -1020,6 +1016,7 @@ var EditUI = (new function($)
 			cell.className = 'text-right';
 		}
 	}
+
 }(jQuery));
 
 
@@ -1617,19 +1614,30 @@ var HistoryEditDialog = (new function()
 
 			var size = Util.formatSizeMB(hist.FileSizeMB, hist.FileSizeLo);
 			var completion = hist.SuccessArticles + hist.FailedArticles > 0 ? Util.round0(hist.SuccessArticles * 100.0 / (hist.SuccessArticles +  hist.FailedArticles)) + '%' : '--';
-
+			if (hist.FailedArticles > 0 && completion === '100%')
+			{
+				completion = '99.9%';
+			}
+			var time = Util.formatTimeHMS(hist.DownloadTimeSec + hist.PostTotalTimeSec);
+			
 			var table = '';
-			table += '<tr><td>Total</td><td class="text-right">' + size + '</td></tr>';
-			table += '<tr><td>Files (total/parked)</td><td class="text-center">' + hist.FileCount + ' / ' + hist.RemainingFileCount + '</td></tr>';
+			table += '<tr><td><a href="#" id="HistoryEdit_TimeStats" data-tab="HistoryEdit_TimeStatsTab" title="Size and time statistics">Total '+
+				'<i class="icon-forward" style="opacity:0.6;"></i></a>' +
+				'</td><td class="text-center">' + size + '</td></tr>';
+			table += '<tr><td>Files (total/remaining)</td><td class="text-center">' + hist.FileCount + ' / ' + hist.RemainingFileCount + '</td></tr>';
 			table += '<tr><td>' +
 				(hist.ServerStats.length > 0 ? '<a href="#" id="HistoryEdit_ServStats" data-tab="HistoryEdit_ServStatsTab" title="Per-server statistics">' : '') +
 				'Articles (total/completion)' +
 				(hist.ServerStats.length > 0 ? ' <i class="icon-forward" style="opacity:0.6;"></i></a>' : '') +
 				'</td><td class="text-center">' + hist.TotalArticles + ' / ' + completion + '</td></tr>';
 			$('#HistoryEdit_Statistics').html(table);
+
 			$('#HistoryEdit_ServStats').click(tabClick);
 			EditUI.fillServStats($ServStatsTable, hist);
 			$ServStatsTable.fasttable('setCurPage', 1);
+
+			$('#HistoryEdit_TimeStats').click(tabClick);
+			fillTimeStats();
 		}
 
 		$('#HistoryEdit_DupeKey').val(hist.DupeKey);
@@ -1671,6 +1679,7 @@ var HistoryEditDialog = (new function()
 		$('#HistoryEdit_GeneralTab').show();
 		$('#HistoryEdit_ParamTab').hide();
 		$('#HistoryEdit_ServStatsTab').hide();
+		$('#HistoryEdit_TimeStatsTab').hide();
 		$('#HistoryEdit_DupeTab').hide();
 		$('#HistoryEdit_Back').hide();
 		$('#HistoryEdit_BackSpace').show();
@@ -1704,6 +1713,8 @@ var HistoryEditDialog = (new function()
 				return '<span class="label label-status">' + prefix + 'manual</span>';
 			case 'DELETED-HEALTH':
 				return '<span class="label label-status label-important">' + prefix + 'health</span>';
+			case 'DELETED-BAD':
+				return '<span class="label label-status label-important">' + prefix + 'bad</span>';
 			case 'SCAN_SKIPPED':
 				return '<span class="label label-status label-warning"">' + prefix + 'skipped</span>';
 			case 'NONE':
@@ -1713,6 +1724,23 @@ var HistoryEditDialog = (new function()
 		}
 	}
 	
+	function fillTimeStats()
+	{
+		var hist = curHist;
+		var downloaded = Util.formatSizeMB(hist.DownloadedSizeMB, hist.DownloadedSizeLo);
+		var speed = hist.DownloadTimeSec > 0 ? Util.formatSpeed((hist.DownloadedSizeMB > 1024 ? hist.DownloadedSizeMB * 1024.0 * 1024.0 : hist.DownloadedSizeLo) / hist.DownloadTimeSec) : '--';
+		var table = '';
+		table += '<tr><td>Downloaded size</td><td class="text-center">' + downloaded + '</td></tr>';
+		table += '<tr><td>Download speed</td><td class="text-center">' + speed + '</td></tr>';
+		table += '<tr><td>Total time</td><td class="text-center">' + Util.formatTimeHMS(hist.DownloadTimeSec + hist.PostTotalTimeSec) + '</td></tr>';
+		table += '<tr><td>Download time</td><td class="text-center">' + Util.formatTimeHMS(hist.DownloadTimeSec) + '</td></tr>';
+		table += '<tr><td>Verification time </td><td class="text-center">' + Util.formatTimeHMS(hist.ParTimeSec - hist.RepairTimeSec) + '</td></tr>';
+		table += '<tr><td>Repair time</td><td class="text-center">' + Util.formatTimeHMS(hist.RepairTimeSec) + '</td></tr>';
+		table += '<tr><td>Unpack time</td><td class="text-center">' + Util.formatTimeHMS(hist.UnpackTimeSec) + '</td></tr>';
+
+		$('#HistoryEdit_TimeStatsTable tbody').html(table);
+	}
+	
 	function tabClick(e)
 	{
 		e.preventDefault();
