@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 992 $
- * $Date: 2014-04-15 00:06:23 +0200 (Tue, 15 Apr 2014) $
+ * $Revision: 1142 $
+ * $Date: 2014-10-12 16:23:54 +0200 (Sun, 12 Oct 2014) $
  *
  */
 
@@ -160,7 +160,13 @@ var History = (new function($)
 			var category = '';
 
 			var textname = hist.Name;
-			if (kind === 'URL')
+			var age = '';
+			if (kind === 'NZB')
+			{
+				age = Util.formatAge(hist.MinPostTime + UISettings.timeZoneCorrection*60*60);
+				textname += ' URL';
+			}
+			else if (kind === 'URL')
 			{
 				textname += ' URL';
 			}
@@ -178,8 +184,8 @@ var History = (new function($)
 			{
 				id: hist.ID,
 				hist: hist,
-				data: {time: time, size: size},
-				search: statustext + ' ' + time + ' ' + textname + ' ' + dupe + ' ' + category + ' ' + size
+				data: { time: time, age: age, size: size },
+				search: statustext + ' ' + time + ' ' + textname + ' ' + dupe + ' ' + category + ' ' +  age + ' ' + size
 			};
 
 			data.push(item);
@@ -217,7 +223,7 @@ var History = (new function($)
 
 		if (!UISettings.miniTheme)
 		{
-			item.fields = ['<div class="check img-check"></div>', status, item.data.time, name + dupe, category, item.data.size];
+			item.fields = ['<div class="check img-check"></div>', status, item.data.time, name + dupe, category, item.data.age, item.data.size];
 		}
 		else
 		{
@@ -241,7 +247,7 @@ var History = (new function($)
 		{
 			cell.className = 'text-center';
 		}
-		else if (index === 5)
+		else if (index === 5 || index === 6)
 		{
 			cell.className = 'text-right';
 		}
