@@ -17,8 +17,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Revision: 1035 $
- * $Date: 2014-06-02 22:28:36 +0200 (Mon, 02 Jun 2014) $
+ * $Revision: 1057 $
+ * $Date: 2014-07-19 00:48:35 +0200 (Sat, 19 Jul 2014) $
  *
  */
 
@@ -374,6 +374,7 @@ var StatDialog = (new function($)
 	var $StatDialog_DataAverageSpeed;
 	var $StatDialog_DataCurrentSpeed;
 	var $StatDialog_DataSpeedLimit;
+	var $StatDialog_ArticleCache;
 	var $StatDialog_ChartBlock;
 	var $StatDialog_Server;
 	var $StatRangeDialog;
@@ -420,6 +421,7 @@ var StatDialog = (new function($)
 		$StatDialog_DataAverageSpeed = $('#StatDialog_DataAverageSpeed');
 		$StatDialog_DataCurrentSpeed = $('#StatDialog_DataCurrentSpeed');
 		$StatDialog_DataSpeedLimit = $('#StatDialog_DataSpeedLimit');
+		$StatDialog_ArticleCache = $('#StatDialog_ArticleCache');		
 		$StatDialog_ChartBlock = $('#StatDialog_ChartBlock');
 		$StatDialog_Server = $('#StatDialog_Server');
 		$StatRangeDialog = $('#StatRangeDialog');
@@ -495,6 +497,7 @@ var StatDialog = (new function($)
 		$('#StatDialog_Back').hide();
 		$('#StatDialog_BackSpace').show();
 		$('#StatDialog_Title').text('Statistics and Status');
+		Util.show('#StatDialog_ArticleCache_Row', Options.option('ArticleCache') !== '0');
 		$StatDialog.removeClass('modal-large').addClass('modal-mini');
 		monthListInitialized = false;
 		updateServerList();
@@ -542,6 +545,7 @@ var StatDialog = (new function($)
 		$StatDialog_DataAverageSpeed.html(Util.formatSpeed(status.AverageDownloadRate));
 		$StatDialog_DataCurrentSpeed.html(Util.formatSpeed(status.DownloadRate));
 		$StatDialog_DataSpeedLimit.html(Util.formatSpeed(status.DownloadLimit));
+		$StatDialog_ArticleCache.html(Util.formatSizeMB(status.ArticleCacheMB, status.ArticleCacheLo));
 
 		var content = '';
 		content += '<tr><td>Download</td><td class="text-right">' +
